@@ -278,38 +278,6 @@ if [ $mtest = 0 ]; then
     get_config
 fi
 
-#------------------------------------------------------------
-#
-#  Kernel module
-#
-#------------------------------------------------------------
-
-KCHECK="no"
-
-if [ "x$I386_LINUX" = "xyes" ]; then
-
-$dialog \
---title "Kernel module rootkit detection" \
---backtitle "Samhain $VERSION" \
---inputbox "SAMHAIN can detect kernel module rootkits if compiled with support\nfor this. If you want to enable this option, please give the path\nto your System.map file, else choose CANCEL.\n\nNOTE: this option will require root privileges for at least one\ncommand during compilation (to read from /dev/kmem)." \
-16 75 "/boot/System.map" \
-2> $tmp
-
-
-mtest=$?
-
-if [ $mtest = -1 ]; then
-    exit 1
-fi
-if [ $mtest = 0 ]; then
-    mfile=`cat $tmp`
-    for z in $mfile ; do
-	cfgline="$cfgline --with-kcheck=$z"
-    done
-    KCHECK="yes"
-fi
-
-fi
 
 #------------------------------------------------------------
 #
