@@ -30,6 +30,16 @@
 
 /*@-fixedformalarray@*/
 
+#include "config_xor.h"
+#if defined(__GNUC__) && (__GNUC__ >= 3)
+#undef  SH_GNUC_NORETURN
+#define SH_GNUC_NORETURN __attribute__((noreturn))
+#else
+#undef  SH_GNUC_NORETURN
+#define SH_GNUC_NORETURN
+#endif
+
+
 /* Set aud functions
  */
 int sh_aud_set_functions(const char * str_s);
@@ -82,9 +92,9 @@ long int aud_open_noatime   (const char * file, int line, int privs,
 			     const char *pathname, int flags, mode_t mode,
 			     int * o_noatime);
 /*@noreturn@*/
-void     aud_exit   (const char * file, int line, int fd);
+void     aud_exit   (const char * file, int line, int fd) SH_GNUC_NORETURN;
 /*@noreturn@*/
-void     aud__exit  (const char * file, int line, int fd);
+void     aud__exit  (const char * file, int line, int fd) SH_GNUC_NORETURN;
 pid_t    aud_fork   (const char * file, int line);
 int      aud_pipe   (const char * file, int line, int modus[2]);
 int      aud_setuid (const char * file, int line, uid_t uid);
