@@ -74,12 +74,14 @@ enum {
  MSG_RECONF,      
  MSG_CHECK_0,     
  MSG_CHECK_1,     
+ MSG_CHECK_2,     
  MSG_STAMP,       
  		  
  MSG_D_START,     
  MSG_D_DSTART,    
  MSG_D_FAIL,      
-
+ MSG_D_DELTAOK,
+ MSG_D_DELTAFAIL,
 
 #ifndef HAVE_URANDOM 
  MSG_ENSTART,     
@@ -99,18 +101,6 @@ enum {
  MSG_SUID_ERROR,
 #endif
 
-#ifdef SH_USE_KERN
- /* FreeBSD */
- MSG_KERN_POLICY,    
- MSG_KERN_POL_CO,
-
- /* Linux */
- MSG_KERN_SYSCALL,
- MSG_KERN_PROC,
- MSG_KERN_IDT,
- MSG_KERN_GATE,
-#endif
-
 #ifdef SH_USE_UTMP
  MSG_UT_CHECK,
 
