@@ -1,6 +1,8 @@
 #ifndef SH_EXTERN_H
 #define SH_EXTERN_H
 
+#include <stdarg.h>
+
 typedef struct 
 {
   char   *  command;
@@ -35,10 +37,21 @@ typedef struct
  */
 int sh_ext_popen (sh_tas_t * task);
 
+/*
+ * -- generic simple safe popen; returns 0 on success, -1 otherwise,
+ *    executes shell command
+ */
+int sh_ext_popen_init (sh_tas_t * task, const char * command, char * argv0, ...) SH_GNUC_SENTINEL;
+
 /*
  * -- Execute command, return first line of output
  */
-char * sh_ext_popen_str (char * command);
+int sh_ext_system (char * command, char * argv0, ...) SH_GNUC_SENTINEL;
+
+/*
+ * -- Execute command, return first line of output
+ */
+char * sh_ext_popen_str (const char * command);
 
 /*
  * -- close the pipe, clear and return task->exit_status
