@@ -25,41 +25,40 @@
 
 /* set a relay server
  */
-int sh_mail_set_relay (char * str_s);
+int sh_mail_set_relay (const char * str_s);
 
 /* send to all recpts. in one mail
  */
-int sh_mail_setFlag (char * str);
+int sh_mail_setFlag (const char * str);
 
 /* set the subject string 
  */
-int set_mail_subject (char * str);
+int set_mail_subject (const char * str);
 
 /* test mailbox
  */
-int sh_mail_sigverify (char * s);
+int sh_mail_sigverify (const char * s);
 
 /* maximum number of mail attempts
  */
 #define SH_MAX_FAIL    48
 
-int sh_mail_setNum (char * str);
+int sh_mail_setNum (const char * str);
 
-int sh_mail_setaddress (char * address);
+int sh_mail_setaddress (const char * address);
 void reset_count_dev_mail(void);
-int sh_mail_setaddress_int (char * address);
+int sh_mail_setaddress_int (const char * address);
 
 /* call if not urgent
  */
-int sh_mail_pushstack (/*@null@*/char * msg);
+int sh_mail_pushstack (int severity, const char * msg, const char * alias);
 
-/* call if mail immediate
+/* Set the port to use (default 25)
  */
-int sh_mail_msg (/*@null@*/char * message);
+int sh_mail_set_port (const char * str);
 
 /* set sender of mail
  */
-int sh_mail_set_sender (char *str);
-
+int sh_mail_set_sender (const char *str);
 
 #endif
