@@ -7,7 +7,12 @@
 #define SH_MAXMSGLEN 64
 
 #if defined (SH_WITH_CLIENT)
-void sh_socket_server_cmd(const char * srvcmd);
+char * sh_socket_get_uuid(int * errflag, unsigned int * count, time_t * last);
+int    sh_socket_store_uuid(const char * cmd);
+int    sh_socket_return_uuid(const char * uuid, unsigned int count, time_t last);
+void   sh_socket_server_cmd(const char * srvcmd);
+int    set_delta_retry_interval(const char * str);
+int    set_delta_retry_count(const char * str);
 #endif
 
 #if defined (SH_WITH_SERVER)
