@@ -43,6 +43,15 @@ UINT32 * sh_tiger_hash_uint32 (char * filename,
  */
 int sh_tiger_get_hashtype (void);
 
+/* set the hash fuction in use in the mask
+ */
+void sh_tiger_get_mask_hashtype(unsigned long * mask);
+
+
+/* reset the hash function to the one in the mask
+ */
+void sh_tiger_set_hashtype_mask(unsigned long mask);
+
 /* GnuPG-like format, returns allocated memory
  */
 /*@owned@*/ char * sh_tiger_hash_gpg (const char * filename, TigerType what, 
