@@ -63,11 +63,6 @@
 #define uint unsigned int
 #define ulong unsigned long
 
-/*
-extern void *malloc(size_t size);
-extern void free();
-*/
-
 char *last_string    = NULL;
 char *big_end_string = NULL;
 
@@ -231,7 +226,7 @@ allocate_digits(ulong alloclen)
 #ifdef COUNT_ALLOC
     dgs_alloc += alloclen;
 #endif
-    digit_ptr = (DIGIT *) malloc(alloclen * sizeof(DIGIT));
+    digit_ptr = calloc(alloclen, sizeof(DIGIT));
     if (digit_ptr == NULL)
     {
 	big_errno = BIG_MEMERR;
@@ -624,7 +619,7 @@ big_init_pkg()
     big_create(&big_one);
     big_set_long((long)1, &big_one);
     length_last_string = 10;
-    if ((last_string = malloc(length_last_string)) == NULL)
+    if ((last_string = calloc(1,length_last_string)) == NULL)
     {
 	big_errno = BIG_MEMERR;
     }
@@ -954,7 +949,7 @@ int base;
     {
 	if (last_string != NULL)
 	  free(last_string);
-	if ((last_string = malloc(str_length)) == NULL)
+	if ((last_string = calloc(1,str_length)) == NULL)
 	{
 	    big_errno = BIG_MEMERR;
 	    return NULL;
