@@ -70,6 +70,7 @@
 #include "sh_getopt.h"
 #include "sh_readconf.h"
 #include "sh_hash.h"
+#include "sh_dbIO.h"
 #include "sh_restrict.h"
 
 #include "sh_nmail.h"
@@ -77,7 +78,7 @@
 #include "sh_tiger.h"
 #include "sh_gpg.h"
 #include "sh_mem.h"
-#include "sh_forward.h"
+#include "sh_xfer.h"
 #include "sh_tools.h"
 #include "sh_hash.h"
 #if defined(WITH_EXTERNAL)
@@ -87,6 +88,7 @@
 #include "sh_modules.h"
 #include "sh_ignore.h"
 #include "sh_prelink.h"
+#include "sh_sem.h"
 #endif
 
 #undef  FIL__
@@ -119,6 +121,9 @@ volatile  int      sig_config_read_again;  /* SIGHUP  */
 volatile  int      sig_terminate;          /* SIGQUIT */
 volatile  int      sig_termfast;           /* SIGTERM */
 volatile  int      sig_force_check;        /* SIGTTOU */
+volatile  int      sig_force_silent;       /* SIGTSTP */
+volatile  int      sh_global_check_silent;
+volatile  int      sh_load_delta_flag;
 long int           eintr__result;
 char               sh_sig_msg[SH_MINIBUF];
 
@@ -149,13 +154,16 @@ pthread_key_t g_key;
 
 int sh_g_thread()
 {
-  struct gt * ptr = malloc(sizeof(struct gt));
+  struct gt * ptr = calloc(1,sizeof(struct gt));
   if (!ptr)
     return -1;
   ptr->g_count    = 0;
   ptr->g_glob     = calloc(1, SH_MAX_GLOBS * (GLOB_LEN+1));
   if (!(ptr->g_glob))
-    return -1;
+    {
+      free(ptr);
+      return -1;
+    }
   return pthread_setspecific(g_key, ptr);
 }
 
@@ -309,6 +317,9 @@ void sh_init (void)
 #ifdef MKA_12
   ErrFlag[0] |= (1 << 11);
 #endif
+
+  sh.delayload = 0;
+
 #ifdef MKA_13
   ErrFlag[0] |= (1 << 12);
 #endif
@@ -333,6 +344,9 @@ void sh_init (void)
   sig_terminate          = 0;           /* SIGQUIT */
   sig_termfast           = 0;           /* SIGTERM */
   sig_force_check        = 0;           /* SIGTTOU */
+  sig_force_silent       = 0;           /* SIGTSTP */
+  sh_global_check_silent = 0;
+  sh_load_delta_flag     = 0;
   strcpy ( sh_sig_msg, _("None"));
 
 #ifdef MKB_01
@@ -370,6 +384,8 @@ void sh_init (void)
 
   sh.pid = (UINT64) getpid();
 
+  sh.outpath = NULL;
+
   /* The flags.
    */
   if (is_samhainctl_init == S_FALSE)
@@ -639,11 +655,11 @@ void sh_init (void)
 
   /* The struct to hold privileged information.
    */
-  skey = (sh_key_t *) malloc (sizeof(sh_key_t));
+  skey = calloc(1,sizeof(sh_key_t));
   if (skey != NULL)
     {
 
-      skey->mlock_failed = SL_FALSE;
+      skey->mlock_failed = S_FALSE;
       skey->rngI         = BAD;
       /* properly initialized later 
        */
@@ -714,8 +730,6 @@ static void exit_handler(void)
   extern int sh_html_zero();
 #endif
 
-  SL_ENTER(_("exit_handler"));
-
 #if defined(SH_WITH_SERVER)
   sh_socket_remove ();
 #endif
@@ -747,7 +761,6 @@ static void exit_handler(void)
    * of client becomes unknown in the beltane interface
    */
   sh_html_zero();
-  /* sh_forward_html_write(); */
 #endif
 
   /* --- Clean up memory to check for problems. ---
@@ -771,7 +784,7 @@ static void exit_handler(void)
     }
 #endif
 #if defined(SH_WITH_SERVER)
-  sh_forward_free_all ();
+  sh_xfer_free_all ();
 #endif
 #if defined(SH_WITH_MAIL)
   sh_nmail_free();
@@ -787,11 +800,21 @@ static void exit_handler(void)
 
   /* --- Checksum of executable. ---
    */
-  (void) sh_unix_self_check();
+  {
+    volatile  int sig_store = sig_termfast;
+
+    sig_termfast = 0;
+    (void) sh_unix_self_check();
+    sig_termfast = sig_store;
+  }
 
 
   /* --- Exit Message. ---
    */
+  if (sh.flag.exit == EXIT_FAILURE && !strcmp(sh_sig_msg, _("None")))
+    sl_strlcpy(sh_sig_msg, _("exit_failure"), SH_MINIBUF);
+  if (sh.flag.exit == EXIT_SUCCESS && !strcmp(sh_sig_msg, _("None")))
+    sl_strlcpy(sh_sig_msg, _("exit_success"), SH_MINIBUF);
   sh_error_handle ((-1), FIL__, __LINE__, sh.flag.exit, MSG_EXIT_NORMAL, 
 		   sh.prg_name, sh_sig_msg);
 #ifdef SH_USE_XML
@@ -801,12 +824,9 @@ static void exit_handler(void)
 
   /* --- Restrict error logging to stderr. ---
    */
-#ifdef WITH_MESSAGE_QUEUE
   close_ipc ();
-#endif
   sh_error_only_stderr (S_TRUE);
 
-
   /* --- Remove lock, delete critical information. ---
    */
   (void) sh_unix_rm_lock_file (sh.srvlog.name);
@@ -861,7 +881,7 @@ static pid_t * procdirSamhain (void)
 
   SH_MUTEX_LOCK(mutex_readdir);
 
-  pidlist =  malloc(sizeof(pid_t) * 65535);
+  pidlist =  calloc(1, sizeof(pid_t) * 65535);
   if (!pidlist)
     goto unlock_and_out;
 
@@ -1178,7 +1198,7 @@ static sh_schedule_t * sh_set_schedule_int (const char * str,
       (void) free_sched(FileSchedIn);
       FileSchedIn = NULL;
       *status = 0;
-      return NULL;
+      SL_RETURN(NULL, _("sh_set_schedule_int"));
     }
 
   FileSched = SH_ALLOC(sizeof(sh_schedule_t));
@@ -1189,7 +1209,9 @@ static sh_schedule_t * sh_set_schedule_int (const char * str,
       FileSched = NULL;
       SL_RETURN(FileSchedIn , _("sh_set_schedule_int"));
     }
-  FileSched->next = FileSchedIn;
+  else {
+    FileSched->next = FileSchedIn;
+  }
   SL_RETURN(FileSched , _("sh_set_schedule_int"));
 }
 
@@ -1211,6 +1233,287 @@ int sh_set_schedule_two (const char * str)
   return status;
 }
 
+static int sh_flag_silent = S_FALSE;
+
+int sh_set_silent_full (const char * str)
+{
+  int status = sh_util_flagval(str, &sh_flag_silent);
+  return status;
+}
+
+
+void do_reconf()
+{
+  int status, modnum;
+
+  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_RECONF);
+  
+  sh_thread_pause_flag = S_TRUE;
+
+  for (modnum = 0; modList[modnum].name != NULL; ++modnum) 
+    {
+      if (0 != (SH_MODFL_NEEDPAUSED & modList[modnum].flags) &&
+	  modList[modnum].initval == SH_MOD_THREAD)
+	{
+	  int count = 50;
+	  while (count && 0 == (SH_MODFL_ISPAUSED & modList[modnum].flags))
+	    retry_msleep(0, 100), --count;
+	}
+    }
+  
+#if defined(WITH_EXTERNAL)
+  /* delete list of external tasks
+   */
+  (void) sh_ext_cleanup();
+#endif
+#if defined(SH_WITH_MAIL)
+  sh_nmail_free();
+#endif
+  
+  /* delete the file list, make all database
+   * entries visible (allignore = FALSE)
+   */
+  (void) sh_files_deldirstack ();
+  (void) sh_files_delfilestack ();
+  (void) sh_files_delglobstack ();
+  (void) sh_ignore_clean ();
+  (void) hash_full_tree ();
+  sh_audit_delete_all ();
+  
+  
+#if defined(SH_WITH_CLIENT)
+  reset_count_dev_server();
+#endif
+#if defined(SH_WITH_CLIENT) || defined(SH_STANDALONE)
+  sh_restrict_purge ();
+  
+  
+  FileSchedOne = free_sched(FileSchedOne);
+  FileSchedTwo = free_sched(FileSchedTwo);
+  
+  for (modnum = 0; modList[modnum].name != NULL; ++modnum) 
+    {
+      /* sh_thread_pause_flag is true, and we block in lock
+       * until check has returned, so we are sure check will
+       * not run until sh_thread_pause_flag is set to false
+       */
+      /* if (modList[modnum].initval >= SH_MOD_ACTIVE) */
+      (void) modList[modnum].mod_reconf();
+    }
+#endif
+  
+  reset_count_dev_console();
+  reset_count_dev_time();
+  
+  (void) sh_unix_maskreset();
+  
+#ifdef RELOAD_DATABASE
+  sh_hash_hashdelete();
+
+  if (0 != sl_strcmp(file_path('D', 'R'), _("REQ_FROM_SERVER")))
+    {
+      char hashbuf[KEYBUF_SIZE];
+      (void) sl_strlcpy(sh.data.hash,
+			sh_tiger_hash (file_path('D', 'R'), 
+				       TIGER_FILE, TIGER_NOLIM, 
+				       hashbuf, sizeof(hashbuf)), 
+			KEY_LEN+1);
+    }
+#endif
+  (void) sl_trust_purge_user();
+  (void) sh_files_hle_reg (NULL);
+  (void) sh_prelink_run (NULL, NULL, 0, 0);
+  
+  /* --------------------------
+   * --- READ CONFIGURATION ---
+   * --------------------------
+   */
+  (void) sh_readconf_read ();
+  sig_config_read_again = 0;
+  (void) sh_files_setrec();
+  (void) sh_files_test_setup();
+  sh_audit_commit ();
+  
+  if (0 != sh.flag.nice)
+    {
+#ifdef HAVE_SETPRIORITY
+      setpriority(PRIO_PROCESS, 0, sh.flag.nice);
+#else
+      nice(sh.flag.nice);
+#endif
+    }
+  
+  if (sh.flag.checkSum == SH_CHECK_INIT)
+    {
+      sh.flag.isdaemon = S_FALSE;
+      sh.flag.loop     = S_FALSE;
+    }
+  
+  
+  /* --- Initialize modules. ---
+   */
+  TPT((0, FIL__, __LINE__, _("msg=<Initialize modules.>\n")));
+  for (modnum = 0; modList[modnum].name != NULL; ++modnum) 
+    {
+      status = modList[modnum].mod_init(&(modList[modnum]));
+      
+      if (status < 0)
+	{
+	  if (status == (-1)) {
+	    sh_error_handle (SH_ERR_NOTICE, FIL__, __LINE__, 
+			     status, MSG_MOD_FAIL,
+			     _(modList[modnum].name),
+			     status+SH_MOD_OFFSET);
+	  } else {
+	    sh_error_handle ((-1), FIL__, __LINE__, 
+			     status, MSG_MOD_FAIL,
+			     _(modList[modnum].name),
+			     status+SH_MOD_OFFSET);
+	  }
+	  modList[modnum].initval = SH_MOD_FAILED;
+	}
+      else
+	{
+	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_MOD_OK,
+			   _(modList[modnum].name));
+	  modList[modnum].initval = status;
+	}
+    }
+  
+  /* module is properly set up now
+   */
+  sh_thread_pause_flag = S_FALSE;
+
+  return;
+}
+
+static void check_signals (volatile int *flag_check_1, 
+			   volatile int * flag_check_2)
+{
+  if (sig_raised > 0) 
+    {
+      
+      TPT((0, FIL__, __LINE__, _("msg=<Process a signal.>\n")))
+	
+	if (sig_termfast == 1)  /* SIGTERM */
+	  {
+	    TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
+	    --sig_raised; --sig_urgent;
+	    aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
+	  }
+      
+      if (sig_force_check == 1) /* SIGTTOU / SIGTSTP */
+	{
+	  TPT((0, FIL__, __LINE__, _("msg=<Check run triggered.>\n")));
+	  *flag_check_1 = 1; *flag_check_2 = 1;
+	  sig_force_check = 0; --sig_raised;
+	  if (sig_force_silent)
+	    {
+	      sig_force_silent       = 0;
+	      sh_global_check_silent = (sh_flag_silent == S_FALSE) ? SH_SILENT_STD : SH_SILENT_FULL;
+	    }
+	  sh_sem_trylock();
+	}
+      
+      if (sig_config_read_again == 1 && /* SIGHUP */
+	  sh_global_suspend_flag == 0)
+	{
+	  TPT((0, FIL__, __LINE__, _("msg=<Re-read configuration.>\n")));
+	  do_reconf();
+	  --sig_raised;
+	}
+      
+      if (sig_fresh_trail == 1) /* SIGIOT */
+	{
+	  if (sh_global_suspend_flag == 0)
+	    {
+	      SH_MUTEX_LOCK(mutex_thread_nolog);
+	      
+	      /* Logfile access 
+	       */
+#ifdef SH_USE_XML
+	      (void) sh_log_file (NULL, NULL);
+#endif
+	      TPT((0, FIL__, __LINE__, _("msg=<Logfile stop/restart.>\n")));
+	      sh_error_only_stderr (S_TRUE);
+	      (void) sh_unix_rm_lock_file(sh.srvlog.name);
+	      (void) retry_msleep(3, 0);
+	      sh.flag.log_start = S_TRUE;
+	      sh_error_only_stderr (S_FALSE);
+	      sh_thread_pause_flag = S_FALSE;
+	      sig_fresh_trail       = 0;
+	      --sig_raised;
+	      SH_MUTEX_UNLOCK(mutex_thread_nolog);
+	    }
+	}
+      
+      if (sig_terminate == 1)  /* SIGQUIT */
+	{
+	  TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
+	  strncpy (sh_sig_msg, _("Quit"), 20);
+	  --sig_raised;
+	  aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
+	}
+	  
+      if (sig_debug_switch == 1)  /* SIGUSR1 */
+	{
+	  TPT((0, FIL__, __LINE__, _("msg=<Debug switch.>\n")));
+	  sh_error_dbg_switch();
+	  sig_debug_switch = 0;
+	  --sig_raised;
+	}
+	  
+      if (sig_suspend_switch > 0)  /* SIGUSR2 */
+	{
+	  TPT((0, FIL__, __LINE__, _("msg=<Suspend switch.>\n")));
+	  if (sh_global_suspend_flag != 1) {
+	    SH_MUTEX_LOCK_UNSAFE(mutex_thread_nolog);
+	    sh_global_suspend_flag = 1;
+	    sh_error_handle((-1), FIL__, __LINE__, 0, MSG_SUSPEND, 
+			    sh.prg_name);
+	  } else {
+	    sh_global_suspend_flag = 0;
+	    SH_MUTEX_UNLOCK_UNSAFE(mutex_thread_nolog);
+	  }
+	  --sig_suspend_switch;
+	  --sig_raised; --sig_urgent;
+	}
+
+      if (sh_load_delta_flag > 0 &&
+	  sh_global_suspend_flag == 0)  /* DELTA Command */
+	{
+	  if (0 == sh_dbIO_load_delta())
+	    {
+	      --sh_load_delta_flag; --sig_raised;
+	    } 
+	}
+
+      sig_raised = (sig_raised < 0) ? 0 : sig_raised;
+      sig_urgent = (sig_urgent < 0) ? 0 : sig_urgent;
+      TPT((0, FIL__, __LINE__, _("msg=<End signal processing.>\n")));
+    }
+  return;
+}
+
+void check_for_delta_db()
+{
+  /* Need to contact the server.
+   */
+  if (sh_global_check_silent < SH_SILENT_FULL)
+    sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_CHECK_2);
+
+  if (sig_raised > 0 && sh_load_delta_flag > 0)  /* DELTA Command */
+    {
+      /* There was a DELTA Command
+       */
+      if (0 == sh_dbIO_load_delta())
+	{
+	  --sh_load_delta_flag; --sig_raised;
+	} 
+    }
+  return;
+}
+
 #endif
 
 /*******************************************************
@@ -1225,7 +1528,7 @@ int undef_main(int argc, char * argv[])
 #endif
 {
 #if defined(INET_SYSLOG)
-  extern int    create_syslog_socket (int flag);
+  extern int    sh_xfer_create_syslog_socket (int flag);
 #endif
 #if defined(SH_WITH_SERVER)
   extern int    sh_create_tcp_socket(void);
@@ -1268,7 +1571,7 @@ int undef_main(int argc, char * argv[])
 
   /* --- Close all but first three file descriptors. ---
    */
-  sh_unix_closeall(3, -1, SL_FALSE); /* at program start */
+  sh_unix_closeall(3, -1, S_FALSE); /* at program start */
 
 
   if (argc >= 2 && 0 != getuid() &&
@@ -1375,7 +1678,7 @@ int undef_main(int argc, char * argv[])
       tzlen       = strlen(tzptr);
       if (tzlen < 1024)
 	{
-	  sh.timezone = malloc (tzlen + 1);
+	  sh.timezone = calloc(1, tzlen + 1);
 	  if (sh.timezone != NULL)
 	    (void) sl_strlcpy (sh.timezone, tzptr, tzlen + 1);
 	}
@@ -1531,24 +1834,9 @@ int undef_main(int argc, char * argv[])
 
   /* --- load database; checksum of database
    */
-#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE) 
-  TPT((0, FIL__, __LINE__, _("msg=<Get checksum of the database.>\n")))
-  if (sh.flag.checkSum == SH_CHECK_CHECK) 
-    {
-      if (0 != sl_strcmp(file_path('D', 'R'), _("REQ_FROM_SERVER")))
-	{
-	  char hashbuf[KEYBUF_SIZE];
-	  (void) sl_strlcpy(sh.data.hash,
-			    sh_tiger_hash (file_path('D', 'R'), 
-					   TIGER_FILE, TIGER_NOLIM, 
-					   hashbuf, sizeof(hashbuf)), 
-			    KEY_LEN+1);
-	}
-
-      /* this eventually fetches the file from server to get checksum
-       */
-      sh_hash_init ();
-    }
+#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
+  if (0 == sh.delayload)
+    sh_hash_init_and_checksum();
 #endif
 
   /* --- initialize signal handling etc.; fork daemon
@@ -1560,12 +1848,20 @@ int undef_main(int argc, char * argv[])
       aud_exit(FIL__, __LINE__, EXIT_FAILURE);
     }
 
+#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
+  if (sh.delayload)
+    {
+      sleep(sh.delayload);
+      sh_hash_init_and_checksum();
+    }
+#endif
+
   /* --- drop privileges eventually ---
    */
 #if defined(SH_WITH_SERVER)
   sh_create_tcp_socket ();
 #if defined(INET_SYSLOG)
-  create_syslog_socket (S_TRUE);
+  sh_xfer_create_syslog_socket (S_TRUE);
 #endif
   SL_REQUIRE(sl_policy_get_real(DEFAULT_IDENT) == SL_ENONE, 
 	     _("sl_policy_get_real(DEFAULT_IDENT) == SL_ENONE"));
@@ -1587,7 +1883,7 @@ int undef_main(int argc, char * argv[])
   /* --- now check whether we really wanted it; if not, close ---
    */
 #if defined(INET_SYSLOG) && defined(SH_WITH_SERVER)
-  create_syslog_socket (S_FALSE);
+  sh_xfer_create_syslog_socket (S_FALSE);
 #endif
 
 
@@ -1652,7 +1948,7 @@ int undef_main(int argc, char * argv[])
 #endif
 
  
-  if ((skey == NULL) || (skey->mlock_failed == SL_TRUE))
+  if ((skey == NULL) || (skey->mlock_failed == S_TRUE))
     sh_error_handle ((-1), FIL__, __LINE__, EPERM, MSG_MLOCK);
 
   /* timer
@@ -1674,12 +1970,12 @@ int undef_main(int argc, char * argv[])
 #if defined (SH_WITH_CLIENT)
   if (sh.flag.isserver == S_TRUE)
     { 
-      sh_receive();
+      sh_xfer_start_server();
       TPT((0, FIL__, __LINE__, _("msg=<End server.>\n")))
       aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
     }
 #else
-  sh_receive();
+  sh_xfer_start_server();
   TPT((0, FIL__, __LINE__, _("msg=<End server.>\n")))
   aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
 #endif
@@ -1741,6 +2037,10 @@ int undef_main(int argc, char * argv[])
 #endif
     }
 
+  /*  --------  CREATE SEMAPHORE  ---------
+   */
+  sh_sem_open();
+
   /*  --------  MAIN LOOP  ---------
    */
   sh.statistics.bytes_speed   = 0;
@@ -1759,230 +2059,11 @@ int undef_main(int argc, char * argv[])
 
       tcurrent = time (NULL);
 
-      if (sig_raised > 0) 
-	{
-
-	  TPT((0, FIL__, __LINE__, _("msg=<Process a signal.>\n")))
-
-	  if (sig_termfast == 1)  /* SIGTERM */
-	    {
-	      TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
-	      /* strncpy (sh_sig_msg, _("SIGTERM"), 20); */
-	      --sig_raised; --sig_urgent;
-	      aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
-	    }
-
-	  if (sig_force_check == 1) /* SIGTTOU */
-	    {
-	      TPT((0, FIL__, __LINE__, _("msg=<Check run triggered.>\n")));
-	      flag_check_1 = 1;
-	      flag_check_2 = 1;
-	      sig_force_check = 0;
-	      --sig_raised; 
-	    }
-	  
-	  if (sig_config_read_again == 1 && /* SIGHUP */
-	      sh_global_suspend_flag == 0)
-	    {
-	      TPT((0, FIL__, __LINE__, _("msg=<Re-read configuration.>\n")))
-	      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_RECONF);
-
-	      sh_thread_pause_flag = S_TRUE;
-
-#if defined(WITH_EXTERNAL)
-	      /* delete list of external tasks
-	       */
-	      (void) sh_ext_cleanup();
-#endif
-#if defined(SH_WITH_MAIL)
-	      sh_nmail_free();
-#endif
-
-	      /* delete the file list, make all database
-	       * entries visible (allignore = FALSE)
-	       */
-	      (void) sh_files_deldirstack ();
-	      (void) sh_files_delfilestack ();
-	      (void) sh_files_delglobstack ();
-	      (void) sh_ignore_clean ();
-	      (void) hash_full_tree ();
-	      sh_audit_delete_all ();
-
-
-#if defined(SH_WITH_CLIENT)
-	      reset_count_dev_server();
-#endif
-#if defined(SH_WITH_CLIENT) || defined(SH_STANDALONE)
-	      sh_restrict_purge ();
-
-
-	      FileSchedOne = free_sched(FileSchedOne);
-	      FileSchedTwo = free_sched(FileSchedTwo);
-
-	      for (modnum = 0; modList[modnum].name != NULL; ++modnum) 
-		{
-		  /* sh_thread_pause_flag is true, and we block in lock
-		   * until check has returned, so we are sure check will
-		   * not run until sh_thread_pause_flag is set to false
-		   */
-		  /* if (modList[modnum].initval >= SH_MOD_ACTIVE) */
-		  (void) modList[modnum].mod_reconf();
-		}
-#endif
-
-	      reset_count_dev_console();
-	      reset_count_dev_time();
-
-	      (void) sh_unix_maskreset();
- 
-	      /* Should this be included ??? 
-	       * (i.e. should we reload the database ?)
-	       */
-#ifdef RELOAD_DATABASE
-	      sh_hash_hashdelete();
-
-	      if (0 != sl_strcmp(file_path('D', 'R'), _("REQ_FROM_SERVER")))
-		{
-		  char hashbuf[KEYBUF_SIZE];
-		  (void) sl_strlcpy(sh.data.hash,
-				    sh_tiger_hash (file_path('D', 'R'), 
-						   TIGER_FILE, TIGER_NOLIM, 
-						   hashbuf, sizeof(hashbuf)), 
-				    KEY_LEN+1);
-		}
-#endif
-	      (void) sl_trust_purge_user();
-	      (void) sh_files_hle_reg (NULL);
-	      (void) sh_prelink_run (NULL, NULL, 0);
-
-	      /* --------------------------
-	       * --- READ CONFIGURATION ---
-	       * --------------------------
-	       */
-	      (void) sh_readconf_read ();
-	      sig_config_read_again = 0;
-	      (void) sh_files_setrec();
-	      (void) sh_files_test_setup();
-	      sh_audit_commit ();
-
-	      if (0 != sh.flag.nice)
-		{
-#ifdef HAVE_SETPRIORITY
-		  setpriority(PRIO_PROCESS, 0, sh.flag.nice);
-#else
-		  nice(sh.flag.nice);
-#endif
-		}
-
-	      if (sh.flag.checkSum == SH_CHECK_INIT)
-		{
-		  sh.flag.isdaemon = S_FALSE;
-		  sh.flag.loop     = S_FALSE;
-		}
-
-
-	      /* --- Initialize modules. ---
-	       */
-	      TPT((0, FIL__, __LINE__, _("msg=<Initialize modules.>\n")));
-	      for (modnum = 0; modList[modnum].name != NULL; ++modnum) 
-		{
-		  status = modList[modnum].mod_init(&(modList[modnum]));
-
-		  if (status < 0)
-		    {
-		      if (status == (-1)) {
-			sh_error_handle (SH_ERR_NOTICE, FIL__, __LINE__, 
-					 status, MSG_MOD_FAIL,
-					 _(modList[modnum].name),
-					 status+SH_MOD_OFFSET);
-		      } else {
-			sh_error_handle ((-1), FIL__, __LINE__, 
-					 status, MSG_MOD_FAIL,
-					 _(modList[modnum].name),
-					 status+SH_MOD_OFFSET);
-		      }
-		      modList[modnum].initval = SH_MOD_FAILED;
-		    }
-		  else
-		    {
-		      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_MOD_OK,
-				       _(modList[modnum].name));
-		      modList[modnum].initval = status;
-		    }
-		}
-
-	      /* module is properly set up now
-	       */
-	      sh_thread_pause_flag = S_FALSE;
-	      
-	      --sig_raised;
-	    }
-	  
-	  if (sig_fresh_trail == 1) /* SIGIOT */
-	    {
-	      if (sh_global_suspend_flag == 0)
-		{
-		  SH_MUTEX_LOCK(mutex_thread_nolog);
-
-		  /* Logfile access 
-		   */
-#ifdef SH_USE_XML
-		  (void) sh_log_file (NULL, NULL);
-#endif
-		  TPT((0, FIL__, __LINE__, _("msg=<Logfile stop/restart.>\n")));
-		  sh_error_only_stderr (S_TRUE);
-		  (void) sh_unix_rm_lock_file(sh.srvlog.name);
-		  (void) retry_msleep(3, 0);
-		  sh.flag.log_start = S_TRUE;
-		  sh_error_only_stderr (S_FALSE);
-		  sh_thread_pause_flag = S_FALSE;
-		  sig_fresh_trail       = 0;
-		  --sig_raised;
-		  SH_MUTEX_UNLOCK(mutex_thread_nolog);
-		}
-	    }
-	  
-	  if (sig_terminate == 1)  /* SIGQUIT */
-	    {
-	      TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
-	      strncpy (sh_sig_msg, _("Quit"), 20);
-	      --sig_raised; --sig_urgent;
-	      aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
-	    }
-	  
-	  if (sig_debug_switch == 1)  /* SIGUSR1 */
-	    {
-	      TPT((0, FIL__, __LINE__, _("msg=<Debug switch.>\n")));
-	      sh_error_dbg_switch();
-	      sig_debug_switch = 0;
-	      --sig_raised;
-	    }
-	  
-	  if (sig_suspend_switch > 0)  /* SIGUSR2 */
-	    {
-	      TPT((0, FIL__, __LINE__, _("msg=<Suspend switch.>\n")));
-	      if (sh_global_suspend_flag != 1) {
-		SH_MUTEX_LOCK_UNSAFE(mutex_thread_nolog);
-		sh_global_suspend_flag = 1;
-		sh_error_handle((-1), FIL__, __LINE__, 0, MSG_SUSPEND, 
-				sh.prg_name);
-	      } else {
-		sh_global_suspend_flag = 0;
-		SH_MUTEX_UNLOCK_UNSAFE(mutex_thread_nolog);
-	      }
-	      --sig_suspend_switch;
-	      --sig_raised; --sig_urgent;
-	    }
-	  sig_raised = (sig_raised < 0) ? 0 : sig_raised;
-	  sig_urgent = (sig_urgent < 0) ? 0 : sig_urgent;
-	  TPT((0, FIL__, __LINE__, _("msg=<End signal processing.>\n")));
-	}
-      
-      if (sh_global_suspend_flag == 1)
-	{
+      do {
+	check_signals(&flag_check_1, &flag_check_2);
+	if (sh_global_suspend_flag == 1)
 	  (void) retry_msleep (1, 0);
-	  continue;
-	}
+      } while (sh_global_suspend_flag == 1);
       
       /* see whether its time to check files
        */
@@ -2022,6 +2103,13 @@ int undef_main(int argc, char * argv[])
       if (sh.flag.checkSum != SH_CHECK_NONE &&
 	  (flag_check_1 == 1 || flag_check_2 == 1))
 	{
+	  sh_sem_trylock();
+
+	  /* Starting a check now, so make sure to fetch delta DB
+	   * if there is one to download.
+	   */
+	  check_for_delta_db();
+
 	  SH_INOTIFY_IFUSED( sh.flag.inotify |= SH_INOTIFY_INSCAN; );
 	  /* Refresh list files matching glob patterns.
 	   */
@@ -2061,9 +2149,7 @@ int undef_main(int argc, char * argv[])
 	  if (flag_check_1 == 1)
 	    (void) sh_files_chk ();
 
-	  if (sig_urgent > 0)
-	    continue;
-
+	  if (sig_urgent > 0) { sh_sem_unlock(sh.statistics.files_report); sh_global_check_silent = 0; continue; }
 	  /*
 	   * check for files not visited
 	   */
@@ -2073,33 +2159,35 @@ int undef_main(int argc, char * argv[])
 	      sh_hash_unvisited (ShDFLevel[SH_ERR_T_FILE]);
 	    }
 
-	  if (sig_urgent > 0)
-	    continue;
+	  if (sig_urgent > 0) { sh_sem_unlock(sh.statistics.files_report); sh_global_check_silent = 0; continue; }
 
 	  /* reset
 	   */
 	  TPT((0, FIL__, __LINE__, _("msg=<Reset status.>\n")))
 	  sh_dirs_reset  ();
-	  if (sig_urgent > 0)
-	    continue;
+
+	  if (sig_urgent > 0) { sh_sem_unlock(sh.statistics.files_report); sh_global_check_silent = 0; continue; }
 
 	  sh_files_reset ();
+	  check_done   = 1;
+
 	  flag_check_1 = 0;
 	  flag_check_2 = 0;
-	  check_done   = 1;
+	  sh_sem_unlock(sh.statistics.files_report);
+
 	  SH_INOTIFY_IFUSED( sh.flag.inotify &= ~SH_INOTIFY_INSCAN; );
 	  SH_INOTIFY_IFUSED( sh.flag.inotify &= ~SH_INOTIFY_DOSCAN; );
 
-	  (void) sh_prelink_run (NULL, NULL, 0);
+	  (void) sh_prelink_run (NULL, NULL, 0, 0);
 
-	  if (sig_urgent > 0)
-	    continue;
+	  if (sig_urgent > 0) { sh_global_check_silent = 0; continue; }
 
 	  runtim = time(NULL) - sh.statistics.time_start;
 	  sh.statistics.time_check = runtim;
 	
 	  if ((sh.statistics.dirs_checked == 0) && 
-	      (sh.statistics.files_checked == 0))
+	      (sh.statistics.files_checked == 0) &&
+	      (sh_global_check_silent < SH_SILENT_FULL))
 	    sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_CHECK_0);
 
 	  else
@@ -2117,17 +2205,21 @@ int undef_main(int argc, char * argv[])
 
 	      sh.statistics.bytes_speed = (unsigned long) st_1;
 
-	      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_CHECK_1,
-			       (long) runtim, 
-			       0.001 * st_1);
+	      if (sh_global_check_silent < SH_SILENT_FULL)
+		sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_CHECK_1,
+				 (long) runtim, 
+				 0.001 * st_1);
 
 	      if (sh.flag.checkSum != SH_CHECK_INIT)
 		sh_efile_report();
 	    }
-	  sh.fileCheck.alarm_last = time (NULL);
+	  
+	  if (0 == sh_global_check_silent)
+	    sh.fileCheck.alarm_last = time (NULL);
 
-	  if (sig_urgent > 0)
-	    continue;
+	  sh_global_check_silent = 0;
+	  
+	  if (sig_urgent > 0) continue;
 
 	  /*
 	   * flush mail queue
@@ -2138,8 +2230,7 @@ int undef_main(int argc, char * argv[])
 #endif
 	}
       
-      if (sig_urgent > 0)
-	continue;
+      if (sig_urgent > 0) continue;
       
       /* execute modules
        */
@@ -2157,14 +2248,14 @@ int undef_main(int argc, char * argv[])
        * 22.10.2002 moved here b/o suid check initialization
        */ 
       if      (sh.flag.checkSum == SH_CHECK_INIT)
-	sh_hash_pushdata (NULL, NULL);
+	sh_dbIO_data_write (NULL, NULL);
 
       /* write out database
        */
       if (sh.flag.checkSum == SH_CHECK_CHECK && 
 	  sh.flag.update == S_TRUE && 
 	  check_done == 1)
-	sh_hash_writeout ();
+	sh_dbIO_writeout_update ();
 
       /* no-op unless MEM_LOG is defined in sh_mem.c
        */
@@ -2192,10 +2283,8 @@ int undef_main(int argc, char * argv[])
 
       /* no loop if not daemon
        */
-      if (sh.flag.isdaemon != S_TRUE && sh.flag.loop == S_FALSE)
-	break; 
-      if (sig_urgent > 0)
-	continue;
+      if (sh.flag.isdaemon != S_TRUE && sh.flag.loop == S_FALSE) break;
+      if (sig_urgent > 0) continue;
 
       /* see whether its time to send mail
        */
@@ -2207,8 +2296,7 @@ int undef_main(int argc, char * argv[])
 	  sh.mailTime.alarm_last = time (NULL);
 	}
 #endif
-      if (sig_urgent > 0)
-	continue;
+      if (sig_urgent > 0) continue;
             
       /* log the timestamp
        */
@@ -2228,8 +2316,7 @@ int undef_main(int argc, char * argv[])
        */
       (void) taus_seed();
       
-      if (sig_urgent > 0)
-	continue;
+      if (sig_urgent > 0) continue;
       
       /* reset cache
        */
@@ -2262,15 +2349,6 @@ int undef_main(int argc, char * argv[])
   /* #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE) */
 #endif
 
-#if 0
-  {
-    char command[128];
-    sprintf(command, "/bin/cat /proc/%d/status", (int) getpid());
-    system(command); /* flawfinder: ignore *//* debug code */
-    malloc_stats();
-  }
-#endif
-
   aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
   SL_RETURN(0, _("main"));
 }
