@@ -152,13 +152,8 @@ int sh_calls_set_bind_addr (const char * str)
   if (sh.flag.opts == S_TRUE)  
     reject = 1;
 
-#if defined(USE_IPVX)
   if (0 == sh_ipvx_aton(str, &bind_addr)) 
     return -1;
-#else
-  if (0 == inet_aton(str, &(bind_addr.sin.sin_addr))) 
-    return -1;
-#endif
 
   use_bind_addr = 1;
   return 0;
@@ -169,6 +164,7 @@ long int retry_connect(const char * file, int line, int sockfd,
 		       struct sockaddr *serv_addr, int addrlen)
 {
   int error;
+  int err_bind = 0;
   long int val_retry = 0;
   char errbuf[SH_ERRBUF_SIZE];
 
@@ -176,11 +172,18 @@ long int retry_connect(const char * file, int line, int sockfd,
 
   errno = 0;
 
-  if (0 != use_bind_addr) 
+  if (use_bind_addr) 
     {
       int slen = SH_SS_LEN(bind_addr);
+      struct sockaddr *b_addr = sh_ipvx_sockaddr_cast(&bind_addr);
 
-      val_retry = bind(sockfd, sh_ipvx_sockaddr_cast(&bind_addr), slen);
+      if (bind_addr.ss_family == AF_INET)
+	b_addr->sa_family = AF_INET;
+#if defined(USE_IPVX)
+      else
+	b_addr->sa_family = AF_INET6;
+#endif
+      val_retry = bind(sockfd, b_addr, slen);
     }
 
   if (val_retry == 0)
@@ -189,23 +192,43 @@ long int retry_connect(const char * file, int line, int sockfd,
 	val_retry = connect(sockfd, serv_addr, addrlen);
       } while (val_retry < 0 && (errno == EINTR || errno == EINPROGRESS));
     }
+  else
+    {
+      err_bind = 1;
+    }
 
   error = errno;
   if (val_retry != 0) {
-    long eport;
+    long eport = 0;
     char eaddr[SH_IP_BUF];
-
+    char emesg[SH_BUFSIZE];
+    struct sockaddr *err_addr = serv_addr;
     struct sh_sockaddr ss;
-    sh_ipvx_save(&ss, serv_addr->sa_family, serv_addr);
+
+    if (err_bind)
+      err_addr = sh_ipvx_sockaddr_cast(&bind_addr);
+
+    sh_ipvx_save(&ss, err_addr->sa_family, err_addr);
     sh_ipvx_ntoa(eaddr, sizeof(eaddr), &ss);
     
-    if (serv_addr->sa_family == AF_INET)
+    if (err_addr->sa_family == AF_INET)
       eport = (long) ntohs(((struct sockaddr_in *)serv_addr)->sin_port);
+#if defined(USE_IPVX)
     else
       eport = (long) ntohs(((struct sockaddr_in6 *)serv_addr)->sin6_port);
+#endif
 
-    sh_error_handle ((-1), file, line, error, MSG_ERR_CONNECT, 
-		     sh_error_message(error, errbuf, sizeof(errbuf)),
+    sl_strlcpy(emesg, sh_error_message(error, errbuf, sizeof(errbuf)), sizeof(emesg));
+    sl_strlcat(emesg, 
+	       (err_addr->sa_family == AF_INET) ? _(", AF_INET") : _(", AF_INET6"),
+	       sizeof(emesg));
+
+    sl_strlcat(emesg, 
+	       (err_bind) ? _(", bind") : _(", connect"),
+	       sizeof(emesg));
+    
+    sh_error_handle ((-1), file, line, error, MSG_ERR_CONNECT,
+		     emesg,
 		     (long) sockfd, eport, eaddr);
   }
   errno = error;    
@@ -238,7 +261,7 @@ long int retry_accept(const char * file, int line, int fd,
   }
   errno = error;
 
-  if (flag_err_debug == SL_TRUE)
+  if (flag_err_debug == S_TRUE)
     {
       char ipbuf[SH_IP_BUF];
       char buf[SH_BUFSIZE];
@@ -267,7 +290,7 @@ long int retry_accept(const char * file, int line, int fd,
 
   sh_ipvx_save(serv_addr, ss.ss_family, (struct sockaddr *) &ss);
 
-  if (flag_err_debug == SL_TRUE)
+  if (flag_err_debug == S_TRUE)
     {
       char ipbuf[SH_IP_BUF];
       char ipbuf2[SH_IP_BUF];
@@ -296,7 +319,11 @@ long int retry_accept(const char * file, int line, int fd,
 static int sh_enable_use_sub = 0;
 
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
+#if defined(HOST_IS_CYGWIN) || defined(__cygwin__) || defined(__CYGWIN32__) || defined(__CYGWIN__)
+static int sh_use_sub = 0;
+#else
 static int sh_use_sub = 1;
+#endif
 #else
 static int sh_use_sub = 0;
 #endif
@@ -466,8 +493,6 @@ long int retry_msleep (int sec, int millisec)
   struct timespec req, rem;
 #endif
 
-  SL_ENTER(_("retry_msleep"));
-
   errno  = 0;
   if (millisec > 999) millisec = 999;
   if (millisec < 0)   millisec = 0;
@@ -487,25 +512,18 @@ long int retry_msleep (int sec, int millisec)
   } while ((result == -1) && (errno == EINTR));
 #else
   if (sec > 0)
-    {
-      sleep (sec); /* nanosleep not available */
-    }
-  else
-    {
+    sleep (sec); /* nanosleep not available */
+  else {
 #ifdef HAVE_USLEEP
-      if (millisec > 0)
-	{
-	  usleep(1000 * millisec);
-	}
+    if (millisec > 0)
+      usleep(1000 * millisec);
 #else
-      if (millisec > 0)
-	{
-	  sleep (1);
-	}
+    if (millisec > 0)
+      sleep (1);
 #endif
-    }
+  }
 #endif
-  SL_RETURN(result, _("retry_msleep"));
+  return result;
 }
 
 /***************************************************
@@ -811,29 +829,29 @@ long int aud_kill (const char * file, int line, pid_t pid, int sig)
 }
   
 /*@noreturn@*/
-void aud_exit (const char * file, int line, int fd)
+void aud_exit (const char * file, int line, int exitval)
 {
   if (sh.flag.audit != 0 && (sh.flag.aud_mask & AUD_EXIT) != 0)
     sh_error_handle ((-1), file, line, 0, MSG_AUD_EXIT,
-		      (long) fd);
+		      (long) exitval);
 
   SL_ENTER(_("aud_exit"));
 
-  sh.flag.exit = fd;
-  exit(fd);
+  sh.flag.exit = exitval;
+  exit(exitval);
 }
 
 /*@noreturn@*/
-void aud__exit (const char * file, int line, int fd)
+void aud__exit (const char * file, int line, int exitval)
 {
   if (sh.flag.audit != 0 && (sh.flag.aud_mask & AUD_EXIT) != 0)
     sh_error_handle ((-1), file, line, 0, MSG_AUD_EXIT,
-		      (long) fd);
+		      (long) exitval);
 
   SL_ENTER(_("aud__exit"));
 
-  sh.flag.exit = fd;
-  _exit(fd);
+  sh.flag.exit = exitval;
+  _exit(exitval);
 }
 
 pid_t aud_fork (const char * file, int line)
