@@ -474,7 +474,7 @@ void SHA256_Final(sha2_byte digest[], SHA256_CTX* context)
   }
   
   /* Clean up state data: */
-  memset(context, 0, sizeof(context));
+  memset(context, 0, sizeof(SHA256_CTX));
   usedspace = 0;
 }
 
@@ -489,7 +489,7 @@ char *SHA256_End(SHA256_CTX* context, char buffer[])
     SHA256_Final(digest, context);
     sh_util_base64_enc ((unsigned char *)buffer, digest,  SHA256_DIGEST_LENGTH);
   } else {
-    memset(context, 0, sizeof(context));
+    memset(context, 0, sizeof(SHA256_CTX));
   }
   memset(digest, 0, SHA256_DIGEST_LENGTH);
   return buffer;
