@@ -179,7 +179,8 @@ static my_attr attr_tab[] = {
   { NULL, N_("chksum_new"),  0,  48,   50, 0, offsetof(struct dbins_, chksum_new)},
   { NULL, N_("link_old"),    0,  49, MAX_PATH_STORE+1, 0, offsetof(struct dbins_, link_old)},
   { NULL, N_("link_new"),    0,  50, MAX_PATH_STORE+1, 0, offsetof(struct dbins_, link_new)},
-               				    
+
+  /* These go into  dbins.ulong_data[n - START_SEC_LONGS] */             				    
   { NULL, N_("size_old"),     0,  51,    0, 0, 0  },
   { NULL, N_("size_new"),     0,  52,    0, 0, 0  },
   { NULL, N_("hardlinks_old"),0,  53,    0, 0, 0  },
@@ -200,21 +201,30 @@ static my_attr attr_tab[] = {
 
   { NULL, N_("port"),         0,  67,    0, 0, 0  },
   { NULL, N_("return_code"),  0,  68,    0, 0, 0  },
-  /* { NULL, N_("userid"),        0,  69,    0, 0  }, old 'userid', 1.8.1 */
 
-  { NULL, N_("host"),         0,  70,   64, 0, offsetof(struct dbins_, fromhost)},
-  { NULL, N_("attr_old"),     0,  71,   16, 0, offsetof(struct dbins_, attr_old)},
-  { NULL, N_("attr_new"),     0,  72,   16, 0, offsetof(struct dbins_, attr_new)},
-  { NULL, N_("acl_old"),      0,  73, 1024, 0, offsetof(struct dbins_, acl_old)},
-  { NULL, N_("acl_new"),      0,  74, 1024, 0, offsetof(struct dbins_, acl_new)},
+  { NULL, N_("checkflags_old"), 0,  69,    0, 0, 0  },
+  { NULL, N_("checkflags_new"), 0,  70,    0, 0, 0  },
+
+  /* END_SEC_LONGS                                         */
+
+  { NULL, N_("host"),         0,  71,   64, 0, offsetof(struct dbins_, fromhost)},
+  { NULL, N_("attr_old"),     0,  72,   16, 0, offsetof(struct dbins_, attr_old)},
+  { NULL, N_("attr_new"),     0,  73,   16, 0, offsetof(struct dbins_, attr_new)},
+  { NULL, N_("acl_old"),      0,  74, 1024, 0, offsetof(struct dbins_, acl_old)},
+  { NULL, N_("acl_new"),      0,  75, 1024, 0, offsetof(struct dbins_, acl_new)},
 
   { NULL, NULL,      0,  0, 0, 0, 0 }
 };
 
-#define SH_SLOT_HOST    70
+#define SH_SLOT_CHECKFLAGS 69
+
+/* need special attention b/o reserved SQL words    */
+#define SH_SLOT_HOST    71
 #define SH_SLOT_GROUP    7
+
+/* these go into dbins.ulong_data[n-START_SEC_LONGS */
 #define START_SEC_LONGS 51
-#define END_SEC_LONGS   68
+#define END_SEC_LONGS   70
 
 #if defined(HAVE_INT_32)
 typedef unsigned int uint32;
@@ -1162,7 +1172,7 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
       goto err_out;
     }
 
-  if (flag_err_debug == SL_TRUE)
+  if (flag_err_debug == S_TRUE)
     {
       p = mysql_info (db_conn);
       if (p != NULL)
@@ -1359,6 +1369,12 @@ long sh_database_entry (dbins * db_entry, long id)
 
   while (attr_tab[i].attr != NULL)
     {
+      if (SH_SLOT_CHECKFLAGS == attr_tab[i].val) {
+	if (db_entry->ulong_data[attr_tab[i].val-START_SEC_LONGS] == 0 &&
+	    db_entry->ulong_data[attr_tab[i+1].val-START_SEC_LONGS] == 0)
+	  { i+= 2; continue; } 
+      }
+
       if (attr_tab[i].size != 0)
 	{
 	  if (attr_tab[i].val > 40 && attr_tab[i].val < 47)
@@ -1483,7 +1499,7 @@ static void init_attr_table(void)
   while (attr_tab[i].attr_o != NULL)
     {
       j = strlen(attr_tab[i].attr_o);
-      attr_tab[i].attr = malloc (j+1); /* only once */
+      attr_tab[i].attr = calloc(1, j+1); /* only once */
       if (NULL == attr_tab[i].attr)
 	return;
       for (k = 0; k < j; ++k)
@@ -1727,11 +1743,10 @@ char *  sh_database_parse (char * message, dbins * db_entry)
 				       &p[j+2], 
 				       (size_t) res->size);
 		  }
-		else if (res->val >= START_SEC_LONGS)
+		else if (res->val >= START_SEC_LONGS && res->val <= END_SEC_LONGS)
 		  {
 		    db_entry->ulong_data[res->val-START_SEC_LONGS]
-		      = strtoul(&p[j+2], (char **) NULL, 10); 
-		    /* atol(&p[j+2]); */
+		      = strtoul(&p[j+2], (char **) NULL, 0);
 		  }
 
 		*q = '"';
