@@ -299,7 +299,7 @@ int read_mbytes(int timeout_val, const char * path, char * nbuf, int nbytes)
       if (0 == sh_unix_device_readable(fd2)) 
 	{
 	  m_count = sl_read_timeout_fd(fd2, nbuf, nbytes, 
-				       timeout_val, SL_FALSE);
+				       timeout_val, S_FALSE);
 	  if (m_count < 0)
 	    m_count = 0;
 	}
@@ -336,29 +336,29 @@ int sh_entropy(int nbytes, char * nbuf)
   memset(nbuf, '\0', nbytes);
 
 #ifdef NAME_OF_DEV_URANDOM
-  m_count = read_mbytes (  1, NAME_OF_DEV_RANDOM, nbuf, nbytes);
+  m_count = read_mbytes ( 10, NAME_OF_DEV_URANDOM, nbuf, nbytes);
 #else
-  m_count = read_mbytes (300, NAME_OF_DEV_RANDOM, nbuf, nbytes);
+  m_count = read_mbytes (300, NAME_OF_DEV_RANDOM,  nbuf, nbytes);
 #endif
 
   if (m_count == 0)
     {
 #ifdef NAME_OF_DEV_URANDOM
       sh_error_handle (SH_ERR_NOTICE, FIL__, __LINE__, EIO, MSG_NODEV, 
-		       (long) sh.real.uid, NAME_OF_DEV_RANDOM);
+		       (long) sh.real.uid, NAME_OF_DEV_URANDOM);
 #else
       sh_error_handle ((-1), FIL__, __LINE__, EIO, MSG_NODEV, 
 		       (long) sh.real.uid, NAME_OF_DEV_RANDOM);
 #endif
     }
 
-#ifdef NAME_OF_DEV_URANDOM
+#ifdef NAME_OF_DEV_RANDOM
   if (m_count < nbytes)
     {
-      i = read_mbytes(30, NAME_OF_DEV_URANDOM, &nbuf[m_count], nbytes-m_count);
+      i = read_mbytes(300, NAME_OF_DEV_RANDOM, &nbuf[m_count], nbytes-m_count);
       if (i == 0)
 	sh_error_handle ((-1), FIL__, __LINE__, EIO, MSG_NODEV, 
-			 (long) sh.real.uid, NAME_OF_DEV_URANDOM);
+			 (long) sh.real.uid, NAME_OF_DEV_RANDOM);
       else
 	m_count += i;
     }
@@ -547,7 +547,7 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
   if (sh.timezone != NULL)
     {
       len = sl_strlen(sh.timezone) + 4;
-      envp[0] = malloc (len);     /* free() ok     */
+      envp[0] = calloc(1, len);     /* free() ok     */
       if (envp[0] != NULL)
 	sl_snprintf (envp[0], len, "TZ=%s", sh.timezone);
       else
@@ -600,7 +600,7 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
 
       /* don't leak file descriptors
        */
-      sh_unix_closeall (3, -1, SL_TRUE); /* in child process */
+      sh_unix_closeall (3, -1, S_TRUE); /* in child process */
 
       /* zero priv info
        */
@@ -613,9 +613,14 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
 	{
   
 	  if (NULL != tempres) {
-	    i = setgid(tempres->pw_gid); 
-	    if (i == 0)
-	      i = sh_unix_initgroups(DEFAULT_IDENT ,tempres->pw_gid);
+	    i = setgid(tempres->pw_gid);
+
+	    /*** locks up in dnmalloc ***/
+	    /*
+	     * if (i == 0)
+	     *   i = sh_unix_initgroups(DEFAULT_IDENT ,tempres->pw_gid);
+	     */
+
 	    if (i == 0) 
 	      i = setuid(tempres->pw_uid);
 	    /* make sure we cannot get root again
@@ -634,6 +639,7 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
 	_exit(EXIT_FAILURE);
       }
       
+      /* cppcheck-suppress leakNoVarFunctionCall */
       if (NULL != freopen (_("/dev/null"), "r+", stderr))
 	{
       
@@ -935,6 +941,30 @@ int sh_entropy(int nbytes, char * nbuf)
 		      _("sl_drop_cap_sub"));
     }
 
+#ifdef HAVE_GETTIMEOFDAY
+  {
+    unsigned short tseed[3];
+
+    gettimeofday(&tv, 0);
+    tseed[0] = tv.tv_sec & 0xFFFF;
+    tseed[1] = tv.tv_usec & 0xFFFF;
+    tseed[2] = (tv.tv_sec ^ tv.tv_usec) >> 16;
+    keybuf = (char *) sh_tiger_hash_uint32 ((char *) tseed, 
+					    TIGER_DATA, sizeof(tseed),
+					    kbuf, KEY_BYT/sizeof(UINT32));
+    memset(addbuf, '\0', sizeof(addbuf));
+    for (i = 0; i < KEY_BYT; ++i)
+      {
+	addbuf[i]         = keybuf[i];
+	addbuf[i+KEY_BYT] = skey->poolv[i];
+      }
+    SH_MUTEX_LOCK_UNSAFE(mutex_skey);
+    for (i = 0; i < KEY_BYT; ++i)
+      skey->poolv[i] ^= keybuf[i];
+    SH_MUTEX_UNLOCK_UNSAFE(mutex_skey);
+  }
+#endif
+
   if (bufcount > 0) 
     {
       keybuf = (char *) sh_tiger_hash_uint32 (buffer, 
@@ -995,18 +1025,24 @@ void Test_entropy (CuTest *tc)
   char                 bufx[9 * sizeof(UINT32) + 1];
   char                 bufy[9 * sizeof(UINT32) + 1];
   int                  status;
+  int                  count;
 
-  memset(skey->poolv, '\0', KEY_BYT);
+  for (count = 0; count < 20; ++count)
+    {
+      memset(skey->poolv, '\0', KEY_BYT);
+      skey->poolc = 0;
 
-  status = sh_entropy (24, bufx);
-  CuAssertTrue(tc, 0 == status);
+      status = sh_entropy (24, bufx);
+      CuAssertTrue(tc, 0 == status);
+      
+      memset(skey->poolv, '\0', KEY_BYT);
+      skey->poolc = 0;
 
-  memset(skey->poolv, '\0', KEY_BYT);
-
-  status = sh_entropy (24, bufy);
-  CuAssertTrue(tc, 0 == status);
-
-  CuAssertTrue(tc, 0 != memcmp(bufx, bufy, 24));
+      status = sh_entropy (24, bufy);
+      CuAssertTrue(tc, 0 == status);
+      
+      CuAssertTrue(tc, 0 != memcmp(bufx, bufy, 24));
+    }
 }
 #endif
 
