@@ -597,7 +597,8 @@ int sh_log_open (char * inet_peer,
                          _("logfile"), tmp);
         (*service_failure) = 1;
       }
-      SH_FREE(tmp);
+      if (tmp)
+	SH_FREE(tmp);
       SL_RETURN(-1, _("sh_log_open"));
     }
 
@@ -768,21 +769,21 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
   log_msg.msg = (char *) SH_ALLOC ((size_t) (2*KEY_LEN + status + 32)); 
 
 #if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
-  if (skey->mlock_failed == SL_FALSE) 
+  if (skey->mlock_failed == S_FALSE) 
     {
       if ( (-1) == sh_unix_mlock( FIL__, __LINE__, log_msg.msg, 
 				  (size_t)(2*KEY_LEN + status + 32) ) ) 
 	{
-	  skey->mlock_failed = SL_TRUE;
+	  skey->mlock_failed = S_TRUE;
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
 	  sh_error_handle ((-1), FIL__, __LINE__, EPERM, MSG_MLOCK); 
 #endif
 	}
     }
 #else
-  if (skey->mlock_failed == SL_FALSE) 
+  if (skey->mlock_failed == S_FALSE) 
     {
-      skey->mlock_failed = SL_TRUE;
+      skey->mlock_failed = S_TRUE;
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
       sh_error_handle ((-1), FIL__, __LINE__, EPERM, MSG_MLOCK);
 #endif
@@ -1205,17 +1206,17 @@ static void sh_efile_load(size_t * tmp)
 {
   int i;
 
-  if (SL_TRUE == sl_ok_adds (gSave[0], sh.statistics.bytes_hashed))
+  if (S_TRUE == sl_ok_adds (gSave[0], sh.statistics.bytes_hashed))
     gSave[0] += sh.statistics.bytes_hashed;
-  if (SL_TRUE == sl_ok_adds (gSave[1], sh.statistics.dirs_checked))
+  if (S_TRUE == sl_ok_adds (gSave[1], sh.statistics.dirs_checked))
     gSave[1] += sh.statistics.dirs_checked;
-  if (SL_TRUE == sl_ok_adds (gSave[2], sh.statistics.files_checked))
+  if (S_TRUE == sl_ok_adds (gSave[2], sh.statistics.files_checked))
     gSave[2] += sh.statistics.files_checked;
-  if (SL_TRUE == sl_ok_adds (gSave[3], sh.statistics.files_report))
+  if (S_TRUE == sl_ok_adds (gSave[3], sh.statistics.files_report))
     gSave[3] += sh.statistics.files_report;
-  if (SL_TRUE == sl_ok_adds (gSave[4], sh.statistics.files_error))
+  if (S_TRUE == sl_ok_adds (gSave[4], sh.statistics.files_error))
     gSave[4] += sh.statistics.files_error;
-  if (SL_TRUE == sl_ok_adds (gSave[5], sh.statistics.files_nodir))
+  if (S_TRUE == sl_ok_adds (gSave[5], sh.statistics.files_nodir))
     gSave[5] += sh.statistics.files_nodir;
 
   for (i = 0; i < 6; ++i)
