@@ -304,7 +304,7 @@ int sh_ext_popen (sh_tas_t * task)
 	  /* don't leak file descriptors
 	   */
 #if !defined(PDGBFILE)
-	  sh_unix_closeall (3, task->com_fd, SL_TRUE); /* in child process */
+	  sh_unix_closeall (3, task->com_fd, S_TRUE); /* in child process */
 #endif
 
 	  /* drop root privileges, if possible && requested
@@ -422,8 +422,8 @@ int sh_ext_popen (sh_tas_t * task)
 		char str0[128];
 		char str1[128];
 		int ival = pfd;
-		int n = 0;
-		int m = 0;
+		unsigned int n = 0;
+		unsigned int m = 0;
 
 		if (ival < 0) ival = -ival;
 		do {
@@ -632,7 +632,7 @@ int sh_ext_pclose (sh_tas_t * task)
 	  if (WIFEXITED(task->exit_status) != 0)
 	    {
 	      task->exit_status = WEXITSTATUS(task->exit_status);
-	      if ((flag_err_debug == SL_TRUE) || (task->exit_status != 0))
+	      if ((flag_err_debug == S_TRUE) || (task->exit_status != 0))
 		sl_snprintf(infomsg, sizeof(infomsg),
 			    _("Subprocess exited normally with status %d"),
 			    task->exit_status);
@@ -686,7 +686,7 @@ int sh_ext_pclose (sh_tas_t * task)
 	}
       /*@+bufferoverflowhigh@*/
       status = task->exit_status;
-      if (flag_err_debug == SL_TRUE)
+      if (flag_err_debug == S_TRUE)
 	{
 	  sh_error_handle(SH_ERR_ALL, FIL__, __LINE__, task->exit_status, 
 			  MSG_E_SUBGEN, infomsg, _("sh_ext_pclose"));
@@ -866,7 +866,7 @@ static void task_init (sh_tas_t * task)
   return;
 }
 
-int sh_ext_popen_init (sh_tas_t * task, char * command, char * argv0, ...)
+int sh_ext_popen_init (sh_tas_t * task, const char * command, char * argv0, ...)
 {
   va_list vl;
   int status;
@@ -955,7 +955,7 @@ int sh_ext_system (char * command, char * argv0, ...)
 /* Execute command, return first line of output
  * ifconfig | grep -1 lo | tail -n 1 | sed s/.*inet addr:\([0-9.]*\)\(.*\)/\1/
  */
-char * sh_ext_popen_str (char * command)
+char * sh_ext_popen_str (const char * command)
 {
   sh_tas_t task;
   struct  sigaction  new_act;
@@ -977,6 +977,9 @@ char * sh_ext_popen_str (char * command)
    /* ignore SIGPIPE (instead get EPIPE if connection is closed)
     */
    new_act.sa_handler = SIG_IGN;
+   new_act.sa_flags   = 0;
+   sigemptyset( &new_act.sa_mask );
+   
    (void) retry_sigaction (FIL__, __LINE__, SIGPIPE, &new_act, &old_act);
    
   /* read from the open pipe
