@@ -179,10 +179,7 @@ int commit_list (SH_FIFO * fifo)
 	      size_t len = sl_strlen(getit->data);
 	      memset(getit->data, 0, len);
 	      if (NULL != sl_strstr (getit->data, _("LOGKEY")))
-		{
-		  MUNLOCK(getit->data, (len+1));
-		  ;
-		}
+		MUNLOCK(getit->data, (len+1));
 	      if (getit->s_xtra)
 		SH_FREE(getit->s_xtra);
 	      SH_FREE(getit->data);
@@ -196,7 +193,7 @@ int commit_list (SH_FIFO * fifo)
 
 /* push an item on the head of the list
  */
-int push_list (SH_FIFO * fifo, char * indat, int in_i, const char * in_str)
+int push_list (SH_FIFO * fifo, const char * indat, int in_i, const char * in_str)
 {
   struct dlist * item;
   size_t         len;
@@ -223,10 +220,7 @@ int push_list (SH_FIFO * fifo, char * indat, int in_i, const char * in_str)
   item->data      = SH_ALLOC(len+1);
   
   if (NULL != sl_strstr (indat, _("LOGKEY")))
-    {
-      MLOCK(item->data, (len+1));
-      ;
-    }
+    MLOCK(item->data, (len+1));
 
   sl_strlcpy (item->data, indat, len+1);
   item->data[len] = '\0';
@@ -259,7 +253,7 @@ int push_list (SH_FIFO * fifo, char * indat, int in_i, const char * in_str)
 
 /* push an item on the tail of the list
  */
-int push_tail_list (SH_FIFO * fifo, char * indat, int in_i, const char * in_str)
+int push_tail_list (SH_FIFO * fifo, const char * indat, int in_i, const char * in_str)
 {
   struct dlist * item;
   size_t         len;
@@ -286,10 +280,7 @@ int push_tail_list (SH_FIFO * fifo, char * indat, int in_i, const char * in_str)
   item->data      = SH_ALLOC(len+1);
 
   if (NULL != sl_strstr (indat, _("LOGKEY")))
-    {
-      MLOCK(item->data, (len+1));
-      ;
-    }
+    MLOCK(item->data, (len+1));
 
   sl_strlcpy (item->data, indat, len+1);
   item->data[len] = '\0';
@@ -317,7 +308,7 @@ int push_tail_list (SH_FIFO * fifo, char * indat, int in_i, const char * in_str)
 
   ++(fifo->fifo_cts);
 
-  SL_RETURN((0), _("push_tail_list"));
+  SL_RETURN((fifo->fifo_cts), _("push_tail_list"));
 }
 
 /* pop an item from the tail of the list
@@ -355,10 +346,7 @@ int push_tail_list (SH_FIFO * fifo, char * indat, int in_i, const char * in_str)
   memset(getit->data, 0, len);
 
   if (NULL != sl_strstr (retval, _("LOGKEY")))
-    {
-      MUNLOCK(getit->data, (len+1));
-      ;
-    }
+    MUNLOCK(getit->data, (len+1));
 
   if (getit->s_xtra)
     SH_FREE(getit->s_xtra);
@@ -371,5 +359,83 @@ int push_tail_list (SH_FIFO * fifo, char * indat, int in_i, const char * in_str)
 }
 
 
+#ifdef SH_CUTEST
+#include "CuTest.h"
+
+void Test_fifo (CuTest *tc) {
+
+  SH_FIFO ff;
+  int ret;
+  char * p;
+
+  fifo_init(&ff);
+
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrEquals(tc, NULL, p);
+
+  /* first sequence */
+  ret = sh_fifo_push(&ff, "one");
+  CuAssertIntEquals(tc,1,ret);
+  ret = sh_fifo_push(&ff, "two");
+  CuAssertIntEquals(tc,2,ret);
+  ret = sh_fifo_push(&ff, "three");
+  CuAssertIntEquals(tc,3,ret);
+
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"one", p);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"two", p);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"three", p);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrEquals(tc, NULL, p);
+
+  /* second sequence */
+  ret = sh_fifo_push(&ff, "one");
+  CuAssertIntEquals(tc,1,ret);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"one", p);
+  ret = sh_fifo_push_tail(&ff, "one");
+  CuAssertIntEquals(tc,1,ret);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"one", p);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrEquals(tc, NULL, p);
+
+  /* third sequence */
+  ret = sh_fifo_push(&ff, "one");
+  CuAssertIntEquals(tc,1,ret);
+  ret = sh_fifo_push(&ff, "two");
+  CuAssertIntEquals(tc,2,ret);
+  ret = sh_fifo_push(&ff, "three");
+  CuAssertIntEquals(tc,3,ret);
+
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"one", p);
+  ret = sh_fifo_push_tail(&ff, p);
+  CuAssertIntEquals(tc,3,ret);
+
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"one", p);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"two", p);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrNotNull(tc, p);
+  CuAssertStrEquals(tc,"three", p);
+  p = sh_fifo_pop(&ff);
+  CuAssertPtrEquals(tc, NULL, p);
+}
+
+#endif
+
+
 
 
