@@ -405,19 +405,6 @@ static int init_record(unsigned int n, char * define,
       return -1;
     }
 
-  /* fprintf(stderr, "FIXME: %d %d %s ", dtype, length, type); */
-  /**
-  if (dtype == 0)
-    fprintf(stderr, "%s\n", pattern);
-  else
-    {
-      int k;
-      for (k = 0; k < length; ++k)
-	fprintf(stderr, "0x%X", (unsigned int) (pattern[k]));
-      fprintf(stderr, "\n");
-    }
-  **/
-
   for (i = 0; i < n; ++i)
     {
       if (sh_ftype_arr[i]->length <= length &&
@@ -471,7 +458,7 @@ static void file_arr_init()
 #endif
 
 #ifdef  SH_FILE_MAIN
-  sh_ftype_arr = malloc((nn+1) * sizeof(struct sh_ftype_rec *));
+  sh_ftype_arr = calloc((nn+1), sizeof(struct sh_ftype_rec *));
 #else
   sh_ftype_arr = SH_ALLOC((nn+1) * sizeof(struct sh_ftype_rec *));
 #endif
@@ -479,7 +466,7 @@ static void file_arr_init()
   for(i = 0; i < nn; i++)
     {
 #ifdef  SH_FILE_MAIN
-      sh_ftype_arr[i] = malloc(sizeof(struct sh_ftype_rec));
+      sh_ftype_arr[i] = calloc(1, sizeof(struct sh_ftype_rec));
 #else
       sh_ftype_arr[i] = SH_ALLOC(sizeof(struct sh_ftype_rec));
 #endif
@@ -522,21 +509,6 @@ static char * check_filetype(char * filetype,
 	{
 	  p = &buffer[sh_ftype_arr[i]->offset];
 
-#if 0
-	  {
-	    int dd;
-	    /* fprintf(stderr, "FIXME: %03d comp %d:%d  ", i, 
-	       sh_ftype_arr[i]->offset, sh_ftype_arr[i]->length); */
-	    for (dd = 0; dd < sh_ftype_arr[i]->length; ++dd) {
-	      fprintf(stderr, "0x%X ", sh_ftype_arr[i]->pattern[dd]);
-	    }
-	    for (dd = 0; dd < sh_ftype_arr[i]->length; ++dd) {
-	      fprintf(stderr, "0x%X ", p[dd]);
-	    }
-	    fprintf(stderr, "\n");
-	  }
-#endif
-
 	  if (0 == memcmp(p, sh_ftype_arr[i]->pattern, sh_ftype_arr[i]->length))
 	    {
 	      sl_strlcpy(filetype, sh_ftype_arr[i]->type, SH_FTYPE_MAX);
@@ -581,7 +553,7 @@ int matches_filetype(SL_TICKET ft, char * test_type)
   char filetype[SH_FTYPE_MAX];
   long len;
 
-  len = sl_read_timeout (ft, buffer, sizeof(buffer), 12, SL_TRUE);
+  len = sl_read_timeout (ft, buffer, sizeof(buffer), 12, S_TRUE);
 
   sl_rewind(ft);
 
