@@ -469,14 +469,14 @@ static char * get_label_and_time(const char * inprefix, char * str,
   return res;
 }
 
-static struct sh_qeval ** dummy_queue;
-static char            ** dummy_dstr;
+struct sh_qeval ** sh_dummy_472_queue;
+char            ** sh_dummy_473_dstr;
 
 int sh_eval_radd (const char * str)
 {
   struct sh_geval * nr;
   struct sh_geval * tmp;
-  struct sh_qeval * queue = NULL;
+  struct sh_qeval * queue;
   pcre *  rule;
   pcre_extra * rule_extra;
   const char * error;
@@ -505,8 +505,10 @@ int sh_eval_radd (const char * str)
 
   splits = split_array(new, &nfields, ':', lengths);
 
-  dummy_queue = &queue;
-  dummy_dstr  = &dstr;
+  sh_dummy_472_queue = &queue;
+  sh_dummy_473_dstr  = &dstr;
+
+  queue = NULL;
 
   if (nfields < 2 || nfields > 3)
     {
@@ -592,7 +594,7 @@ int sh_eval_radd (const char * str)
   rule_extra = NULL; /* pcre_study(rule, 0, &error); */
   pcre_fullinfo(rule, rule_extra, PCRE_INFO_CAPTURECOUNT, &captures);
 
-  if (flag_err_debug == SL_TRUE)
+  if (flag_err_debug == S_TRUE)
     {
       char * emsg = SH_ALLOC(SH_ERRBUF_SIZE);
       if (dstr)
@@ -653,7 +655,7 @@ int sh_eval_radd (const char * str)
    */
   if (group_open)
     {
-      if (flag_err_debug == SL_TRUE)
+      if (flag_err_debug == S_TRUE)
 	{
 	  char * emsg = SH_ALLOC(SH_ERRBUF_SIZE);
 	  sl_snprintf(emsg,  SH_ERRBUF_SIZE, _("Adding rule to group |%s|"), 
@@ -825,7 +827,7 @@ void sh_eval_cleanup()
 /* Test a list of rules against msg; return matched rule, with ovector 
  * filled in
  */
-static struct sh_geval ** dummy1;
+struct sh_geval ** sh_dummy_828_rule;
 
 static struct sh_geval * test_rule (struct sh_geval * rule, sh_string *msg, time_t tstamp)
 {
@@ -833,7 +835,7 @@ static struct sh_geval * test_rule (struct sh_geval * rule, sh_string *msg, time
   volatile int    count;
   volatile time_t timestamp = tstamp;
 
-  dummy1 = &rule;
+  sh_dummy_828_rule = &rule;
 
   if (!rule)
     DEBUG("debug: (NULL) rule\n");
@@ -843,7 +845,7 @@ static struct sh_geval * test_rule (struct sh_geval * rule, sh_string *msg, time
       count = 1;
       do {
 
-	if (flag_err_debug == SL_TRUE)
+	if (flag_err_debug == S_TRUE)
 	  {
 	    char * emsg = SH_ALLOC(SH_ERRBUF_SIZE);
 	    sl_snprintf(emsg,  SH_ERRBUF_SIZE, _("Check rule %d for |%s|"), 
@@ -863,7 +865,7 @@ static struct sh_geval * test_rule (struct sh_geval * rule, sh_string *msg, time
 	  {
 	    rule->ovecnum = res;
 
-	    if (flag_err_debug == SL_TRUE)
+	    if (flag_err_debug == S_TRUE)
 	      {
 		char * emsg = SH_ALLOC(SH_ERRBUF_SIZE);
 		if ( rule->flags & RFL_KEEP )
@@ -899,7 +901,7 @@ static struct sh_geval * test_rule (struct sh_geval * rule, sh_string *msg, time
 	    break; /* return the matching rule; ovector is filled in */
 	  }
 
-	if (flag_err_debug == SL_TRUE)
+	if (flag_err_debug == S_TRUE)
 	  {
 	    char * emsg = SH_ALLOC(SH_ERRBUF_SIZE);
 	    sl_snprintf(emsg,  SH_ERRBUF_SIZE, _("Rule %d did not match"), 
@@ -918,15 +920,15 @@ static struct sh_geval * test_rule (struct sh_geval * rule, sh_string *msg, time
   if (!rule)
     DEBUG("debug: no match found\n");
   /* If there was no match, this is NULL */
-  dummy1 = NULL;
+  sh_dummy_828_rule = NULL;
   return rule;
 }
   
 /* Test a (struct sh_geval *), which may be single rule or a group of rules,
  * against msg
  */
-static struct sh_geval ** dummy2;
-static struct sh_geval ** dummy3;
+struct sh_geval ** sh_dummy_928_result;
+struct sh_geval ** sh_dummy_929_group;
 
 static struct sh_geval * test_grules (struct sh_heval * host, 
 				      sh_string       * msg,
@@ -935,8 +937,8 @@ static struct sh_geval * test_grules (struct sh_heval * host,
   struct sh_geval * result = NULL;
   struct sh_geval * group  = host->rulegroups;
 
-  dummy2 = &result;
-  dummy3 = &group;
+  sh_dummy_928_result = &result;
+  sh_dummy_929_group  = &group;
 
   if (group && sh_string_len(msg) < (size_t)INT_MAX)
     {
@@ -946,7 +948,7 @@ static struct sh_geval * test_grules (struct sh_heval * host,
 	  {
 	    /* this is a rule group */
 
-	    if (flag_err_debug == SL_TRUE)
+	    if (flag_err_debug == S_TRUE)
 	      {
 		char * emsg = SH_ALLOC(SH_ERRBUF_SIZE);
 		sl_snprintf(emsg,  SH_ERRBUF_SIZE, _("Checking group |%s| of rules against |%s|"), 
@@ -974,7 +976,7 @@ static struct sh_geval * test_grules (struct sh_heval * host,
 	     * rule (not within any group).
 	     */
 
-	    if (flag_err_debug == SL_TRUE)
+	    if (flag_err_debug == S_TRUE)
 	      {
 		char * emsg = SH_ALLOC(SH_ERRBUF_SIZE);
 		sl_snprintf(emsg,  SH_ERRBUF_SIZE, _("Checking solitary rules"));
@@ -994,8 +996,8 @@ static struct sh_geval * test_grules (struct sh_heval * host,
       } while (group);
     }
 
-  dummy2 = NULL;
-  dummy3 = NULL;
+  sh_dummy_928_result = NULL;
+  sh_dummy_929_group  = NULL;
   return result;
 }
 
@@ -1112,7 +1114,7 @@ static void sum_report(int severity, const sh_string * alias,
 
 static zAVLKey sh_eval_getkey(void const *item)
 {
-  return ((struct sh_ceval *)item)->hostname->str;
+  return ((const struct sh_ceval *)item)->hostname->str;
 }
 
 /* Find the counter, or initialize one if there is none already
