@@ -68,7 +68,7 @@
 #undef  GOOD
 #undef  BAD
 
-static int failedMail = SL_FALSE;
+static int failedMail = S_FALSE;
 
 static dnsrep * return_mx (char *domain);
 
@@ -336,7 +336,7 @@ int sh_mail_pushstack (int severity, const char * msg, const char * alias)
 
   SL_ENTER(_("sh_mail_pushstack"));
 
-  if (msg == NULL || failedMail == SL_TRUE /* || sh.srvmail.name[0] == '\0' */) 
+  if (msg == NULL || failedMail == S_TRUE /* || sh.srvmail.name[0] == '\0' */) 
     SL_RETURN((0), (_("sh_mail_pushstack")));
 
   p = split_string(msg);
@@ -549,7 +549,7 @@ int sh_mail_msg (const char * message)
     /* 
      * Return if we cannot mail.
      */
-    if (failedMail == SL_TRUE) 
+    if (failedMail == S_TRUE) 
       SL_RETURN((-1), _("sh_mail_msg"));
 
     /*
@@ -563,7 +563,7 @@ int sh_mail_msg (const char * message)
 			 sh_string_str(all_recipients->recipient));
 	sh_mail_emptystack();
 	sh.mailNum.alarm_last = 0;
-	failedMail = SL_TRUE;
+	failedMail = S_TRUE;
 	SL_RETURN((-1), _("sh_mail_msg"));
       }
 
@@ -1258,16 +1258,23 @@ static FILE * sh_mail_start_conn (struct alias * ma_address,
 
   my_time = time(NULL);
 #if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_LOCALTIME_R)
-  my_tm   = localtime_r(&my_time, &time_tm);
+  my_tm = localtime_r(&my_time, &time_tm);
 #else
-  my_tm   = localtime(&my_time);
+  my_tm = localtime(&my_time);
 #endif
-
+  
+  if (!my_tm) 
+    {
+      sl_strlcpy(my_tbuf, _("Thu, 01 Jan 1970 00:00:00 +00:00"), sizeof(my_tbuf));
+    }
+  else
+    {
 #if defined(HAVE_STRFTIME_Z)
-  (void)    strftime(my_tbuf, 127, _("%a, %d %b %Y %H:%M:%S %z"), my_tm);
+      (void) strftime(my_tbuf, 127, _("%a, %d %b %Y %H:%M:%S %z"), my_tm);
 #else
-  (void)    strftime(my_tbuf, 127, _("%a, %d %b %Y %H:%M:%S %Z"), my_tm);
+      (void) strftime(my_tbuf, 127, _("%a, %d %b %Y %H:%M:%S %Z"), my_tm);
 #endif
+    }
 
   TPT(( 0, FIL__, __LINE__,  _("msg=<From: <%s>%c%cTo: <%s>%c%cDate: %s>%c%c"),
 	this_address, 13, 10, address, 13, 10, my_tbuf, 13, 10));
@@ -1338,7 +1345,7 @@ static void report_smtp (char * reply)
 {
   char * tmp;
 
-  if (flag_err_debug == SL_TRUE)
+  if (flag_err_debug == S_TRUE)
     {
       tmp = sh_util_safe_name_keepspace(reply);
 
@@ -1381,7 +1388,7 @@ static int sh_mail_wait(int code, int ma_socket)
   state    = WAIT_CODE_START;
   reply[0] = '\0';
 
-  while (sl_read_timeout_fd (ma_socket, &c, 1, time_wait, SL_FALSE) > 0) {
+  while (sl_read_timeout_fd (ma_socket, &c, 1, time_wait, S_FALSE) > 0) {
 
     if (ireply < (sizeof(reply) - 1))
       {
