@@ -111,7 +111,7 @@ void sh_mem_dump ()
 }
 #endif
 
-static memlist_t ** sh_mem_merr_1;
+memlist_t ** sh_dummy_114_merrlist;
 
 void sh_mem_stat ()
 {
@@ -120,7 +120,7 @@ void sh_mem_stat ()
 
   SL_ENTER(_("sh_mem_stat"));
 
-  sh_mem_merr_1 = (memlist_t **) &merrlist;
+  sh_dummy_114_merrlist = (memlist_t **) &merrlist;
 
   if (Alloc_Count == Free_Count) 
     {
@@ -141,7 +141,7 @@ void sh_mem_stat ()
 
   while (this != NULL) 
     {
-      memlist_t   * merr = (memlist_t *) malloc (sizeof(memlist_t));
+      memlist_t   * merr = calloc(1,sizeof(memlist_t));
 
       memcpy(merr, this, sizeof(memlist_t));
       merr->next = merrlist;
@@ -165,7 +165,7 @@ void sh_mem_stat ()
   SL_RET0(_("sh_mem_stat"));
 }
 
-static memlist_t ** sh_mem_merr_2;
+memlist_t ** sh_dummy_168_merrlist;
 
 void sh_mem_check ()
 {
@@ -176,7 +176,7 @@ void sh_mem_check ()
 
   SL_ENTER(_("sh_mem_check"));
 
-  sh_mem_merr_2 = (memlist_t **) &merrlist;
+  sh_dummy_168_merrlist = (memlist_t **) &merrlist;
 
   sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_MSTAMP,
 		   Mem_Max, Mem_Current);
@@ -190,7 +190,7 @@ void sh_mem_check ()
     {
       if ( this->address == NULL )
 	{
-	  merr = (memlist_t *) malloc (sizeof(memlist_t));
+	  merr = calloc (1,sizeof(memlist_t));
 
 	  memcpy(merr, this, sizeof(memlist_t));
 	  merr->size = 2;
@@ -203,7 +203,7 @@ void sh_mem_check ()
 	{
 	  if ( this->address[this->size]        != CHECKBYTE )
 	    {
-	      merr = (memlist_t *) malloc (sizeof(memlist_t));
+	      merr = calloc(1, sizeof(memlist_t));
 	      
 	      memcpy(merr, this, sizeof(memlist_t));
 	      merr->size = 1;
@@ -214,7 +214,7 @@ void sh_mem_check ()
 	    }
 	  if ( this->real_address[SH_MEMMULT-1] != CHECKBYTE )
 	    {
-	      merr = (memlist_t *) malloc (sizeof(memlist_t));
+	      merr = calloc(1, sizeof(memlist_t));
 	      
 	      memcpy(merr, this, sizeof(memlist_t));
 	      merr->size = 0;
@@ -245,6 +245,8 @@ void sh_mem_check ()
 	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_E_MUNDER,
 			   tmp->file, tmp->line, FIL__, __LINE__);
       free(tmp);
+      SH_ABORT;
+      _exit (EXIT_FAILURE);
     }
 
   SL_RET0(_("sh_mem_check"));
@@ -261,7 +263,7 @@ void * sh_mem_malloc (size_t size, char * file, int line)
   SH_MUTEX_RECURSIVE_INIT(mutex_mem);
   SH_MUTEX_RECURSIVE_LOCK(mutex_mem);
 
-  the_realAddress = malloc(size + 2 * SH_MEMMULT);
+  the_realAddress = calloc(1,size + 2 * SH_MEMMULT);
   
   if ( the_realAddress  == NULL ) 
     {
@@ -289,7 +291,7 @@ void * sh_mem_malloc (size_t size, char * file, int line)
   Mem_Current += size;
   Mem_Max = ( (Mem_Current > Mem_Max) ? Mem_Current : Mem_Max);
 
-  this = (memlist_t *) malloc (sizeof(memlist_t));
+  this = calloc(1,sizeof(memlist_t));
 
   if ( this == NULL) 
     {
@@ -339,6 +341,8 @@ void sh_mem_free (void * aa, char * file, int line)
     {
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_E_MNULL,
 		       file, line, FIL__, __LINE__);
+      SH_ABORT;
+      _exit (EXIT_FAILURE);
       SL_RET0(_("sh_mem_free"));
     }
     
@@ -369,7 +373,7 @@ void sh_mem_free (void * aa, char * file, int line)
 
       if ( this->address[this->size]        != CHECKBYTE )
 	{
-	  merr = (memlist_t *) malloc (sizeof(memlist_t));
+	  merr = calloc(1, sizeof(memlist_t));
 
 	  memcpy(merr, this, sizeof(memlist_t));
 	  merr->size = 1;
@@ -380,7 +384,7 @@ void sh_mem_free (void * aa, char * file, int line)
 
       if ( this->real_address[SH_MEMMULT-1] != CHECKBYTE )
 	{
-	  merr = (memlist_t *) malloc (sizeof(memlist_t));
+	  merr = calloc(1,sizeof(memlist_t));
 
 	  memcpy(merr, this, sizeof(memlist_t));
 	  merr->size = 0;
@@ -421,6 +425,8 @@ void sh_mem_free (void * aa, char * file, int line)
 	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_E_MUNDER,
 			   tmp->file, tmp->line, file, line);
       free(tmp);
+      SH_ABORT;
+      _exit (EXIT_FAILURE);
     }
 
   if (flag != 0)
@@ -443,6 +449,7 @@ void sh_mem_free (void * a)
   else
     {
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_E_MNULL);
+      SH_ABORT;
     }
   SL_RET0(_("sh_mem_free"));
 }
@@ -453,7 +460,7 @@ void * sh_mem_malloc (size_t size)
 
   SL_ENTER(_("sh_mem_malloc"));
 
-  theAddress = malloc(size);
+  theAddress = calloc(1,size);
 
   if ( theAddress != NULL ) 
     {
@@ -465,6 +472,7 @@ void * sh_mem_malloc (size_t size)
 
       /* use _exit() rather than exit() - we malloc() in atexit()  
        */
+      SH_ABORT;
       _exit (EXIT_FAILURE);
     }
 }
