@@ -50,11 +50,11 @@
 #endif
 
 /* Prototypes for configuration functions */
-int sh_mounts_config_activate (char * opt);
-int sh_mounts_config_timer    (char * opt);
-int sh_mounts_config_mount    (char * opt);
-int sh_mounts_config_sevmnt   (char * opt);
-int sh_mounts_config_sevopt   (char * opt);
+int sh_mounts_config_activate (const char * opt);
+int sh_mounts_config_timer    (const char * opt);
+int sh_mounts_config_mount    (const char * opt);
+int sh_mounts_config_sevmnt   (const char * opt);
+int sh_mounts_config_sevopt   (const char * opt);
 
 /* Prototype for the function to read info on mounted filesystems */
 static struct sh_mounts_mnt *readmounts(void);
@@ -156,16 +156,17 @@ void sh_mounts_mnt_free(struct sh_mounts_mnt *m) {
 static time_t lastcheck         = (time_t) 0;
 static int    ShMountsActive    = S_FALSE;
 static time_t ShMountsInterval  = 86400;
-static int    ShMountsSevMnt    = 7;
-static int    ShMountsSevOpt    = 7;
+static int    ShMountsSevMnt    = SH_ERR_ERR;
+static int    ShMountsSevOpt    = SH_ERR_ERR;
 
 static struct sh_mounts_mnt *mountlist = NULL;
 
 /* Module initialisation
  * This is called once at the start of each samhain run.
  * Non-configuration setup code should be placed here. */
-int sh_mounts_init ()
+int sh_mounts_init (struct mod_type * arg)
 {
+  (void) arg;
   SL_ENTER(_("sh_mounts_init"));
 
   /* This is a little odd. Because we've built the configured mount list at
@@ -178,8 +179,6 @@ int sh_mounts_init ()
     SL_RETURN(-1, _("sh_mounts_init"));
   }
 
-  lastcheck = time(NULL);
-
   SL_RETURN(0, _("sh_mounts_init"));
 }
 
@@ -195,7 +194,7 @@ int sh_mounts_timer (time_t tcurrent)
     lastcheck = tcurrent;
     SL_RETURN(-1, _("sh_mounts_timer"));
   }
-  
+ 
   SL_RETURN(0, _("sh_mounts_timer"));
 }
 
@@ -261,13 +260,21 @@ int sh_mounts_cleanup ()
 }
 
 /* Module reconfiguration
- * Run on receipt of a HUP. Right now this is identical to _end(), but it may
- * not always be. */
+ * Run on receipt of a HUP.
+ */
 int sh_mounts_reconf()
 {
   SL_ENTER(_("sh_mounts_null"));
   sh_mounts_mnt_free(mountlist);
   mountlist = NULL;
+
+  /* re-set defaults
+   */
+  ShMountsActive    = S_FALSE;
+  ShMountsInterval  = 86400;
+  ShMountsSevMnt    = 7;
+  ShMountsSevOpt    = 7;
+
   SL_RETURN( (0), _("sh_mounts_null"));
 }
 
@@ -275,21 +282,21 @@ int sh_mounts_reconf()
  * These functions are called when the configuration file is being parsed. */
 
 /* Configure to check a particular mount */
-int sh_mounts_config_mount (char * opt)
+int sh_mounts_config_mount (const char * opt_in)
 {
   struct sh_mounts_mnt *m;
   struct sh_mounts_opt *o;
-  char *sp, *temp;
+  char *sp, *temp, *opt;
 
   SL_ENTER(_("sh_mounts_config_mount"));
 
   /* It's probably best to make a copy of opt before messing about with it
    * via string functions. Good practice and all that. */
-  temp = sh_util_strdup(opt);
+  temp = sh_util_strdup(opt_in);
 
   /* Since we're going to "consume" this new buffer, it'll be good to have a
    * reference to it's allocated memory so we can free it later. Let's use
-   * temp for that, and the now-unused "opt" for consumption */
+   * temp for that, and "opt" for consumption */
   opt = temp;
   
   m = (struct sh_mounts_mnt *) SH_ALLOC(sizeof(struct sh_mounts_mnt));
@@ -326,7 +333,7 @@ int sh_mounts_config_mount (char * opt)
 }
 
 /* Simply sets our boolean as to whether this module is active */
-int sh_mounts_config_activate (char * opt)
+int sh_mounts_config_activate (const char * opt)
 {
   int i;
   SL_ENTER(_("sh_mounts_config_activate"));
@@ -335,7 +342,7 @@ int sh_mounts_config_activate (char * opt)
 }
 
 /* Sets up our timer */
-int sh_mounts_config_timer (char * opt)
+int sh_mounts_config_timer (const char * opt)
 {
   long val;
   int retval = 0;
@@ -356,7 +363,7 @@ int sh_mounts_config_timer (char * opt)
 }
 
 /* Configure severity for "mount missing" messages */
-int sh_mounts_config_sevmnt  (char * opt)
+int sh_mounts_config_sevmnt  (const char * opt)
 {
   int retval = 0;
   char tmp[32];
@@ -369,7 +376,7 @@ int sh_mounts_config_sevmnt  (char * opt)
   SL_RETURN(retval, _("sh_mounts_config_sevmnt"));
 }
 
-int sh_mounts_config_sevopt  (char * opt)
+int sh_mounts_config_sevopt  (const char * opt)
 {
   int retval = 0;
   char tmp[32];
@@ -390,7 +397,7 @@ int sh_mounts_config_sevopt  (char * opt)
  */
 
 /* FreeBSD includes */
-#ifdef HOST_IS_FREEBSD
+#if defined(HOST_IS_FREEBSD) || defined(HOST_IS_OPENBSD) 
 #include <sys/param.h>
 #include <sys/ucred.h>
 #include <sys/mount.h>
@@ -408,7 +415,166 @@ int sh_mounts_config_sevopt  (char * opt)
 #include <sys/mnttab.h>
 #endif
 
-#if defined(HOST_IS_FREEBSD)
+/* HP_UX includes */
+#ifdef HOST_IS_HPUX
+#include <stdio.h>
+#include <mntent.h>
+#endif
+
+/* AIX includes and helper routines (from gnome-vfs-unix-mounts.c */
+#if 0
+#ifdef HOST_IS_AIX
+#include <stdio.h>
+#include <string.h>
+#include <ctype.h>
+
+/* gnome-vfs-unix-mounts.c - read and monitor fstab/mtab
+
+   Copyright (C) 2003 Red Hat, Inc
+
+   The Gnome Library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public License as
+   published by the Free Software Foundation; either version 2 of the
+   License, or (at your option) any later version.
+
+   The Gnome Library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public
+   License along with the Gnome Library; see the file COPYING.LIB.  If not,
+   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+
+   Author: Alexander Larsson <alexl@redhat.com>
+*/
+
+/* read character, ignoring comments (begin with '*', end with '\n' */
+static int aix_fs_getc (FILE *fd)
+{
+  int c;
+  
+  while ((c = getc (fd)) == '*') {
+    while (((c = getc (fd)) != '\n') && (c != EOF)) {} /* do nothing */
+  }
+}
+
+/* eat all continuous spaces in a file */
+static int aix_fs_ignorespace (FILE *fd)
+{
+  int c;
+  
+  while ((c = aix_fs_getc (fd)) != EOF) {
+    if (! (isascii(c) && isspace (c)) ) {
+      ungetc (c,fd);
+      return c;
+    }
+  }
+  
+  return EOF;
+}
+
+/* read one word from file */
+static int aix_fs_getword (FILE *fd, char *word, int len)
+{
+  int c;
+  int i = 0;
+
+  --len;
+
+  aix_fs_ignorespace (fd);
+
+  while (((c = aix_fs_getc (fd)) != EOF) && !( isascii(c) && isspace(c) )) 
+    {
+      if (c == '"') 
+	{
+	  while (((c = aix_fs_getc (fd)) != EOF) && (c != '"')) 
+	    {
+	      *word++ = c; ++i;
+	      if (i == len)
+		break;
+	    }
+	} 
+      else 
+	{
+	  *word++ = c; ++i;
+	}
+      if (i == len)
+	break;
+    }
+  *word = 0;
+  
+  return c;
+}
+
+/* PATH_MAX is in sys/limits.h, included via stdio.h
+ */
+typedef struct {
+  char mnt_mount[PATH_MAX];
+  char mnt_special[PATH_MAX];
+  char mnt_fstype[16];
+  char mnt_options[128];
+} AixMountTableEntry;
+
+/* read mount points properties */
+static int aix_fs_get (FILE *fd, AixMountTableEntry *prop)
+{
+  /* Need space for PATH_MAX + ':'   (terminating '\0' is in PATH_MAX; SUSv3)
+   */
+  static char word[PATH_MAX+1] = { 0 };
+  char value[PATH_MAX];
+
+  /* reset */
+
+  if (fd == NULL)
+    {
+      word[0] = '\0';
+      return 0;
+    }
+
+  /* read stanza */
+
+  if (word[0] == 0) {
+    if (aix_fs_getword (fd, word, (PATH_MAX+1)) == EOF)
+      return EOF;
+  }
+
+  word[strlen(word) - 1] = 0;
+  sl_strlcpy (prop->mnt_mount, word, PATH_MAX);
+
+  /* read attributes and value */
+
+  while (aix_fs_getword (fd, word, (PATH_MAX+1)) != EOF) {
+    /* test if is attribute or new stanza */
+
+    if (word[strlen(word) - 1] == ':') {
+      return 0;
+    }
+
+    /* read "=" */
+    aix_fs_getword (fd, value, PATH_MAX);
+
+    /* read value */
+    aix_fs_getword (fd, value, PATH_MAX);
+
+    if (strcmp (word, "dev") == 0) {
+      sl_strlcpy (prop->mnt_special, value, PATH_MAX);
+    } else if (strcmp (word, "vfs") == 0) {
+      sl_strlcpy (prop->mnt_fstype, value, 16);
+    } else if (strcmp (word, "options") == 0) {
+      sl_strlcpy(prop->mnt_options, value, 128);
+    }
+  }
+
+  return 0;
+}
+
+/* end AIX helper routines */
+#endif
+#endif
+
+#if defined(HOST_IS_FREEBSD) || defined(HOST_IS_OPENBSD)
 
 /* FreeBSD returns flags instead of strings as mount options, so we'll convert
  * them here. */
@@ -418,14 +584,33 @@ struct sh_mounts_opt * getoptlist(int flags) {
 	int i;
 
 	struct {char *opt; int flag;} table[] = {
+#ifdef MNT_RDONLY
 		{"ro",		MNT_RDONLY},
+#endif
+#ifdef MNT_NOEXEC
 		{"noexec",	MNT_NOEXEC},
+#endif
+#ifdef MNT_NOSUID
 		{"nosuid",	MNT_NOSUID},
+#endif
+#ifdef MNT_NODEV
 		{"nodev",	MNT_NODEV},
+#endif
+#ifdef MNT_SYNCHRONOUS
 		{"sync",	MNT_SYNCHRONOUS},
+#endif
+#ifdef MNT_ASYNC
 		{"async",	MNT_ASYNC},
+#endif
+#ifdef MNT_LOCAL
 		{"local",	MNT_LOCAL},
+#endif
+#ifdef MNT_QUOTA
 		{"quota",	MNT_QUOTA},
+#endif
+#ifdef MNT_NOATIME
+		{"noatime",	MNT_NOATIME},
+#endif
 		{"bound",	-1}
 	};
   	
@@ -489,15 +674,15 @@ struct sh_mounts_opt * getoptlist(char *opt)
 
 /* Read the list of mounts from whereever is appropriate to the OS and return
  * it. Return NULL on error. */
-struct sh_mounts_mnt * readmounts(void) {
+static struct sh_mounts_mnt * readmounts(void) {
 	struct sh_mounts_mnt *list, *m;
 
   	SL_ENTER(_("readmounts"));
 	m    = NULL; /* use it to avoid compiler warning */
 	list = m;
 
-/* The FreeBSD way */
-#ifdef HOST_IS_FREEBSD
+/* The Open/FreeBSD way */
+#if defined(HOST_IS_FREEBSD) || defined(HOST_IS_OPENBSD)
 {
 	struct statfs *fsp;
 	int entries;
@@ -563,10 +748,69 @@ struct sh_mounts_mnt * readmounts(void) {
 		list = m;
 	}
 
-	fclose(tab);
+	sl_fclose(FIL__, __LINE__, tab);
 }
 #endif
 
+
+/* The HP-UX way */
+#ifdef HOST_IS_HPUX
+{
+        struct mntent *mp;
+        FILE *tab = setmntent(MNT_MNTTAB, "r");
+
+        if (tab == NULL) {
+          SL_RETURN((NULL), _("readmounts"));
+        }
+
+        mp = getmntent(tab);
+        while (mp != NULL) {
+                m = (struct sh_mounts_mnt *) SH_ALLOC(sizeof (struct sh_mounts_mnt));
+                m->path = sh_util_strdup(mp->mnt_dir);
+                m->opts = getoptlist(mp->mnt_opts);
+
+                m->next = list;
+                list = m;
+
+                mp = getmntent(tab);
+        }
+
+        (void) endmntent(tab);
+}
+#endif
+
+/* The AIX way */
+#if 0
+#ifdef HOST_IS_AIX
+{
+        AixMountTableEntry mntent;
+        FILE *tab = fopen("/etc/filesystems", "r");
+
+        if (tab == NULL) {
+          SL_RETURN((NULL), _("readmounts"));
+        }
+
+	while (!aix_fs_get (tab, &mntent)) 
+	  {
+                m = (struct sh_mounts_mnt *) SH_ALLOC(sizeof (struct sh_mounts_mnt));
+                m->path = sh_util_strdup(mntent.mnt_mount);
+                m->opts = getoptlist(mntent.mnt_options);
+
+                m->next = list;
+                list = m;
+
+		mntent.mnt_mount[0]   = '\0';
+		mntent.mnt_special[0] = '\0';
+		mntent.mnt_fstype[0]  = '\0';
+		mntent.mnt_options[0] = '\0';
+        }
+
+        (void) sl_fclose(FIL__, __LINE__, tab);
+	aix_fs_get (NULL, NULL); /* reset */
+}
+#endif
+#endif
+
   	SL_RETURN((list), _("readmounts"));
 
 }
