@@ -23,6 +23,12 @@
 #define _XOPEN_SOURCE 500
 #endif
 
+#if defined(GCC_VERSION_MAJOR) && !defined(__clang__)
+#if (GCC_VERSION_MAJOR > 4) || ((GCC_VERSION_MAJOR == 4) && (GCC_VERSION_MINOR > 8))
+#pragma GCC diagnostic ignored "-Wclobbered"
+#endif
+#endif
+
 #include <string.h>
 #include <time.h>
 
@@ -322,7 +328,7 @@ int sh_nmail_valid_message_for_alias (int level,
 				      const char * alias, 
 				      const void * rcv_info)
 {
-  struct alias * rcv = (struct alias *) rcv_info;
+  const struct alias * rcv = (const struct alias *) rcv_info;
 
   if (!alias || 0 == strcmp(alias, sh_string_str(rcv->recipient)))
     {
@@ -956,7 +962,7 @@ int sh_nmail_get_mailkey (const char * alias, char * buf, size_t bufsiz,
       return (t->mailcount);
     }
 
-  mailkeys = zAVLAllocTree (sh_nmail_getkey);
+  mailkeys = zAVLAllocTree (sh_nmail_getkey, zAVL_KEY_STRING);
   goto start;
 }
 
