@@ -944,12 +944,6 @@ char * sh_port2proc_query(int proto, struct sh_sockaddr * saddr, int sport,
     if (!s)
       continue;
 
-    /* fprintf(stderr, "FIXME: %d %d, %d %d, %d %d, %d, %d\n", s->proto, proto,
-       s->family, AF_INET,
-       sport, ntohs(((struct sockaddr_in *)(&s->laddr))->sin_port),
-       (int) xf->xf_uid, (int)xf->xf_pid);
-    */
-
     if (s->proto != proto)
       continue;
 
@@ -970,9 +964,6 @@ char * sh_port2proc_query(int proto, struct sh_sockaddr * saddr, int sport,
       haddr6 = &((struct sockaddr_in6 *)(&s->laddr))->sin6_addr;
     
 
-    /* fprintf(stderr, "FIXME: %s\n", inet_ntoa(*haddr)); */
-    /* fprintf(stderr, "FIXME: %s\n", inet_ntoa(*saddr)); */
-
     if ( (s->family == AF_INET && 
 	  (haddr->s_addr == (saddr->sin).sin_addr.s_addr || 
 	   sh_ipvx_isany(saddr) || 
