@@ -97,6 +97,16 @@
 #define SH_PORT_IGN 3
 #define SH_PORT_BLACKLIST 4
 
+static char * sh_port_type2str (int type)
+{
+  if (type == 0) return _("not");
+  if (type == 1) return _("req");
+  if (type == 2) return _("opt");
+  if (type == 3) return _("ign");
+  if (type == 4) return _("blc");
+  return _("???");
+}
+
 #define SH_PORT_MISS 0
 #define SH_PORT_ISOK 1
 #define SH_PORT_UNKN 2
@@ -518,12 +528,13 @@ static struct sh_port * sh_portchk_kill_blacklist (struct sh_port * head)
  * putting it into a register, and avoids the 'clobbered
  * by longjmp' warning. And no, 'volatile' proved insufficient.
  */
-static void * sh_dummy_pre = NULL;
-static void * sh_dummy_ptr = NULL;
+void * sh_dummy_531_pre = NULL;
+void * sh_dummy_532_ptr = NULL;
 
 /* check the list of open ports for any that are marked as UNKN
  */
-static void sh_portchk_check_list (struct sh_portentry ** head, int proto, int report)
+static void sh_portchk_check_list (struct sh_portentry ** head, 
+				   int proto, int report)
 {
   struct sh_portentry * ptr = *head;
   struct sh_portentry * pre = *head;
@@ -532,8 +543,8 @@ static void sh_portchk_check_list (struct sh_portentry ** head, int proto, int r
   /* Take the address to keep gcc from putting them into registers. 
    * Avoids the 'clobbered by longjmp' warning. 
    */
-  sh_dummy_pre = (void*) &pre;
-  sh_dummy_ptr = (void*) &ptr;
+  sh_dummy_531_pre = (void*) &pre;
+  sh_dummy_532_ptr = (void*) &ptr;
  
   while (ptr)
     {
@@ -605,8 +616,8 @@ static void sh_portchk_check_list (struct sh_portentry ** head, int proto, int r
       ptr = ptr->next;
     }
 
-  sh_dummy_ptr = NULL;
-  sh_dummy_pre = NULL;
+  sh_dummy_532_ptr = NULL;
+  sh_dummy_531_pre = NULL;
 
   return;
 }
@@ -754,6 +765,9 @@ static void sh_portchk_cmp_to_list (int proto, int port,
 	  char   user[USER_MAX];
 	  char   saddr[SH_IP_BUF];
 
+	  if (portchk_debug)
+	    fprintf(stderr, _("call to  sh_ipvx_ntoa (port %d)\n"), port);
+ 
 	  sh_ipvx_ntoa(saddr, sizeof(saddr), paddr);
 
 	  snprintf (errbuf, sizeof(errbuf), _("port: %s:%d/%s (%s)"), 
@@ -961,6 +975,15 @@ static int check_port_udp_internal (int fd, int port, struct sh_sockaddr * paddr
 		}
 	      
 	    }
+	  else
+	    {
+	      if (portchk_debug)
+		{
+		  sh_ipvx_ntoa(ipbuf, sizeof(ipbuf), paddr);
+		  fprintf(stderr, _("check port_udp: %5d/udp on %15s ERRNO %d\n"), 
+			  port, ipbuf, errno);
+		}
+	    }
 	}
     }
   sl_close_fd (FIL__, __LINE__, fd);
@@ -990,7 +1013,7 @@ static int check_port_tcp_internal (int fd, int port, struct sh_sockaddr * paddr
       if (portchk_debug)
 	{
 	  sh_ipvx_ntoa(ipbuf, sizeof(ipbuf), paddr);
-	  fprintf(stderr, _("check port_tcp: %5d on %15s established/time_wait\n"),
+	  fprintf(stderr, _("check port_tcp: %5d on %15s closed\n"),
 		  port, ipbuf);
 	}
     }
@@ -1113,7 +1136,7 @@ static char * portchk_hostname = sh.host.name;
 
 static int sh_portchk_init_internal (void)
 {
-  volatile int     i; /* might be clobbered by ‘longjmp’ or ‘vfork’*/
+  volatile int     i, j; /* might be clobbered by ‘longjmp’ or ‘vfork’*/
   char errbuf[256];
 #if defined(USE_IPVX)
   struct addrinfo hints;
@@ -1146,8 +1169,19 @@ static int sh_portchk_init_internal (void)
   while (hent && hent->h_addr_list[i] && (iface_list.used < SH_IFACE_MAX))
     {
       struct sockaddr_in sin;
+      struct sh_sockaddr iface_tmp;
 
       memcpy(&(sin.sin_addr.s_addr), hent->h_addr_list[i], sizeof(in_addr_t));
+      sh_ipvx_save(&iface_tmp, AF_INET, (struct sockaddr *)&sin);
+
+      for (j = 0; j < iface_list.used; ++j)
+	{
+	  if (0 == sh_ipvx_cmp(&iface_tmp, &(iface_list.iface[j])))
+	    {
+	      goto next_iface;
+	    }
+	}
+
       sh_ipvx_save(&(iface_list.iface[iface_list.used]), 
 		   AF_INET, (struct sockaddr *)&sin);
       
@@ -1155,9 +1189,11 @@ static int sh_portchk_init_internal (void)
 	{
 	  char buf[256];
 	  sh_ipvx_ntoa(buf, sizeof(buf), &(iface_list.iface[iface_list.used]));
-	  fprintf(stderr, _("interface[%d]: %s\n"), i, buf); 
+	  fprintf(stderr, _("added interface[%d]: %s\n"), i, buf); 
 	}
       ++iface_list.used;
+
+    next_iface:
       ++i;
     }
   SH_MUTEX_UNLOCK(mutex_resolv);
@@ -1169,12 +1205,41 @@ static int sh_portchk_init_internal (void)
   if (0 == getaddrinfo(portchk_hostname, NULL, &hints, &res))
     {
       struct addrinfo *p = res;
+      struct sh_sockaddr iface_tmp;
 
       while ((p != NULL) && (iface_list.used < SH_IFACE_MAX))
 	{
+	  sh_ipvx_save(&iface_tmp, p->ai_family, p->ai_addr);
+
+          for (j = 0; j < iface_list.used; ++j)
+	    {
+	      if (portchk_debug)
+		{
+		  char buf1[256], buf2[256];
+		  sh_ipvx_ntoa(buf1, sizeof(buf1), &(iface_list.iface[j]));
+		  sh_ipvx_ntoa(buf2, sizeof(buf2), &iface_tmp);
+		  fprintf(stderr, _("check interface[%d]: %s vs %s\n"), j, buf1, buf2); 
+		}
+	      if (0 == sh_ipvx_cmp(&iface_tmp, &(iface_list.iface[j])))
+		{
+		  if (portchk_debug) 
+		    fprintf(stderr, _("skipping interface[%d]\n"), j);
+		  goto next_iface;
+		}
+	    }
 	  sh_ipvx_save(&(iface_list.iface[iface_list.used]), 
 		       p->ai_family, p->ai_addr);
+
+	  if (portchk_debug)
+	    {
+	      char buf[256];
+	      sh_ipvx_ntoa(buf, sizeof(buf), &(iface_list.iface[iface_list.used]));
+	      fprintf(stderr, _("added interface[%d]: %s\n"), iface_list.used, buf); 
+	    }
+
 	  ++iface_list.used;
+
+	next_iface:
 	  p = p->ai_next;
 	}
       freeaddrinfo(res);
@@ -1184,7 +1249,7 @@ static int sh_portchk_init_internal (void)
   for (i = 0; i < iface_list.used; ++i)
     {
       sh_ipvx_ntoa(ipbuf, sizeof(ipbuf), &(iface_list.iface[i]));
-      sl_snprintf(errbuf, sizeof(errbuf), _("interface: %s"), ipbuf);
+      sl_snprintf(errbuf, sizeof(errbuf), _("added interface: %s"), ipbuf);
 
       SH_MUTEX_LOCK(mutex_thread_nolog);
       sh_error_handle(SH_ERR_INFO, FIL__, __LINE__, 0, MSG_E_SUBGEN, 
@@ -1372,8 +1437,9 @@ static int sh_portchk_scan_ports_generic (int min_port, int max_port_arg,
   volatile int max_port = max_port_arg;
   int retval;
   int sock   = -1;
+#if 0
   int flag   = 1; /* non-zero to enable an option */
-
+#endif
   struct sockaddr_in  addr4;
   struct sockaddr_in6 addr6;
 
@@ -1389,6 +1455,11 @@ static int sh_portchk_scan_ports_generic (int min_port, int max_port_arg,
   if (max_port == -1)
     max_port = 65535;
 
+  if (portchk_debug)
+    fprintf(stderr, _("scan_ports_generic %d-%d %s %s\n"), 
+	    min_port, max_port, (domain == AF_INET6) ? _("AF_INET6") : _("AF_INET"),
+	    (protocol == IPPROTO_TCP) ? _("tcp") : _("udp"));
+	    
   for (port = min_port; port <= max_port; ++port) 
     {
       if ((sock = socket(domain, type, protocol)) < 0 )
@@ -1413,6 +1484,8 @@ static int sh_portchk_scan_ports_generic (int min_port, int max_port_arg,
 #endif
 	  continue;
 	}
+
+#if 0
       if ( setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
 		      (void *) &flag, sizeof(flag)) < 0 )
 	{
@@ -1427,6 +1500,7 @@ static int sh_portchk_scan_ports_generic (int min_port, int max_port_arg,
 #endif
 	  continue;
 	}
+#endif
 
       if (domain == AF_INET)
 	{
@@ -1500,7 +1574,7 @@ static int sh_portchk_scan_ports_udp (int min_port, int max_port)
 
 /* Subroutine to add an interface
  */
-static void * sh_dummy_str    = NULL; /* fix clobbered by.. warning */
+void * sh_dummy_1564_str    = NULL; /* fix clobbered by.. warning */
 
 static int sh_portchk_add_interface (const char * str)
 {
@@ -1508,7 +1582,7 @@ static int sh_portchk_add_interface (const char * str)
   char errbuf[256];
   char buf[64];
 
-  sh_dummy_str    = (void*) &str;
+  sh_dummy_1564_str    = (void*) &str;
 
   if (iface_initialized == 0)
     {
@@ -1549,7 +1623,7 @@ static int sh_portchk_add_interface (const char * str)
       }
   } while (*str);
 
-  sh_dummy_str = NULL;
+  sh_dummy_1564_str = NULL;
   return 0;
 }
 
@@ -1611,6 +1685,13 @@ static int sh_portchk_blacklist(int port, struct sh_sockaddr * saddr, int proto)
     blacklist_tcp = black;
   else
     blacklist_udp = black;
+
+  if (portchk_debug)
+    {
+      int checkit = sh_portchk_is_blacklisted(port, saddr, proto);
+      fprintf(stderr, _("port blacklisted: %d %s\n"), port, 
+	      (checkit == 1) ? _("ok") : _("fail"));
+    }
   return 0;
 }
   
@@ -1620,7 +1701,7 @@ static int sh_portchk_blacklist(int port, struct sh_sockaddr * saddr, int proto)
 static int sh_portchk_add_required_port_generic (char * service, 
 						 char * interface, int type)
 {
-  char buf[256];
+  char buf[256] = { '\0' };
   int proto;
   char * p;
   char * endptr;
@@ -1646,6 +1727,15 @@ static int sh_portchk_add_required_port_generic (char * service,
   *p = '\0';
   port = strtoul(buf, &endptr, 0);
 
+  if (portchk_debug)
+    {
+      char buf[SH_IP_BUF];
+      sh_ipvx_ntoa(buf, sizeof(buf), &saddr);
+      fprintf(stderr, _("add_port_generic: %s (%s) %d %s (%s)\n"),
+	      interface, buf, (int) port, (proto == IPPROTO_TCP) ? _("tcp") : _("udp"),
+	      sh_port_type2str(type));
+    }
+
   /* Blacklisted ports
    */
   if (*endptr == '\0' && port <= 65535 && type == SH_PORT_BLACKLIST)
@@ -1706,7 +1796,6 @@ static int sh_portchk_add_required_port_generic (char * service,
  */
 static int sh_portchk_add_required_generic (const char * str, int type)
 {
-  size_t len;
   size_t ll = 0;
   int    status;
 
@@ -1722,18 +1811,18 @@ static int sh_portchk_add_required_generic (const char * str, int type)
 
   if (strchr(str, ':'))
     {
-      len = strlen(str);
-      for (ll = 0; ll < len; ++ll)
+      char *last = strrchr(str, ':');
+      if (last != NULL)
 	{
-	  if (str[ll] == ':' || str[ll] == ' ' || str[ll] == '\t')
-	    {
-	      interface = SH_ALLOC(ll+1);
-	      sl_strlcpy(interface, str, ll+1);
-	      interface[ll] = '\0';
-	      while (str[ll] == ':' || str[ll] == ' ' || str[ll] == '\t')
-		++ll;
-	      break;
-	    }
+	  size_t tolast = (last - str);
+	  interface = SH_ALLOC(tolast+1);
+	  sl_strlcpy(interface, str, tolast+1);
+	  interface[tolast] = '\0';
+
+	  ll = tolast;
+
+	  while (str[ll] == ':' || str[ll] == ' ' || str[ll] == '\t')
+	    ++ll;
 	}
     }
   else
@@ -1755,7 +1844,7 @@ static int sh_portchk_add_required_generic (const char * str, int type)
     }
 
   if (portchk_debug)
-    fprintf(stderr, "add ports for interface: %s\n", interface);
+    fprintf(stderr, "add ports for interface: %s (%s)\n", interface, sh_port_type2str(type));
 
   list = sh_util_strdup(&str[ll]);
 #if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_STRTOK_R)
