@@ -410,7 +410,7 @@ static int is_in_watchlist (const char *str, unsigned long num)
  * putting them into a register, and avoids the 'clobbered
  * by longjmp' warning. And no, 'volatile' proved insufficient.
  */
-static void * sh_dummy_watchlist = NULL;
+void * sh_dummy_413_watchlist = NULL;
 
 static void check_watchlist (short * res)
 {
@@ -421,7 +421,7 @@ static void check_watchlist (short * res)
   /* Take the address to keep gcc from putting them into registers. 
    * Avoids the 'clobbered by longjmp' warning. 
    */
-  sh_dummy_watchlist = (void*) &list;
+  sh_dummy_413_watchlist = (void*) &list;
 
   while (list) 
     {
@@ -467,7 +467,7 @@ static void check_watchlist (short * res)
       list = list->next;
     }
 
-  sh_dummy_watchlist = NULL;
+  sh_dummy_413_watchlist = NULL;
   return;
 }
 
@@ -624,7 +624,7 @@ int sh_prochk_set_maxpid(const char * str)
 
   value = (size_t) strtoul(str, &foo, 0);
 
-  if (*foo == '\0' && SL_TRUE == sl_ok_adds(value, 1)) {
+  if (*foo == '\0' && S_TRUE == sl_ok_adds(value, 1)) {
     sh_prochk_maxpid = value + 1;
     userdef_maxpid   = 1;
     retval = 0;
@@ -933,7 +933,7 @@ static int sh_processes_readps (FILE * in, short * res,
 	  if (c == '\n')
 	    { 
 	      tstr[pos-1] = '\0';
-	      if (flag_err_debug == SL_TRUE)
+	      if (flag_err_debug == S_TRUE)
 		{
 		  SH_MUTEX_LOCK(mutex_thread_nolog);
 		  sh_error_handle(SH_ERR_ALL, FIL__, __LINE__, num, 
