@@ -27,31 +27,31 @@
 
 
 #include "samhain.h"
-#include "sh_error.h"
-#include "sh_database.h"
-#include "sh_unix.h"
-#include "sh_utils.h"
-#include "sh_files.h"
-#include "sh_mail.h"
 #include "sh_calls.h"
-#include "sh_tiger.h"
-#include "sh_forward.h"
-#include "sh_modules.h"
+#include "sh_error.h"
+#include "sh_extern.h"
+#include "sh_unix.h"
+#include "sh_files.h"
+#include "sh_xfer.h"
 #include "sh_gpg.h"
 #include "sh_hash.h"
+#include "sh_dbIO.h"
 #include "sh_ignore.h"
-#include "sh_prelink.h"
-#include "sh_extern.h"
-
-#ifdef WITH_DATABASE
 #include "sh_database.h"
-#endif
-
-#ifdef HAVE_LIBPRELUDE_9
+#include "sh_mail.h"
+#include "sh_modules.h"
+#include "sh_nmail.h"
+#include "sh_prelink.h"
+#ifdef HAVE_LIBPRELUDE
 #include "sh_prelude.h"
 #endif
+#include "sh_tiger.h"
+#include "sh_tools.h"
+#include "sh_utils.h"
+#include "sh_restrict.h"
+#include "sh_socket.h"
 
-extern int set_reverse_lookup (char * c);
+extern int set_reverse_lookup (const char * c);
 
 #undef  FIL__
 #define FIL__  _("sh_readconf.c")
@@ -68,6 +68,9 @@ typedef enum {
   SH_SECTION_ALLIGNORE,
   SH_SECTION_USER0,
   SH_SECTION_USER1,
+  SH_SECTION_USER2,
+  SH_SECTION_USER3,
+  SH_SECTION_USER4,
   SH_SECTION_PRELINK,
 #if defined (SH_WITH_MAIL) 
   SH_SECTION_MAIL,
@@ -92,7 +95,7 @@ typedef enum {
 } ShSectionType;
 
 typedef struct str_ListSections {
-  char * name;
+  const char * name;
   int    type;
 } sh_str_ListSections;
 
@@ -107,6 +110,9 @@ struct str_ListSections tab_ListSections[] = {
   { N_("[IgnoreNone]"),       SH_SECTION_NOIGNORE},
   { N_("[User0]"),            SH_SECTION_USER0},
   { N_("[User1]"),            SH_SECTION_USER1},
+  { N_("[User2]"),            SH_SECTION_USER2},
+  { N_("[User3]"),            SH_SECTION_USER3},
+  { N_("[User4]"),            SH_SECTION_USER4},
   { N_("[Prelink]"),          SH_SECTION_PRELINK},
 #ifdef WITH_EXTERNAL
   { N_("[External]"),         SH_SECTION_EXTERNAL}, 
@@ -128,6 +134,200 @@ struct str_ListSections tab_ListSections[] = {
   { NULL,                     SH_SECTION_NONE}
 };
 
+static char * sh_readconf_expand_value (const char * str)
+{
+#ifdef SH_EVAL_SHELL
+  char * tmp = sh_util_strdup(str);
+  char * out;
+  char * tmp_orig = tmp;
+
+  while (tmp && isspace((int)*tmp)) ++tmp;
+  
+  if (tmp && tmp[0] == '$' && tmp[1] == '(')
+    {
+      size_t len = strlen(tmp);
+      while (isspace((int) tmp[len-1])) { tmp[len-1] = '\0'; --len; }
+      if (tmp[len-1] == ')')
+	{
+	  tmp[len-1] = '\0';
+	  out = sh_ext_popen_str(&tmp[2]);
+	  SH_FREE(tmp_orig);
+	  return out;
+	}
+    }
+  SH_FREE(tmp_orig);
+#endif
+  return sh_util_strdup(str);
+}
+
+enum {
+  SH_RC_ANY        = 0, 
+  SH_RC_HOST       = 1, 
+  SH_RC_SYSTEM     = 2,
+  SH_RC_FILE       = 3,
+  SH_RC_IFACE      = 4,
+#ifdef SH_EVAL_SHELL
+  SH_RC_CMD        = 5
+#endif
+};
+
+
+static int sh_readconf_cond_match(char * str, int line)
+{
+  int    match  = 0;
+  int    negate = 1;
+  int    cond_type = SH_RC_ANY;
+  char   myident[3*SH_MINIBUF+3];
+  struct stat buf;
+
+  char * p = str;
+
+  if (*p == '!') { negate = 0; ++p; }
+  if (*p == '$') { 
+    cond_type = SH_RC_SYSTEM; ++p; /* [!]$system */ 
+  }
+  else { /* *p == '@' */
+
+    ++p; while (isspace((int)*p)) ++p;
+
+    if (0 != strncasecmp(p, _("if "),   3)) {
+      cond_type = SH_RC_HOST; /* [!]$host */
+    }
+
+    else {
+
+      p += 3; while (isspace((int)*p)) ++p; /* skip the 'if\s+' */
+
+      if (0 == strncasecmp(p, _("not "), 4))
+	{
+	  p += 4; while (isspace((int)*p)) ++p;
+	  negate = 0;
+	  match  = 1;
+	}
+      else if (0 == strncmp(p, _("!"), 1))
+	{
+	  ++p; while (isspace((int)*p)) ++p;
+	  negate = 0;
+	  match  = 1;
+	}
+  
+      if (0 == strncasecmp(p, _("file_exists "), 12))
+	{
+	  p += 12; cond_type = SH_RC_FILE;
+	}
+      else if (0 == strncasecmp(p, _("interface_exists "), 17))
+	{
+	  p += 17; cond_type = SH_RC_IFACE;
+	}
+      else if (0 == strncasecmp(p, _("hostname_matches "), 17))
+	{
+	  p += 17; cond_type = SH_RC_HOST;
+	}
+      else if (0 == strncasecmp(p, _("system_matches "), 15))
+	{
+	  p += 15; cond_type = SH_RC_SYSTEM;
+	}
+#ifdef SH_EVAL_SHELL
+      else if (0 == strncasecmp(p, _("command_succeeds "), 17))
+	{
+	  p += 17; cond_type = SH_RC_CMD;
+	}
+#endif
+      else
+	{
+	  char errbuf[SH_ERRBUF_SIZE];
+	  sl_snprintf(errbuf, sizeof(errbuf), 
+		      _("Unsupported test at line %d of configuration file"),
+		      line);
+	  sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, 0, MSG_E_SUBGEN,
+			  errbuf,
+			  _("sh_readconf_cond_match"));
+	  return 0;
+	}
+    }
+  }
+
+  while (isspace((int)*p)) ++p;
+
+  switch (cond_type)
+    {
+    case SH_RC_HOST:
+      if  (sl_strncmp (p,  sh.host.name, strlen(sh.host.name)) == 0
+#ifdef HAVE_REGEX_H
+	   || sh_util_regcmp (p, sh.host.name) == 0
+#endif
+	   )
+	match = negate;
+      break;
+    case SH_RC_SYSTEM:
+      /*
+       * The system type, release, and machine.
+       */
+      sl_snprintf(myident, sizeof(myident), _("%s:%s:%s"),  
+		  sh.host.system, /* flawfinder: ignore */ 
+		  sh.host.release, sh.host.machine);
+      
+      if  (sl_strncmp (p,  myident, strlen(myident)) == 0
+#ifdef HAVE_REGEX_H
+	   || sh_util_regcmp (p, myident) == 0
+#endif
+	   )
+	match = negate;
+      break;
+    case SH_RC_FILE:
+      if (0 == retry_lstat(FIL__, __LINE__, p, &buf))
+	match = negate;
+      break;
+    case SH_RC_IFACE:
+      if (sh_tools_iface_is_present(p))
+	match = negate;
+      break;
+#ifdef SH_EVAL_SHELL
+    case SH_RC_CMD:
+      if (0 == sh_unix_run_command(p))
+	match = negate;
+      break;
+#endif
+    default:
+      /* do nothing */;
+    }
+  return match;
+}
+
+static int sh_readconf_is_end (char * str)
+{
+  int retval = 0;
+
+  if (str[0] == '@' || str[0] == '$')
+    {
+      char * p = str;
+      ++p; while (isspace((int)*p)) ++p;
+      if ( 
+	  (0 == strncasecmp (p, _("end"), 3) && (p[3] == '\0' || isspace((int)p[3]))) ||
+	  (0 == strncasecmp (p, _("fi"),  2) && (p[2] == '\0' || isspace((int)p[2])))
+	   )
+	{
+	  return 1;
+	}
+    }
+  return retval;
+}
+   
+static int sh_readconf_is_else (char * str)
+{
+  int retval = 0;
+
+  if (str[0] == '@')
+    {
+      char * p = str;
+      ++p; while (isspace((int)*p)) ++p;
+      if ( 0 == strncasecmp (p, _("else"), 4) && (p[4] == '\0' || isspace((int)p[4])) )
+	{
+	  return 1;
+	}
+    }
+  return retval;
+}
    
 static int sh_readconfig_line (char * line);
 
@@ -150,29 +350,21 @@ int sh_readconf_read (void)
   SL_TICKET    fd    = -1;
 #if defined(SH_STEALTH) && !defined(SH_STEALTH_MICRO)
   SL_TICKET    fdTmp = -1;
-  SL_TICKET open_tmp (void);
+#endif
+#if defined(WITH_GPG) || defined(WITH_PGP)
+  SL_TICKET    fdGpg = -1;
 #endif
   char * tmp;
-  char * lptr;
 
-  char   line_in[512+2];
+#define SH_LINE_IN 16384
+  char * line_in;
   char * line;
-  int    line_int;
-
-  char   myident[3*SH_MINIBUF+3];
 
   /* This is for nested conditionals.
    */
-  int    some_other_host[16]   = { 0 };
-  int    some_other_system[16] = { 0 };
-  int    seen_host   = 0;
-  int    seen_system = 0;
-  int    host_int    = 0;
-  int    sys_int     = 0;
-
-  int    invert = 0;
-  int    length = sl_strlen(sh.host.name);
-
+  int    cond_depth  = 0;
+  int    cond_excl   = 0;
+  
   int    local_file = 1;
   char   local_flag = 'R';
 
@@ -184,6 +376,7 @@ int sh_readconf_read (void)
   int    hidden_count = 0;
 #endif
   uid_t  euid;
+  char hashbuf[KEYBUF_SIZE];
 
   SL_ENTER(_("sh_readconf_read"));
 
@@ -194,15 +387,21 @@ int sh_readconf_read (void)
     {
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_D_START);
 
-      fd = sh_forward_req_file(_("CONF"));
+      fd = sh_xfer_request_file(_("CONF"));
 
       if (!SL_ISERROR(fd))
-	local_file = 0;
+	{
+	  local_file = 0;
+	}
       else if (sh.flag.checkSum != SH_CHECK_INIT)
-	aud_exit (FIL__, __LINE__, EXIT_FAILURE);
+	{
+	  sh_error_handle((-1), FIL__, __LINE__, 0, MSG_TCP_FBAD);
+	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
+	}
       else
 	{
-	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_D_FAIL);
+	  sh_error_handle(SH_ERR_WARN, FIL__, __LINE__, 0, MSG_TCP_FBAD);
+	  sh_error_handle ((-1), FIL__, __LINE__, fd, MSG_D_FAIL);
 	  local_file = 1;
 	  local_flag = 'I';
 	}
@@ -227,13 +426,14 @@ int sh_readconf_read (void)
 			   );
 	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
 	}
-      if (SL_ISERROR(fd = sl_open_read(file_path('C',local_flag),SL_YESPRIV)))
+      if (SL_ISERROR(fd = sl_open_read(FIL__, __LINE__, 
+				       file_path('C',local_flag),SL_YESPRIV)))
 	{
 	  sl_get_euid(&euid);
 	  dlog(1, FIL__, __LINE__, 
-	       _("Could not open the local configuration file for reading because\nof the following error: %s\nIf this is a permission problem, you need to change file permissions\nto make the file readable for the effective UID: %d\n"), 
-	       sl_get_errmsg(), (int) euid);
-	  sh_error_handle ((-1), FIL__, __LINE__, EACCES, MSG_NOACCESS, 
+	       _("Could not open the local configuration file for reading because\nof the following error: %s (errnum = %ld)\nIf this is a permission problem, you need to change file permissions\nto make the file readable for the effective UID: %d\n"), 
+	       sl_get_errmsg(), fd, (int) euid);
+	  sh_error_handle ((-1), FIL__, __LINE__, fd, MSG_NOACCESS, 
 			   (long) euid, 
 			   ( (NULL == file_path('C', local_flag)) 
 			     ? _("(null)") : file_path('C', local_flag) )
@@ -244,17 +444,22 @@ int sh_readconf_read (void)
 
   /* Compute the checksum of the open file.
    */
-  tiger_fd = fd;
   sl_strlcpy(sh.conf.hash, 
-	     sh_tiger_hash(file_path('C',local_flag),TIGER_FD, 0),
+	     sh_tiger_hash(file_path('C',local_flag), fd, TIGER_NOLIM, 
+			   hashbuf, sizeof(hashbuf)),
 	     KEY_LEN+1);
   sl_rewind (fd);
 
+  line_in = SH_ALLOC(SH_LINE_IN);
+
 #if defined(SH_STEALTH) && !defined(SH_STEALTH_MICRO)
     /* extract the data and copy to temporary file
      */
   fdTmp = open_tmp(); 
-  while ( sh_unix_getline_stealth (fd, line_in, 512) > 0) {
+
+  sh_unix_getline_stealth (0, NULL, 0); /* initialize */
+
+  while ( sh_unix_getline_stealth (fd, line_in, SH_LINE_IN-2) > 0) {
     hidden_count++;
     if (line_in[0] == '\n')
       {
@@ -271,7 +476,7 @@ int sh_readconf_read (void)
     if (0 == sl_strncmp(line_in, _("[EOF]"), 5))
       break;
 #endif
-    if (hidden_count > 4096)  /* arbitrary safeguard */
+    if (hidden_count > 1048576)  /* arbitrary safeguard, 1024*1024 */
       break;
   }
   sl_close(fd);
@@ -279,17 +484,32 @@ int sh_readconf_read (void)
   sl_rewind (fd);
 #endif
 
-  /* The system type, release, and machine.
+#if defined(WITH_GPG) || defined(WITH_PGP)
+
+  /* extract the data and copy to temporary file
    */
-  sprintf(myident, _("%s:%s:%s"),                  /* known to fit  */
-	  sh.host.system, sh.host.release, sh.host.machine);
+  fdGpg = sh_gpg_extract_signed(fd);
+
+  sl_close(fd);
+  fd = fdGpg;
+
+  /* Validate signature of open file.
+   */
+  if (0 != sh_gpg_check_sign (fd, SIG_CONF))
+    {
+      SH_FREE(line_in);
+      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EXIT_ABORT1, sh.prg_name);
+      aud_exit (FIL__, __LINE__, EXIT_FAILURE);
+    }
+  sl_rewind (fd);
+#endif
 
 
   /* ---  Start reading lines.  ---
    */
   conf_line = 0;
 
-  while ( sh_unix_getline (fd, line_in, 512) > 0) {
+  while ( sh_unix_getline (fd, line_in, SH_LINE_IN-2) > 0) {
 
     ++conf_line;
 
@@ -327,6 +547,7 @@ int sh_readconf_read (void)
 		 _("There seems to be more than one signed message in the configuration\nfile. Please make sure there is only one signed message.\n"));
 	    sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EXIT_ABORT1,
 			     sh.prg_name);
+	    SH_FREE(line_in);
 	    aud_exit (FIL__, __LINE__,EXIT_FAILURE);
 	  }
       }
@@ -334,13 +555,8 @@ int sh_readconf_read (void)
 
     /* Skip leading white space.
      */
-    while (*line)
-      {
-	line_int = *line;
-	if (!isspace(line_int))
-	  break;
-	++line;
-      }
+    while (isspace((int)*line)) ++line;
+
 
     /* Skip header etc. 
      */
@@ -354,26 +570,80 @@ int sh_readconf_read (void)
     while( isspace((int) *tmp ) && tmp >= line ) *tmp-- = '\0';
 
 
+    /* ---  an @host/@if/$system directive -------------- */
+
+    if (line[0] == '@' || (line[0] == '!' && line[1] == '@') || 
+	line[0] == '$' || (line[0] == '!' && line[1] == '$'))
+      {
+	if (sh_readconf_is_end(line))
+	  {
+	    if (0 == cond_depth) {
+	      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EINVALD,
+			       _("config file"), 
+			       (long) conf_line);
+	    }
+	    else {
+	      if (cond_excl == cond_depth)
+		cond_excl = 0;
+	      --cond_depth;
+	    }
+	  }
+	else if (sh_readconf_is_else(line))
+	  {
+	    if (0 == cond_depth) {
+	      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EINVALD,
+			       _("config file"), 
+			       (long) conf_line);
+	    }
+	    else if (cond_excl == cond_depth) {
+	      cond_excl = 0;
+	    }
+	    else if (cond_excl == 0) {
+	      cond_excl = cond_depth;
+	    }
+	  }
+	else
+	  {
+	    if (sh_readconf_cond_match(line, conf_line)) {
+	      ++cond_depth;
+	    }
+	    else {
+	      ++cond_depth;
+	      if (cond_excl == 0)
+		cond_excl = cond_depth;
+	    }
+	  }
+	continue;
+      }
+
+    /* fprintf(stderr, "%d %s\n", cond_excl, line); */
+
+    /****************************************************
+     *
+     * Only carry on if this section is intended for us
+     *
+     ****************************************************/
+    
+    if (cond_excl != 0) {
+      continue;
+    }
+
     /* -------  starts a section  ------------  */
     
-    if (line[0] == '['                       && 
-	some_other_host[seen_host] == 0      &&
-	some_other_system[seen_system] == 0) 
+    else if (line[0] == '[')
       { 
 	read_mode = SH_SECTION_NONE;
 
-	if (sl_strncmp (line,  _("[EOF]"), 
-			  5) == 0)
-	  {
-	    goto nopel;
-	  }
+	if (0 == sl_strncasecmp (line,  _("[EOF]"), 5)) {
+	  goto nopel;
+	}
 
 	i = 0;
 
 	while (tab_ListSections[i].name != 0)
 	  {
-	    if (sl_strncmp (line, _(tab_ListSections[i].name), 
-			    sl_strlen(tab_ListSections[i].name)) == 0)
+	    if (sl_strncasecmp (line, _(tab_ListSections[i].name), 
+				sl_strlen(tab_ListSections[i].name)) == 0)
 	      { 
 		read_mode = tab_ListSections[i].type;
 		break;
@@ -386,8 +656,8 @@ int sh_readconf_read (void)
 	  {
 	    for (modnum = 0; modList[modnum].name != NULL; ++modnum) 
 	      {
-		if (0 == sl_strncmp (line,  _(modList[modnum].conf_section),
-				     sl_strlen(modList[modnum].conf_section)) )
+		if (0 == sl_strncasecmp (line, _(modList[modnum].conf_section),
+					 sl_strlen(modList[modnum].conf_section)) )
 		  read_mode = SH_SECTION_OTHER;
 	      }
 	  }
@@ -399,151 +669,24 @@ int sh_readconf_read (void)
 	  }
       } 
 
-    /* ---  an @host directive -------------- */
-
-    else if (line[0] == '@' || (line[0] == '!' && line[1] == '@')) 
-      {
-	if (line[0] == '!')
-	  {
-	    lptr   = &line[2];
-	    invert = 1;
-	  }
-	else
-	  {
-	    lptr   = &line[1];
-	    invert = 0;
-	  }
-
-	if (sl_strncmp (lptr, _("end"), 3) == 0)
-	  {
-	    if (0 == seen_host)
-	      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EINVALD,
-			       _("config file"), 
-			       (long) conf_line);
-
-	    else if (host_int == 0)
-	      {
-		/* end  of an @host directive 
-		 */
-		some_other_host[seen_host] = 0;
-		--seen_host;
-		seen_host = (seen_host < 0) ? 0 : seen_host;
-	      }
-
-	    else 
-	      {
-		--host_int;
-		host_int = (host_int < 0) ? 0 : host_int;
-	      }
-	  }
-	else if (some_other_host[seen_host] == 0      &&
-		 some_other_system[seen_system] == 0  && 
-		 seen_host < 15)
-	  {
-	    if  (sl_strncmp (lptr,  sh.host.name, length) == 0
-#ifdef HAVE_REGEX_H
-		 || sh_util_regcmp (lptr, sh.host.name) == 0
-#endif
-		 )
-	      {
-		/* if match and '@',  set some_other_host = 0;
-		 * if match and '!@', set some_other_host = 1;
-		 */
-		++seen_host;
-		some_other_host[seen_host] = invert; 
-	      }
-	    else
-	      {
-		/* if no match and '@',  set some_other_host = 1;
-		 * if no match and '!@', set some_other_host = 0;
-		 */
-		++seen_host;
-		some_other_host[seen_host] = (invert == 0) ? 1 : 0;
-	      }
-	  }
-	else
-	  ++host_int;
-      } 
-
     /* ---  an %schedule directive ------------ */
 
     else if (line[0] == '%' || (line[0] == '!' && line[1] == '%')) 
       {
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
-	if (line[0] == '!' && 0 == sl_strcmp(&(line[2]), _("SCHEDULE_TWO")))
+	if (line[0] == '!' && 0 == sl_strcasecmp(&(line[2]), _("SCHEDULE_TWO")))
 	  set_dirList(1);
-	else if (0 == sl_strcmp(&(line[1]), _("SCHEDULE_TWO")))
+	else if (0 == sl_strcasecmp(&(line[1]), _("SCHEDULE_TWO")))
 	  set_dirList(2);
 #else
 	;
 #endif
       }
-	  
-    /* ---  an $system directive -------------- */
-
-    else if (line[0] == '$' || (line[0] == '!' && line[1] == '$')) 
-      {
-	if (line[0] == '!')
-	  {
-	    lptr   = &line[2];
-	    invert = 1;
-	  }
-	else
-	  {
-	    lptr   = &line[1];
-	    invert = 0;
-	  }
-
-	if (sl_strncmp (lptr, _("end"), 3) == 0)
-	  {
-	    if (0 == seen_system)
-	      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EINVALD,
-			       _("config file"),
-			       (long) conf_line);
-
-	    else if (sys_int == 0)
-	      {
-		/* end  of an $system directive 
-		 */
-		some_other_system[seen_system] = 0;
-		--seen_system;
-		seen_system = (seen_system < 0) ? 0 : seen_system;
-	      }
-	    else 
-	      {
-		--sys_int;
-		sys_int = (sys_int < 0) ? 0 : sys_int;
-	      }
-	  }
-	else if (some_other_host[seen_host] == 0      &&
-		 some_other_system[seen_system] == 0  && 
-		 seen_system < 15)
-	  {
-	    if  (sl_strncmp (lptr,  myident, sl_strlen(myident)) == 0
-#ifdef HAVE_REGEX_H
-		 || sh_util_regcmp (lptr, myident) == 0
-#endif
-		 )
-	      {
-		++seen_system;
-		some_other_system[seen_system] = invert;
-	      }
-	    else
-	      {
-		++seen_system;
-		some_other_system[seen_system] = (invert == 0) ? 1 : 0;
-	      }
-	  }
-	else
-	  ++sys_int;
-      }
 
     /* ------  no new section -------------- */
 
 
-    else if (some_other_host[seen_host] == 0          && 
-	     some_other_system[seen_system] == 0      && 
-	     read_mode != SH_SECTION_NONE) 
+    else if (read_mode != SH_SECTION_NONE)
       { 
 	if (0 != sh_readconfig_line (line))
 	  {
@@ -551,35 +694,26 @@ int sh_readconf_read (void)
 			     (long) conf_line);
 	  }
       }
-
-  }
+  } /* while getline() */
 
  nopel:
 	   
-  if (0 != seen_host || 0 != seen_system)
+  if (0 != cond_depth)
     sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EINVALDD,
 		     _("config file"), 
 		     (long) conf_line);
 
-#if defined(WITH_GPG) || defined(WITH_PGP)
-  /* Validate signature of open file.
-   */
-  sl_rewind (fd);
-  sh_error_only_stderr (S_FALSE);
-  if (0 != sh_gpg_check_sign (fd, 0, 1))
-    aud_exit (FIL__, __LINE__, EXIT_FAILURE);
-#endif
-
   sl_close (fd);
 
   sh_error_fixup();
 
   read_mode = SH_SECTION_NONE; /* reset b/o sighup reload */
 
+  SH_FREE(line_in);
   SL_RETURN( 0, _("sh_readconf_read"));
 }
 
-int sh_readconf_set_path (char * which, char * what)
+int sh_readconf_set_path (char * which, const char * what)
 {
   int len;
   SL_ENTER( _("sh_readconf_set_path"));
@@ -620,17 +754,17 @@ int sh_readconf_set_path (char * which, char * what)
   SL_RETURN( 0, _("sh_readconf_set_path"));
 }
 
-int sh_readconf_set_database_path (char * what)
+int sh_readconf_set_database_path (const char * what)
 {
   return (sh_readconf_set_path(sh.data.path, what));
 }
 
-int sh_readconf_set_logfile_path (char * what)
+int sh_readconf_set_logfile_path (const char * what)
 {
   return (sh_readconf_set_path(sh.srvlog.name, what));
 }
 
-int sh_readconf_set_lockfile_path (char * what)
+int sh_readconf_set_lockfile_path (const char * what)
 {
   return( sh_readconf_set_path(sh.srvlog.alt, what));
 }
@@ -644,7 +778,7 @@ typedef enum {
 } ShTimerItem;
  
     
-int sh_readconf_setTime (char * str, ShTimerItem what)
+int sh_readconf_setTime (const char * str, ShTimerItem what)
 {
   unsigned long i = atoi (str);
 
@@ -673,17 +807,17 @@ int sh_readconf_setTime (char * str, ShTimerItem what)
     }
 }
 
-int sh_readconf_setMailtime (char * c)
+int sh_readconf_setMailtime (const char * c)
 {
   return sh_readconf_setTime (c, SET_MAILTIME);
 }
 
-int sh_readconf_setFiletime (char * c)
+int sh_readconf_setFiletime (const char * c)
 {
   return sh_readconf_setTime (c, SET_FILETIME);
 }
 
-int sh_readconf_set_nice (char * c)
+int sh_readconf_set_nice (const char * c)
 {
   long val;
 
@@ -702,8 +836,21 @@ int sh_readconf_set_nice (char * c)
   SL_RETURN((0), _("sh_readconf_set_nice"));
 }
 
+
+#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE) 
+static int sh_readconf_set_delay (const char * c)
+{
+  unsigned long i = atol (c);
+
+  if (i  > INT_MAX)
+    return -i;
+  sh.delayload = (int) i;
+  return 0;
+}
+#endif
+
 #ifdef FANCY_LIBCAP
-int sh_readconf_setCaps(char * c)
+int sh_readconf_setCaps(const char * c)
 {
   int i;
   SL_ENTER(_("sh_readconf_setCaps"));
@@ -714,26 +861,29 @@ int sh_readconf_setCaps(char * c)
 #endif
 
 typedef struct _cfg_options {
-  char * optname;
+  const char * optname;
   ShSectionType   section;
   ShSectionType   alt_section;
-  int (*func)(char * opt);
+  int (*func)(const char * opt);
 } cfg_options;
 
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
-extern int sh_set_schedule_one(char * str);
-extern int sh_set_schedule_two(char * str);
+extern int sh_set_schedule_one(const char * str);
+extern int sh_set_schedule_two(const char * str);
+extern int sh_set_silent_full (const char * str);
 #endif
 #if defined (SH_WITH_SERVER)
-extern int sh_socket_use (char * c);
-extern int sh_socket_uid (char * c);
-extern int sh_socket_password (char * c);
+extern int sh_socket_use (const char * c);
+extern int sh_socket_uid (const char * c);
+extern int sh_socket_password (const char * c);
 #endif
 
 cfg_options ext_table[] = {
 #if defined(WITH_EXTERNAL)
   { N_("opencommand"),     SH_SECTION_EXTERNAL, SH_SECTION_NONE,  
     sh_ext_setcommand },
+  { N_("closecommand"),    SH_SECTION_EXTERNAL, SH_SECTION_NONE,  
+    sh_ext_close_command },
   { N_("setcommandline"),  SH_SECTION_EXTERNAL, SH_SECTION_NONE,  
     sh_ext_add_argv },
   { N_("setchecksum"),     SH_SECTION_EXTERNAL, SH_SECTION_NONE,  
@@ -783,6 +933,7 @@ cfg_options ext_table[] = {
     set_enter_wrapper },
 #endif
 
+
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
   { N_("dir"),            SH_SECTION_ATTRIBUTES, SH_SECTION_NONE, 
     sh_files_pushdir_attr },
@@ -817,51 +968,94 @@ cfg_options ext_table[] = {
     sh_files_pushdir_user1 },
   { N_("file"),           SH_SECTION_USER1,      SH_SECTION_NONE, 
     sh_files_pushfile_user1 },
+  { N_("dir"),            SH_SECTION_USER2,      SH_SECTION_NONE, 
+    sh_files_pushdir_user2 },
+  { N_("file"),           SH_SECTION_USER2,      SH_SECTION_NONE, 
+    sh_files_pushfile_user2 },
+  { N_("dir"),            SH_SECTION_USER3,      SH_SECTION_NONE, 
+    sh_files_pushdir_user3 },
+  { N_("file"),           SH_SECTION_USER3,      SH_SECTION_NONE, 
+    sh_files_pushfile_user3 },
+  { N_("dir"),            SH_SECTION_USER4,      SH_SECTION_NONE, 
+    sh_files_pushdir_user4 },
+  { N_("file"),           SH_SECTION_USER4,      SH_SECTION_NONE, 
+    sh_files_pushfile_user4 },
   { N_("dir"),            SH_SECTION_PRELINK,    SH_SECTION_NONE, 
     sh_files_pushdir_prelink },
   { N_("file"),           SH_SECTION_PRELINK,    SH_SECTION_NONE, 
     sh_files_pushfile_prelink },
 
-  { N_("ignoreadded"), SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("ignoreadded"),          SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_ignore_add_new },
-  { N_("ignoremissing"), SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("ignoremissing"),        SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_ignore_add_del },
+  { N_("ignoremodified"),       SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_ignore_add_mod },
+
+  { N_("skipchecksum"),         SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_restrict_define },
+  { N_("filetype"),             SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_restrict_add_ftype },
+
 
   { N_("filecheckscheduleone"), SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_set_schedule_one },
   { N_("filecheckscheduletwo"), SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_set_schedule_two },
 
-  { N_("usehardlinkcheck"),   SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("usehardlinkcheck"),     SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_files_check_hardlinks },
-  { N_("hardlinkoffset"),     SH_SECTION_MISC,   SH_SECTION_NONE,
+  { N_("usersrccheck"),         SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_files_use_rsrc },
+  { N_("hardlinkoffset"),       SH_SECTION_MISC,   SH_SECTION_NONE,
     sh_files_hle_reg },
-  { N_("addokchars"),         SH_SECTION_MISC,   SH_SECTION_NONE,
+#if defined(USE_XATTR)
+  { N_("useselinuxcheck"),      SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_unix_setcheckselinux },
+#endif
+#if defined(USE_ACL)
+  { N_("useaclcheck"),          SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_unix_setcheckacl },
+#endif
+  { N_("loosedircheck"),        SH_SECTION_MISC,   SH_SECTION_NONE,
+    sh_hash_loosedircheck },
+  { N_("addokchars"),           SH_SECTION_MISC,   SH_SECTION_NONE,
     sh_util_obscure_ok },
-  { N_("setrecursionlevel"),  SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("filenamesareutf8"),     SH_SECTION_MISC,   SH_SECTION_NONE,
+    sh_util_obscure_utf8 },
+  { N_("setrecursionlevel"),    SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_files_setrecursion },
-  { N_("checksumtest"),       SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("checksumtest"),         SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_util_setchecksum },
-  { N_("reportonlyonce"),     SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("reportonlyonce"),       SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_files_reportonce },
-  { N_("reportfulldetail"),   SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("reportfulldetail"),     SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_files_fulldetail },
-  { N_("uselocaltime"),       SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("uselocaltime"),         SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_unix_uselocaltime },
 
-  { N_("setnicelevel"),   SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("setnicelevel"),         SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_readconf_set_nice },
 
+  { N_("startuploaddelay"),     SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_readconf_set_delay },
+
 #if defined(FANCY_LIBCAP)
-  { N_("usecaps"),        SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("usecaps"),              SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_readconf_setCaps },
 #endif
 
-  { N_("setiolimit"),   SH_SECTION_MISC,   SH_SECTION_NONE, 
+  { N_("reportcheckflags"),     SH_SECTION_MISC,   SH_SECTION_NONE, 
+    set_report_checkflags },
+
+  { N_("setdropcache"),         SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sl_set_drop_cache },
+
+  { N_("setiolimit"),           SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_unix_set_io_limit },
 
   { N_("versionstring"),        SH_SECTION_MISC,   SH_SECTION_NONE,
-    sh_hash_version_string },
+    sh_dbIO_version_string },
 
   { N_("digestalgo"),           SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_tiger_hashtype },
@@ -890,6 +1084,15 @@ cfg_options ext_table[] = {
   { N_("redefuser1"),           SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_files_redef_user1 },
 
+  { N_("redefuser2"),           SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_files_redef_user2 },
+
+  { N_("redefuser3"),           SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_files_redef_user3 },
+
+  { N_("redefuser4"),           SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_files_redef_user4 },
+
   { N_("redefprelink"),         SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_files_redef_prelink },
 
@@ -898,6 +1101,10 @@ cfg_options ext_table[] = {
     sh_prelink_set_path },
   { N_("setprelinkchecksum"),   SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_prelink_set_hash },
+
+  { N_("setfullsilent"),        SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_set_silent_full },
+
   /* client or standalone
    */
 #endif
@@ -914,36 +1121,49 @@ cfg_options ext_table[] = {
   { N_("setsocketpassword"),   SH_SECTION_SRV,  SH_SECTION_MISC, 
     sh_socket_password },
   { N_("setstripdomain"),      SH_SECTION_SRV,  SH_SECTION_MISC, 
-    sh_forward_set_strip },
+    sh_xfer_set_strip },
   { N_("useseparatelogs"),     SH_SECTION_SRV,  SH_SECTION_MISC, 
     set_flag_sep_log },
   { N_("setchrootdir"),        SH_SECTION_SRV,  SH_SECTION_MISC, 
     sh_unix_set_chroot },
   { N_("setclienttimelimit"),  SH_SECTION_SRV,  SH_SECTION_MISC, 
-    sh_forward_set_time_limit },
+    sh_xfer_set_time_limit },
+  { N_("setconnectiontimeout"),SH_SECTION_SRV,  SH_SECTION_MISC, 
+    sh_xfer_set_timeout },
   { N_("useclientseverity"),   SH_SECTION_SRV,  SH_SECTION_MISC, 
-  sh_forward_use_clt_sev },
+  sh_xfer_use_clt_sev },
   { N_("useclientclass"),      SH_SECTION_SRV,  SH_SECTION_MISC, 
-  sh_forward_use_clt_class },
+  sh_xfer_use_clt_class },
   { N_("severitylookup"),      SH_SECTION_SRV,  SH_SECTION_MISC, 
-  sh_forward_lookup_level },
+  sh_xfer_lookup_level },
   { N_("setclientfromaccept"), SH_SECTION_SRV,  SH_SECTION_MISC, 
     set_socket_peer },
   { N_("setserverport"),       SH_SECTION_SRV,  SH_SECTION_MISC, 
-    sh_forward_set_port },
+    sh_xfer_set_port },
   { N_("setserverinterface"),  SH_SECTION_SRV,  SH_SECTION_MISC, 
-    sh_forward_set_interface },
+    sh_xfer_set_interface },
   { N_("client"),              SH_SECTION_CLIENTS,           SH_SECTION_NONE, 
-    sh_forward_register_client },
+    sh_xfer_register_client },
 #endif
 
-#ifdef SH_WITH_CLIENT
+#if defined(SH_WITH_CLIENT) || defined(SH_WITH_SERVER)
   { N_("exportseverity"),      SH_SECTION_LOG,  SH_SECTION_NONE, 
     sh_error_setexport },
   { N_("exportclass"),         SH_SECTION_LOG,  SH_SECTION_NONE, 
     sh_error_export_mask },
+#if defined(SH_WITH_SERVER)
+  { N_("setlogserver"),        SH_SECTION_SRV,  SH_SECTION_MISC, 
+    sh_xfer_set_logserver },
+#else
   { N_("setlogserver"),        SH_SECTION_CLT,  SH_SECTION_MISC, 
-    sh_forward_setlogserver },
+    sh_xfer_set_logserver },
+  { N_("setthrottle"),         SH_SECTION_CLT,  SH_SECTION_MISC, 
+    sh_xfer_set_throttle_delay},
+  { N_("setdeltaretrycount"),   SH_SECTION_CLT,  SH_SECTION_MISC, 
+    set_delta_retry_count},
+  { N_("setdeltaretryinterval"),SH_SECTION_CLT,  SH_SECTION_MISC, 
+    set_delta_retry_interval},
+#endif
 #endif
   { N_("setfilechecktime"),  SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_readconf_setFiletime },
@@ -959,16 +1179,30 @@ cfg_options ext_table[] = {
     sh_readconf_setMailtime },
   { N_("setmailnum"),        SH_SECTION_MAIL,  SH_SECTION_MISC, 
     sh_mail_setNum },
-  { N_("setmailaddress"),    SH_SECTION_MAIL,  SH_SECTION_MISC, 
-    sh_mail_setaddress },
   { N_("setmailrelay"),      SH_SECTION_MAIL,  SH_SECTION_MISC, 
     sh_mail_set_relay },
+  { N_("setmailport"),       SH_SECTION_MAIL,  SH_SECTION_MISC,
+    sh_mail_set_port },
   { N_("mailsingle"),        SH_SECTION_MAIL,  SH_SECTION_MISC, 
     sh_mail_setFlag },
   { N_("mailsubject"),       SH_SECTION_MAIL,  SH_SECTION_MISC, 
     set_mail_subject },
   { N_("setmailsender"),     SH_SECTION_MAIL,  SH_SECTION_MISC, 
     sh_mail_set_sender },
+  { N_("setmailalias"),       SH_SECTION_MAIL,  SH_SECTION_MISC, 
+    sh_nmail_add_alias },
+  { N_("setmailaddress"),    SH_SECTION_MAIL,  SH_SECTION_MISC, 
+    sh_nmail_add_recipient },
+  { N_("closeaddress"),      SH_SECTION_MAIL,  SH_SECTION_MISC, 
+    sh_nmail_close_recipient },
+  { N_("setaddrseverity"),   SH_SECTION_MAIL,  SH_SECTION_MISC,
+    sh_nmail_set_severity },
+  { N_("setmailfilternot"),  SH_SECTION_MAIL,  SH_SECTION_MISC,
+    sh_nmail_add_not },
+  { N_("setmailfilterand"),  SH_SECTION_MAIL,  SH_SECTION_MISC,
+    sh_nmail_add_and },
+  { N_("setmailfilteror"),   SH_SECTION_MAIL,  SH_SECTION_MISC,
+    sh_nmail_add_or },
 #endif
   { N_("setbindaddress"),    SH_SECTION_MISC,  SH_SECTION_NONE,
     sh_calls_set_bind_addr },
@@ -1000,7 +1234,6 @@ cfg_options ext_table[] = {
     sh_error_set_prelude },
   { N_("preludeclass"),      SH_SECTION_LOG,   SH_SECTION_NONE, 
     sh_error_prelude_mask },
-#ifdef HAVE_LIBPRELUDE_9
   { N_("preludeprofile"),    SH_SECTION_MISC,  SH_SECTION_NONE,
     sh_prelude_set_profile },
   { N_("preludemaptoinfo"),    SH_SECTION_MISC,  SH_SECTION_NONE,
@@ -1011,7 +1244,6 @@ cfg_options ext_table[] = {
     sh_prelude_map_medium },
   { N_("preludemaptohigh"),    SH_SECTION_MISC,  SH_SECTION_NONE,
     sh_prelude_map_high },
-#endif
 #endif
 
   { N_("logcalls"),          SH_SECTION_LOG,   SH_SECTION_NONE, 
@@ -1023,6 +1255,11 @@ cfg_options ext_table[] = {
   { N_("setconsole"),        SH_SECTION_MISC,  SH_SECTION_NONE, 
     sh_log_set_console },
 
+  { N_("setreportfile"),     SH_SECTION_MISC,  SH_SECTION_NONE, 
+    sh_efile_path },
+  { N_("setreportgroup"),    SH_SECTION_MISC,  SH_SECTION_NONE, 
+    sh_efile_group },
+
 #ifdef WITH_MESSAGE_QUEUE
   { N_("messagequeueactive"),SH_SECTION_MISC,  SH_SECTION_NONE, 
     enable_msgq },
@@ -1046,9 +1283,20 @@ cfg_options ext_table[] = {
   { N_("syslogfacility"),    SH_SECTION_LOG,   SH_SECTION_MISC, 
     sh_log_set_facility },
 
+  { N_("syslogmapstampto"),    SH_SECTION_LOG,   SH_SECTION_MISC, 
+    sh_log_set_stamp_priority },
+
   { N_("mactype"),     SH_SECTION_MISC,  SH_SECTION_NONE, 
     sh_util_sigtype },
 
+  { N_("avoidblock"),     SH_SECTION_MISC,  SH_SECTION_NONE, 
+    sh_calls_set_sub },
+
+#ifdef SCREW_IT_UP
+  { N_("setsigtrapmaxduration"),        SH_SECTION_MISC,  SH_SECTION_MISC, 
+    sh_sigtrap_max_duration_set },
+#endif
+
   { NULL,    0,   0,  NULL}
 };
 
@@ -1056,8 +1304,10 @@ cfg_options ext_table[] = {
 
 
 static int sh_readconfig_line (char * line)
-{    
-  char * c;
+{
+  char * key;
+  const char * value;
+  char * value_dup;
   char * tmp;
   int    i;
   int    good_opt = -1;
@@ -1066,7 +1316,17 @@ static int sh_readconfig_line (char * line)
   int    modnum, modkey;
 #endif
 
-  static char  *ident[] = {
+  static const char  *dummy = N_("dummy");
+
+  static const char  *closing[] = {
+    N_("closecommand"),
+    N_("closeaddress"),
+    N_("logmonendgroup"),
+    N_("logmonendhost"),
+    NULL
+  };
+
+  static const char  *ident[] = {
     N_("severityreadonly"),
     N_("severitylogfiles"),
     N_("severitygrowinglogs"),
@@ -1078,6 +1338,9 @@ static int sh_readconfig_line (char * line)
     N_("severitynames"),
     N_("severityuser0"),
     N_("severityuser1"),
+    N_("severityuser2"),
+    N_("severityuser3"),
+    N_("severityuser4"),
     N_("severityprelink"),
     NULL
   };
@@ -1094,34 +1357,60 @@ static int sh_readconfig_line (char * line)
     SH_ERR_T_NAME,       
     SH_ERR_T_USER0,       
     SH_ERR_T_USER1,       
+    SH_ERR_T_USER2,       
+    SH_ERR_T_USER3,       
+    SH_ERR_T_USER4,       
     SH_ERR_T_PRELINK,       
   };
     
   SL_ENTER(_("sh_readconf_line"));
 
+  /* convert to lowercase                              */
+
+  tmp = line;
+  while (*tmp != '=' && *tmp != '\0')
+    {
+      *tmp = tolower( (int) *tmp);
+      ++tmp;
+    }
+
+  key = line;
+
   /* interpret line                                    */
 
-  c = strchr(line, '=');
-  if (c == NULL || (*c) == '\0')
+  value = strchr(line, '=');
+
+  if (value == NULL || (*value) == '\0')
     {
-      if (line != NULL)
+      if (key != NULL)
 	{
+	  i = 0;
+	  while (closing[i] != NULL) 
+	    {
+	      if (sl_strncmp(key,_(closing[i]),sl_strlen(closing[i])-1) == 0)
+		{
+		  value = dummy;
+		  goto ok_novalue;
+		}
+	      ++i;
+	    }
+
 	  TPT(( 0, FIL__, __LINE__, _("msg=<ConfigFile: not key=value: %s>\n"),
 		line));
 	}
       SL_RETURN(good_opt, _("sh_readconf_line"));
     }
   else
-    ++c;
+    ++value;
 
   /* skip leading whitespace
    */
-  while ((*c) == ' ' || (*c) == '\t')
-    ++c;
+  while ((*value) == ' ' || (*value) == '\t')
+    ++value;
 
-  if ((*c) == '\0')     /* no value                    */
+  if ((*value) == '\0')     /* no value                    */
     {
-      if (line != NULL)
+      if (key != NULL)
 	{
 	  TPT(( 0, FIL__, __LINE__, _("msg=<ConfigFile: not key=value: %s>\n"),
 		line));
@@ -1129,20 +1418,27 @@ static int sh_readconfig_line (char * line)
       SL_RETURN(good_opt, _("sh_readconf_line"));
     }
 
-  /* convert to lowercase                              */
-
-  tmp = line;
-  while (*tmp != '=')
-    {
-      *tmp = tolower( (int) *tmp);
-      ++tmp;
-    }
+ ok_novalue:
 
   if (!sl_is_suid())
     {
       TPT(( 0, FIL__, __LINE__, _("msg=<ConfigFile: %s>\n"), line));
     }
 
+  /* Expand shell expressions. This return allocated memory which we must free.
+   * If !defined(SH_EVAL_SHELL), this will reduce to a strdup.
+   */
+  value_dup = sh_readconf_expand_value(value);
+
+  if (!value_dup || (*value_dup) == '\0')
+    {
+      TPT(( 0, FIL__, __LINE__, 
+	    _("msg=<ConfigFile: empty after shell expansion: %s>\n"),
+	    line));
+      SL_RETURN(good_opt, _("sh_readconf_line"));
+    }
+
+  value = value_dup;
 
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE) 
   if      (read_mode == SH_SECTION_OTHER) 
@@ -1152,14 +1448,14 @@ static int sh_readconfig_line (char * line)
 	  for (modkey = 0; modList[modnum].conf_table[modkey].the_opt != NULL; 
 	       ++modkey) 
 	    {
-	      if (sl_strncmp (line,   
+	      if (sl_strncmp (key,
 			      _(modList[modnum].conf_table[modkey].the_opt),
 			      sl_strlen(modList[modnum].conf_table[modkey].the_opt) ) == 0)
 		{
 		  good_opt = 0;
-		  if (0 != modList[modnum].conf_table[modkey].func(c))
+		  if (0 != modList[modnum].conf_table[modkey].func(value))
 		    sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EINVALS,
-				     _(modList[modnum].conf_table[modkey].the_opt), c);
+				     _(modList[modnum].conf_table[modkey].the_opt), value);
 		  if (!sl_is_suid())
 		    {
 		      TPT(( 0, FIL__, __LINE__, 
@@ -1179,10 +1475,10 @@ static int sh_readconfig_line (char * line)
     {
       i = 0;
       while (ident[i] != NULL) {
-	if (sl_strncmp (line, _(ident[i]), sl_strlen(ident[i])-1) == 0)
+	if (sl_strncmp (key, _(ident[i]), sl_strlen(ident[i])) == 0)
 	  {
 	    good_opt = 0;
-	    sh_error_set_iv (identnum[i], c);
+	    sh_error_set_iv (identnum[i], value);
 	    break;
 	  }
 	++i;
@@ -1195,20 +1491,22 @@ static int sh_readconfig_line (char * line)
 	{
 	  if ((ext_table[i].section == read_mode || 
 	       ext_table[i].alt_section == read_mode) &&
-	      sl_strncmp (line, _(ext_table[i].optname), 
+	      sl_strncmp (key, _(ext_table[i].optname), 
 			  sl_strlen(ext_table[i].optname)) == 0)
 	    {
 	      good_opt = 0;
-	      if (0 != ext_table[i].func (c))
+	      if (0 != ext_table[i].func (value))
 		sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EINVALS,
-				 _(ext_table[i].optname), c);
+				 _(ext_table[i].optname), value);
 	      break;
 	    }
 	  ++i;
 	}
     }
 
+  SH_FREE(value_dup);
+
   SL_RETURN(good_opt, _("sh_readconf_line"));
 }
-  
+
     
