@@ -337,6 +337,11 @@ static char * get_arg(char * str)
       e = strrchr(s, ')');
       if (e && (e != s))
 	{
+	  /* check that nothing follows */
+	  char * u = e;
+	  ++u; for (; *u && isspace((int)*u); ++u) /* nothing */;
+	  if (*u != '\0') return NULL;
+	  
 	  /* strip trailing space */
 	  *e = '\0'; --e;
 	  while ( (e != s) && isspace((int)*e) )
@@ -552,6 +557,11 @@ void Test_restrict (CuTest *tc) {
   CuAssertIntEquals(tc,0,res);
   CuAssertPtrNotNull(tc, sh_restrict_list);
 
+  strcpy(str, "size_exceeds(800); match_prefix(/foo), have_permission(0100)");
+  res = sh_restrict_define(str);
+  CuAssertIntEquals(tc,-1,res);
+  CuAssertPtrNotNull(tc, sh_restrict_list);
+  
   res = sh_restrict_this("/home/foo.mpg", 1000, 0755, 0);
   CuAssertIntEquals(tc,1,res);
 
