@@ -54,6 +54,7 @@
 
 #include "samhain.h"
 #include "sh_mem.h"
+#include "sh_error_min.h"
 
 /* TESTONLY */
 #else
@@ -142,6 +143,14 @@ int test_sched_int (sh_schedule_t * isched)
 #else
   tval = localtime(&now);
 #endif
+
+  if (!tval)
+    {
+      sh_error_handle ((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
+                       _("localime() failed"), _("test_sched_int") );
+      return 0;
+    }
+
   count = 0;
   for (i = 0; i < 5; ++i)
     {
@@ -334,7 +343,7 @@ int parse_sched (const char * ssched, sh_schedule_t * isched)
 
   len = strlen(ssched)+1;
 #ifdef TESTONLY
-  copy = malloc(len);                 /* testonly code */
+  copy = calloc(1,len);                 /* testonly code */
 #else
   copy = SH_ALLOC(len);
 #endif
