@@ -468,12 +468,12 @@ int  sh_initgroups(const char *user, gid_t gid)
 	char **m;
 	struct group group;
 
-	char * buff = malloc(GRP_BUFFER_SIZE_MALLOC);
+	char * buff = calloc(1,GRP_BUFFER_SIZE_MALLOC);
 
 	rv = -1;
 
 	/* We alloc space for 8 gids at a time. */
-	if (((group_list = (gid_t *) malloc(8*sizeof(gid_t *))) != NULL)
+	if (((group_list = calloc(8,sizeof(gid_t *))) != NULL)
 		&& ((grf = fopen(_PATH_GROUP, "r")) != NULL)
 		) {
 
@@ -998,15 +998,29 @@ static int __length_dotted(const unsigned char *data, int offset)
 	if (!data)
 		return -1;
 
-	while ((l = data[offset++])) {
+	do {
+
+	        if (offset < INT_MAX)
+	                offset++;
+		else
+		        return -1;
+
+		l = data[offset];
 
 		if ((l & 0xc0) == (0xc0)) {
-			offset++;
+		        if (offset < INT_MAX)
+			  offset++;
+			else
+			  return -1;
 			break;
 		}
 
-		offset += l;
-	}
+		if (offset <= (INT_MAX - l))
+		  offset += l;
+		else
+		  return -1;
+
+	} while (l);
 
 	return offset - orig_offset;
 }
@@ -1018,8 +1032,10 @@ static int __length_question(unsigned char *message, int offset)
 	i = __length_dotted(message, offset);
 	if (i < 0)
 		return i;
-
-	return i + 4;
+	if (i < (INT_MAX - 4))
+	  return i + 4;
+	else
+	  return -1;
 }
 
 /* Decode a dotted string from nameserver transport-level encoding.
@@ -1035,28 +1051,46 @@ static int __decode_dotted(const unsigned char *data, int offset,
 
 	if (!data)
 		return -1;
-
-	while ((l=data[offset++])) {
+	if ((offset < 0) || (offset > (PACKETSZ-1)))
+	        return -1;
+	while ((l=data[offset])) {
+	        if (offset < (PACKETSZ-1)) offset++;
+		else return -1;
 		if (measure)
-		    total++;
+		    { if (total < INT_MAX) total++; else return -1; }
 		if ((l & 0xc0) == (0xc0)) {
-			if (measure)
-				total++;
-			/* compressed item, redirect */
+		        if (measure)
+			  { if (total < INT_MAX) total++; else return -1; }
+		        /* compressed item, redirect */ 
 			offset = ((l & 0x3f) << 8) | data[offset];
+			if ((offset < 0) || (offset > (PACKETSZ-1)))
+			  return -1; 
 			measure = 0;
 			continue;
 		}
+	        
+		if (used >= (INT_MAX - l))
+		  return -1;
 
 		if ((used + l + 1) >= maxlen)
-			return -1;
+		  return -1; 
 
 		memcpy(dest + used, data + offset, l);
-		offset += l;
-		used += l;
-		if (measure)
-			total += l;
+		
+		if (offset <= ((PACKETSZ-1) - l))
+		  offset += l;
+		else
+		  return -1;
 
+		if (used <= (INT_MAX - l))
+		  used += l;
+		else
+		  return -1;
+		if (measure)
+		  { if (total <= (INT_MAX -l)) total += l; else return -1; }
+
+		if (used >= maxlen)
+		  return -1;
 		if (data[offset] != 0)
 			dest[used++] = '.';
 		else
@@ -1065,7 +1099,7 @@ static int __decode_dotted(const unsigned char *data, int offset,
 
 	/* The null byte must be counted too */
 	if (measure) {
-	    total++;
+	    if (total < INT_MAX) total++; else return -1; 
 	}
 
 	DPRINTF("Total decode len = %d\n", total);
@@ -1077,13 +1111,16 @@ static int __decode_answer(unsigned char *message, int offset,
 				  struct resolv_answer *a)
 {
 	char temp[256];
-	int i;
+	int i = 0;
 
 	i = __decode_dotted(message, offset, temp, sizeof(temp));
-	if (i < 0)
-		return i;
+	if (i < 0 || i > PACKETSZ)
+		return -1;
 
-	message += offset + i;
+	if (offset <= ((PACKETSZ - 10) - i))
+	  message += offset + i;
+	else
+	  return -1;
 
 	a->dotted = strdup(temp);
 	a->atype = (message[0] << 8) | message[1];
@@ -1100,7 +1137,14 @@ static int __decode_answer(unsigned char *message, int offset,
 
 	DPRINTF("i=%d,rdlength=%d\n", i, a->rdlength);
 
-	return i + RRFIXEDSZ + a->rdlength;
+	if (RRFIXEDSZ <= (INT_MAX - i))
+	  i += RRFIXEDSZ;
+	else
+	  return -1;
+	if (a->rdlength <= (INT_MAX - i))
+	  return i + a->rdlength;
+	else
+	  return -1;
 }
 
 
@@ -1147,6 +1191,8 @@ static int __encode_question(struct resolv_question *q,
 		return i;
 
 	dest += i;
+	if (maxlen < i)
+	  return -1;
 	maxlen -= i;
 
 	if (maxlen < 4)
@@ -1157,7 +1203,10 @@ static int __encode_question(struct resolv_question *q,
 	dest[2] = (q->qclass & 0xff00) >> 8;
 	dest[3] = (q->qclass & 0x00ff) >> 0;
 
-	return i + 4;
+	if (i <= (INT_MAX - 4))
+	  return i + 4;
+	else
+	  return -1;
 }
 
 
@@ -1176,8 +1225,8 @@ static int __dns_lookup(const char *name, int type, int nscount, char **nsip,
 	struct resolv_header h;
 	struct resolv_question q;
 	int retries = 0;
-	unsigned char * packet = malloc(PACKETSZ);
-	char *dns, *lookup = malloc(MAXDNAME);
+	unsigned char * packet = calloc(1,PACKETSZ);
+	char *dns, *lookup = calloc(1,MAXDNAME);
 	int variant = 0;
 	struct sockaddr_in sa;
 #ifdef __UCLIBC_HAS_IPV6__
@@ -1301,6 +1350,7 @@ static int __dns_lookup(const char *name, int type, int nscount, char **nsip,
 			goto again;
 		}
 
+		/* ok because we have checked that recv at least HFIXEDSZ */
 		__decode_header(packet, &h);
 
 		DPRINTF("id = %d, qr = %d\n", h.id, h.qr);
@@ -1334,6 +1384,8 @@ static int __dns_lookup(const char *name, int type, int nscount, char **nsip,
 			if (i < 0)
 				goto again;
 			pos += i;
+			if (pos >= PACKETSZ)
+			        goto again;
 		}
 		DPRINTF("Decoding answer at pos %d\n", pos);
 
@@ -1352,6 +1404,8 @@ static int __dns_lookup(const char *name, int type, int nscount, char **nsip,
 		    DPRINTF("skipping T_SIG %d\n", i);
 		    free(a->dotted);
 		    pos += i;
+		    if (pos >= PACKETSZ)
+		            goto again;
 		}
 
 		DPRINTF("Answer name = |%s|\n", a->dotted);
@@ -1640,6 +1694,7 @@ static int __open_nameservers(void)
  the_end:
 	; /* 'label at end of compound statement' */
 	SH_MUTEX_UNLOCK(resolv_lock);
+	/* cppcheck-suppress resourceLeak */
 	return 0;
 }
 
