@@ -77,10 +77,12 @@
 #include "sh_modules.h"
 #include "sh_suidchk.h"
 #include "sh_hash.h"
+#include "sh_dbIO.h"
 #include "sh_unix.h"
 #include "sh_files.h"
 #include "sh_schedule.h"
 #include "sh_calls.h"
+#include "zAVLTree.h"
 
 
 sh_rconf sh_suidchk_table[] = {
@@ -138,23 +140,46 @@ sh_rconf sh_suidchk_table[] = {
 static time_t  lastcheck         = (time_t) 0;
 static int     ShSuidchkActive   = S_TRUE;
 static time_t  ShSuidchkInterval = 7200;
-static long    ShSuidchkFps      = 0;
+static unsigned long    ShSuidchkFps      = 0;
 static int     ShSuidchkNosuid   = S_FALSE;
 static int     ShSuidchkYield    = S_FALSE;
 static int     ShSuidchkQEnable  = S_FALSE;
 static int     ShSuidchkQMethod  = SH_Q_CHANGEPERM;
 static int     ShSuidchkQDelete  = S_FALSE;
 static int     ShSuidchkSeverity = SH_ERR_SEVERE;
-static char *  ShSuidchkExclude  = NULL;
-static size_t  ExcludeLen        = 0;
 
 static time_t  FileLimNow        = 0;
 static time_t  FileLimStart      = 0;
-static long    FileLimNum        = 0;
-static long    FileLimTotal      = 0;
+static unsigned long    FileLimNum        = 0;
+static unsigned long    FileLimTotal      = 0;
 
 static sh_schedule_t * ShSuidchkSched = NULL;
 
+
+static zAVLTree *  ShSuidchkExclude  = NULL;
+static void sh_suid_exclude_free()
+{
+  zAVL_string_reset(ShSuidchkExclude);
+  ShSuidchkExclude  = NULL;
+  return;
+}
+static int sh_suid_exclude_add(const char * str)
+{
+  size_t len;
+  int    ret;
+  char * key = sh_util_strdup(str);
+
+  len = sl_strlen (key);
+  if (len && key[len-1] == '/')
+    {
+      key[len-1] = '\0';
+    }
+  ret = zAVL_string_set(&ShSuidchkExclude, key);
+  SH_FREE(key);
+  return ret;
+}
+
+
 static char *
 filesystem_type (char * path, char * relpath, struct stat * statp);
 
@@ -178,9 +203,7 @@ static void set_defaults (void)
   ShSuidchkQDelete  = S_FALSE;
   ShSuidchkSeverity = SH_ERR_SEVERE;
   if (ShSuidchkExclude != NULL)
-    SH_FREE(ShSuidchkExclude);
-  ShSuidchkExclude  = NULL;
-  ExcludeLen        = 0;
+    sh_suid_exclude_free();
 
   FileLimNow        = 0;
   FileLimStart      = 0;
@@ -1111,7 +1134,7 @@ int sh_suidchk_check_internal (char * iname)
 
     /* Rate limit (Fps == Files per second)
      */
-    if ((ShSuidchkFps > 0 && FileLimNum > ShSuidchkFps && FileLimTotal > 0)&&
+    if ((ShSuidchkFps > 0 && FileLimNum > ShSuidchkFps) &&
 	(ShSuidchkYield == S_FALSE))
       {
 	FileLimNum  = 0;
@@ -1135,6 +1158,8 @@ int sh_suidchk_check_internal (char * iname)
 	tlen = strlen(tmp);
 	if (tlen >= 6 && 0 == strcmp(&tmp[tlen-6], _("/.gvfs")))
 	  elevel = SH_ERR_NOTICE;
+	else if (tlen >= 5 && 0 == strcmp(&tmp[tlen-5], _("/gvfs")))
+	  elevel = SH_ERR_NOTICE;
 
         /* If we are scanning a temporary directory where dirs and files
 	 * can be created/deleted, an lstat() error is something which
@@ -1155,7 +1180,7 @@ int sh_suidchk_check_internal (char * iname)
       {
 	if (/*@-usedef@*/S_ISDIR(buf.st_mode)/*@+usedef@*/ &&
 	    (ShSuidchkExclude == NULL || 
-	     0 != strcmp(tmpcat, ShSuidchkExclude)))
+	     NULL == zAVL_string_get(ShSuidchkExclude, tmpcat)))
 	  {
 	    /* fs is a STATIC string or NULL
 	     */
@@ -1197,21 +1222,19 @@ int sh_suidchk_check_internal (char * iname)
 	  {
 	    int dummy;
 	    int class;
-	    unsigned long check_mask = 0;
+	    unsigned long check_flags = 0;
 
 	    theFile = SH_ALLOC(sizeof(file_type));
 
 	    (void) sl_strlcpy (theFile->fullpath, tmpcat, PATH_MAX);
-	    theFile->check_mask  = sh_files_maskof(SH_LEVEL_READONLY);
+	    theFile->check_flags  = sh_files_maskof(SH_LEVEL_READONLY);
 	    CLEAR_SH_FFLAG_REPORTED(theFile->file_reported);
 	    theFile->attr_string = NULL;
 	    theFile->link_path   = NULL;
 	    
-	    sh_files_search_file(tmpcat, &class,  &check_mask, &dummy);
-	    if ((check_mask & MODI_PREL) != 0)
-	      {
-		theFile->check_mask |= MODI_PREL;
-	      }
+	    sh_files_search_file(tmpcat, &class,  &check_flags, &dummy);
+	    if ((check_flags & MODI_PREL) != 0)
+	      MODI_SET(theFile->check_flags, MODI_PREL);
 
 	    status = sh_unix_getinfo (ShDFLevel[SH_ERR_T_RO], 
 				      dirlist->sh_d_name,
@@ -1269,7 +1292,7 @@ int sh_suidchk_check_internal (char * iname)
 			sh_hash_pushdata_memory (theFile, fileHash); /* no call to sh_error_handle */
 		      }
 		    
-		    sh_hash_addflag(tmpcat, SH_FFLAG_SUIDCHK); /* no call to sh_error_handle */
+		    sh_hash_set_flag(tmpcat, SH_FFLAG_SUIDCHK); /* no call to sh_error_handle */
 		    
 		  }
 		
@@ -1278,7 +1301,7 @@ int sh_suidchk_check_internal (char * iname)
 		  {
 		    /* Running init. Report on files detected.
 		     */
-		    sh_hash_pushdata (theFile, fileHash); /* no call to sh_error_handle */
+		    sh_dbIO_data_write (theFile, fileHash); /* no call to sh_error_handle */
 		    SH_MUTEX_LOCK(mutex_thread_nolog);
 		    sh_error_handle ((-1), FIL__, __LINE__, 
 				     0, MSG_SUID_FOUND, tmp );
@@ -1342,7 +1365,7 @@ int sh_suidchk_check_internal (char * iname)
 						     ShSuidchkSeverity);
 			    SH_MUTEX_UNLOCK(mutex_thread_nolog);
 			    
-			    sh_hash_addflag(tmpcat, SH_FFLAG_SUIDCHK); /* no call to sh_error_handle */
+			    sh_hash_set_flag(tmpcat, SH_FFLAG_SUIDCHK); /* no call to sh_error_handle */
 			    
 			  }
 		      }
@@ -1356,7 +1379,7 @@ int sh_suidchk_check_internal (char * iname)
 						 _("[SuidCheck]"),
 						 ShSuidchkSeverity);
 			SH_MUTEX_UNLOCK(mutex_thread_nolog);	
-			sh_hash_addflag(tmpcat, SH_FFLAG_SUIDCHK); /* no call to sh_error_handle */
+			sh_hash_set_flag(tmpcat, SH_FFLAG_SUIDCHK); /* no call to sh_error_handle */
 			
 		      }
 		  }
@@ -1528,6 +1551,7 @@ int sh_suidchk_set_severity  (const char * c)
 
 int sh_suidchk_set_exclude (const char * c)
 {
+  int ret = 0;
   SL_ENTER(_("sh_suidchk_set_exclude"));
 
   if (c == NULL || c[0] == '\0')
@@ -1538,22 +1562,13 @@ int sh_suidchk_set_exclude (const char * c)
   if (0 == sl_strncmp(c, _("NULL"), 4))
     {
       if (ShSuidchkExclude != NULL)
-	SH_FREE(ShSuidchkExclude);
-      ShSuidchkExclude = NULL;
+	sh_suid_exclude_free();
       SL_RETURN(0, _("sh_suidchk_set_exclude"));
     }
 
-  if (ShSuidchkExclude != NULL)
-    SH_FREE(ShSuidchkExclude);
+  ret = sh_suid_exclude_add(c);
 
-  ShSuidchkExclude = sh_util_strdup (c);
-  ExcludeLen       = sl_strlen (ShSuidchkExclude);
-  if (ShSuidchkExclude && ShSuidchkExclude[ExcludeLen-1] == '/')
-    {
-      ShSuidchkExclude[ExcludeLen-1] = '\0';
-      ExcludeLen--;
-    }
-  SL_RETURN(0, _("sh_suidchk_set_exclude"));
+  SL_RETURN(ret, _("sh_suidchk_set_exclude"));
 }
 
 int sh_suidchk_set_timer (const char * c)
@@ -2198,19 +2213,13 @@ filesystem_type (char * path, char * relpath, struct stat * statp)
   return current_fstype;
 }
 
-/* This variable is not used anywhere. It only exists
- * to assign &dirlist to it, which keeps gcc from
- * putting it into a register, and avoids the 'clobbered
- * by longjmp' warning. And no, 'volatile' proved insufficient.
- */
-static void * sh_dummy_type = NULL;
-
-
 /* Return a newly allocated string naming the type of filesystem that the
    file PATH, described by STATP, is on.
    RELPATH is the file name relative to the current directory.
    Return "unknown" if its filesystem type is unknown.  */
 
+void * sh_dummy_2229_type;
+
 static char *
 filesystem_type_uncached (path, relpath, statp)
      char *path;
@@ -2227,6 +2236,11 @@ filesystem_type_uncached (path, relpath, statp)
   FILE *mfp;
   struct mntent *mnt;
 
+  /* Take the address to keep gcc from putting it into a register. 
+   * Avoids the 'clobbered by longjmp' warning. 
+   */
+  sh_dummy_2229_type = (void *) &type;
+
   if (path == NULL || relpath == NULL)
     return NULL;
 
@@ -2241,11 +2255,6 @@ filesystem_type_uncached (path, relpath, statp)
       return NULL;
     }
 
-  /* Take the address to keep gcc from putting it into a register. 
-   * Avoids the 'clobbered by longjmp' warning. 
-   */
-  sh_dummy_type = (void*) &type;
-
   /* Find the entry with the same device number as STATP, and return
      that entry's fstype. */
   while (type == NULL && (mnt = getmntent (mfp)) != NULL)
@@ -2337,7 +2346,7 @@ filesystem_type_uncached (path, relpath, statp)
   /* Take the address to keep gcc from putting it into a register. 
    * Avoids the 'clobbered by longjmp' warning. 
    */
-  sh_dummy_type = (void*) &type;
+  sh_dummy_2229_type = (void*) &type;
 
   while (type == NULL
 	 && getmnt (&offset, &fsd, sizeof (fsd), NOSTAT_MANY, 0) > 0)
@@ -2357,7 +2366,7 @@ filesystem_type_uncached (path, relpath, statp)
   /* Take the address to keep gcc from putting it into a register. 
    * Avoids the 'clobbered by longjmp' warning. 
    */
-  sh_dummy_type = (void*) &type;
+  sh_dummy_2229_type = (void*) &type;
 
   if (statfs (relpath, &fss, sizeof (struct statfs), 0) == -1)
     {
@@ -2385,7 +2394,7 @@ filesystem_type_uncached (path, relpath, statp)
   /* Take the address to keep gcc from putting it into a register. 
    * Avoids the 'clobbered by longjmp' warning. 
    */
-  sh_dummy_type = (void*) &type;
+  sh_dummy_2229_type = (void*) &type;
 
   if (statvfs (relpath, &fss) == -1)
     {
@@ -2426,7 +2435,7 @@ filesystem_type_uncached (path, relpath, statp)
   /* Take the address to keep gcc from putting it into a register. 
    * Avoids the 'clobbered by longjmp' warning. 
    */
-  sh_dummy_type = (void*) &type;
+  sh_dummy_2229_type = (void*) &type;
 
   if (S_ISLNK (statp->st_mode))
     p = sh_dirname (relpath);
