@@ -93,6 +93,8 @@
 #undef  FIL__
 #define FIL__  _("sh_tools.c")
 
+static int tools_debug = 0;
+
 #ifdef SH_ENCRYPT
 #include "rijndael-api-fst.h"
 char * errorExplain (int err_num, char * buffer, size_t len)
@@ -106,7 +108,7 @@ char * errorExplain (int err_num, char * buffer, size_t len)
   else if (err_num == BAD_KEY_INSTANCE) 
     p = (_("Key passed is not valid"));
   else if (err_num == BAD_CIPHER_MODE) 
-    p = (_("Params struct passed to cipherInit invalid"));
+    p = (_("Params struct passed to rijndael_cipherInit invalid"));
   else if (err_num == BAD_CIPHER_STATE) 
     p = (_("Cipher in wrong state"));
   else if (err_num == BAD_BLOCK_LENGTH) 
@@ -563,6 +565,9 @@ int connect_port (char * address, int port,
 
   SL_ENTER(_("connect_port"));
 
+  if (tools_debug)
+    fprintf(stderr, _("-00- <%s> <%d> no IPv6 support\n"), address, port);
+
   if (errsiz > 0) errmsg[0] = '\0';
 
   /* paranoia -- should not happen
@@ -574,6 +579,10 @@ int connect_port (char * address, int port,
     {
       while (check_cache && check_cache->address)
 	{
+          if (tools_debug)
+            fprintf(stderr, _("-01- <%s> <%s>\n"), 
+                    address, check_cache->address);
+
 	  if ( 0 == sl_strncmp(check_cache->address, 
 			       address, sl_strlen(address)) )
 	    {
@@ -583,6 +592,16 @@ int connect_port (char * address, int port,
 	      cached = 1;
 	      break;
 	    }
+	  if (tools_debug)
+	    {
+	      char eaddr[SH_IP_BUF];
+	      sl_strlcpy(eaddr, 
+			 inet_ntoa(*(struct in_addr *) &(sinr.sin_addr)), 
+			 sizeof(eaddr));
+	      fprintf(stderr, _("-02- <AF_INET> <%s> <%d> <%d>\n"), 
+		      eaddr,
+		      port, cached);
+	    }
 	  if (check_cache->next)
 	    check_cache = check_cache->next;
 	  else
@@ -594,6 +613,8 @@ int connect_port (char * address, int port,
    */
   if (cached == 0)
     {
+      if (tools_debug)
+        fputs(_("-03- not cached\n"), stderr); 
 #ifdef HAVE_INET_ATON
       if (0 == inet_aton(address, &haddr))
 #else
@@ -624,6 +645,13 @@ int connect_port (char * address, int port,
 	      sinr.sin_port   = htons (port);
 	      sinr.sin_addr   = *(struct in_addr *) host_entry->h_addr;
 
+	      if (tools_debug)
+                fprintf(stderr, 
+			_("-04- <%s> <%s> hostent->h_name %s <%s> hostent->h_addr\n"), 
+                        address, 
+			(host_entry->h_name == NULL) ? _("NULL") : host_entry->h_name,
+			(host_entry->h_addrtype == AF_INET) ? _("AF_INET") : _("AF_INET6"),
+			inet_ntoa(*(struct in_addr *) &(sinr.sin_addr)));
 
 	      /* reverse DNS lookup
 	       */
@@ -687,6 +715,12 @@ int connect_port (char * address, int port,
 	  sinr.sin_family = AF_INET;
 	  sinr.sin_port   = htons (port);
 	  sinr.sin_addr   = haddr;
+
+	  if (tools_debug)
+	    fprintf(stderr, 
+		    _("-04- <%s> is_numeric AF_INET <%s> \n"), 
+		    address, 
+		    inet_ntoa(*(struct in_addr *) &(sinr.sin_addr)));
 	}
 
 
@@ -742,6 +776,9 @@ int connect_port (char * address, int port,
 	sl_strlcpy(ecall, _("connect"), SH_MINIBUF);
 	*errnum = status;
 	sl_strlcpy(errmsg, sh_error_message (status, errbuf, sizeof(errbuf)), errsiz);
+	sl_strlcat(errmsg, 
+		   (sinr.sin_family == AF_INET) ? _(", AF_INET ") : _(", AF_INET6 "),
+		   errsiz);
 	sl_strlcat(errmsg, _(", address "), errsiz);
 	sl_strlcat(errmsg, address, errsiz);
 	sl_close_fd(FIL__, __LINE__, fd);
@@ -775,10 +812,17 @@ int connect_port (char * address, int port,
   if (cached_addr > 128)
     delete_cache();
 
+  if (tools_debug)
+    fprintf(stderr, _("-00- <%s> <%d>\n"), address, port);
+
   if (check_cache != NULL)
     {
       while (check_cache && check_cache->address)
 	{
+	  if (tools_debug)
+	    fprintf(stderr, _("-01- <%s> <%s>\n"), 
+		    address, check_cache->address);
+
 	  if ( 0 == sl_strcmp(check_cache->address, address) )
 	    {
 	      memcpy (&ss, &(check_cache->saddr), sizeof(struct sh_sockaddr));
@@ -797,6 +841,15 @@ int connect_port (char * address, int port,
 		default:
 		  break;
 		}
+	      if (tools_debug)
+		{
+		  char eaddr[SH_IP_BUF];
+		  sh_ipvx_ntoa(eaddr, sizeof(eaddr), &ss);
+		  fprintf(stderr, _("-02- <%s> <%s> <%d> <%d>\n"), 
+			  (ss.ss_family == AF_INET) ? _("AF_INET") : _("AF_INET6"),
+			  eaddr,
+			  port, cached);
+		}
 	      break;
 	    }
 	  if (check_cache->next)
@@ -808,6 +861,8 @@ int connect_port (char * address, int port,
 
   if (cached != 0)
     {
+      if (tools_debug)
+	fputs(_("-03- cached\n"), stderr); 
       fd = socket(ss.ss_family, SOCK_STREAM, 0);
       if (fd < 0) 
 	{
@@ -853,6 +908,9 @@ int connect_port (char * address, int port,
       struct addrinfo *ai;
       struct addrinfo hints;
 
+      if (tools_debug)
+	fputs(_("-03- not cached\n"), stderr);
+
       memset (&hints, '\0', sizeof (hints));
       hints.ai_flags = AI_ADDRCONFIG;
 #if defined(AI_CANONNAME)
@@ -881,17 +939,27 @@ int connect_port (char * address, int port,
 	  char hostname[SH_BUFSIZE];
 	  const char * canonical;
 
+
 #if defined(AI_CANONNAME)
 	  if (ai->ai_canonname && strlen(ai->ai_canonname) > 0)
 	    {
 	      canonical = ai->ai_canonname;
+	      if (tools_debug)
+		fprintf(stderr, _("-04- <%s> <%s> ai->ai_canonname\n"), 
+			address, canonical);
 	    }
 	  else
 	    {
 	      canonical = address;
+	      if (tools_debug)
+		fprintf(stderr, _("-04- <%s> <%s> defined ai_canonname\n"), 
+			address, canonical);
 	    }
 #else
 	  canonical = address;
+	  if (tools_debug)
+	    fprintf(stderr, _("-04- <%s> <%s> not defined ai_canonname\n"), 
+		    address, canonical);
 #endif
 
 	  while (p != NULL)
@@ -902,11 +970,23 @@ int connect_port (char * address, int port,
 	      
 	      if (e == 0)
 		{
+		  if (tools_debug)
+		    {
+		      fprintf(stderr, _("-05- <%s> <%s> <%s>\n"), 
+			      (p->ai_family == AF_INET) ? _("AF_INET") : _("AF_INET6"),
+			      sh_ipvx_print_sockaddr (p->ai_addr, p->ai_family),
+			      hostname);
+		    }
+
 		  if (sl_strcasecmp(hostname, canonical) == 0)
 		    {
+		      if (tools_debug)
+			fprintf(stderr, _("-06- <%s> <%s> match\n"), 
+				hostname, canonical);
 		      success = 1;
 		      break;
 		    }
+
 		}
 	    
 	      p = p->ai_next;
@@ -1290,10 +1370,91 @@ int check_request_s (char * have, char * need, char * clt)
 
 #if defined (SH_WITH_CLIENT) || defined (SH_WITH_SERVER)
 
+#if defined (SH_WITH_CLIENT)
+
+static int           probe_done = S_FALSE;
+static unsigned char probe_flag = '\0';
+
+void sh_tools_probe_reset()
+{
+  probe_done = S_FALSE;
+  probe_flag = '\0';
+  return;
+}
+
+static int probe_ok(int flag)
+{
+  (void) flag;
+  if ((probe_flag & SH_PROTO_IVA) != 0)
+    return S_TRUE;
+  return S_FALSE;
+}
+
+static unsigned char probe_header_set(unsigned char protocol)
+{
+  if (probe_done || (protocol & SH_PROTO_SRP) == 0)
+    return 0;
+
+  return (char) SH_PROTO_IVA;
+}
+
+static void probe_header_get(unsigned char protocol)
+{
+  if (probe_done || (protocol & SH_PROTO_SRP) == 0)
+    return;
+
+  /* If the server doesn't know about it,
+   * it will simply mirror it back. */
+  
+  if ((protocol & SH_PROTO_IVA) != 0)
+    {
+      /* probe was mirrored */;
+    }
+  else
+    {
+      /* probe was UNset */
+      probe_flag |= SH_PROTO_IVA;
+    }
+  probe_done = S_TRUE;
+  return;
+}
+
+#else
+static unsigned char probe_header_set(unsigned char protocol) { 
+  (void) protocol; return 0; }
+static void probe_header_get(unsigned char protocol) { 
+  (void) protocol; return; }
+void sh_tools_probe_reset() { return; }
+
+unsigned char sh_tools_probe_store(unsigned char protocol, int * probe_flag)
+{
+  if ((protocol & SH_PROTO_SRP) == 0)
+    return protocol;
+
+  if ((protocol & SH_PROTO_IVA) != 0)
+    {
+      /* probe received */
+      *probe_flag |=  SH_PROTO_IVA;
+      protocol    &= ~SH_PROTO_IVA;
+    }
+  return protocol; 
+}
+
+static int probe_ok(int flag)
+{
+  if ((flag & SH_PROTO_IVA) != 0)
+    return S_TRUE;
+  return S_FALSE;
+}
+#endif
+
+
 void get_header (unsigned char * head, unsigned long * bytes, char * u)
 {
   SL_ENTER(_("get_header"));
 
+  probe_header_get(head[0]);
+
   *bytes = 
     (256 * (unsigned int)head[1] + (unsigned int)head[2]);
 
@@ -1312,22 +1473,23 @@ void get_header (unsigned char * head, unsigned long * bytes, char * u)
 
 #if defined(SH_WITH_CLIENT) || defined(SH_WITH_SERVER)
 
-#ifdef  SH_ENCRYPT_2
+#ifdef  SH_ENCRYPT
 #define TRANS_BYTES 65120
 #else
 #define TRANS_BYTES 65280
 #endif
 
-void put_header (unsigned char * head, int protocol, 
+void put_header (unsigned char * head, const int protocol, 
 		 unsigned long * length, char * u)
 {
+  unsigned char probe = probe_header_set(protocol);
 
   /* static long transfer_limit = (8 * SH_BUFSIZE); V0.8 */
   static unsigned long transfer_limit = TRANS_BYTES + 6 + KEY_LEN;
 
   SL_ENTER(_("put_header"));
 
-  head[0]   = protocol;
+  head[0]   = protocol|probe;
 
   ASSERT((*length < transfer_limit), _("*length < transfer_limit"))
 
@@ -1353,7 +1515,6 @@ void put_header (unsigned char * head, int protocol,
 
   SL_RET0(_("put_header"));
 }
-#endif
 
 /* ------------------------------------------
  *
@@ -1372,17 +1533,7 @@ void put_header (unsigned char * head, int protocol,
 /* 
  * only SH_V2_FULLSIZE is used, and only once
  */
-#if 0
-#ifdef SH_WITH_SERVER
-#define SH_V2_FULLSIZE  240
-#define SH_V2_PAYLOAD   224
-#define SH_V2_MESSAGE   176
-#else
-#define SH_V2_FULLSIZE 1024
-#define SH_V2_PAYLOAD  1008
-#define SH_V2_MESSAGE   960
-#endif
-#endif
+
 #define SH_V2_FULLSIZE 1024
 
 #ifdef SH_ENCRYPT
@@ -1417,52 +1568,54 @@ void sh_tools_show_header (unsigned char * head, char sign)
       SH_IS_ASCII(head[4]) && isalpha(head[4]) &&			   
       SH_IS_ASCII(head[5]) && isalpha(head[5]) &&			   
       SH_IS_ASCII(head[6]) && isalpha(head[6])) {
-    fprintf(stderr, "%c %3o %s %5d  %c  %c  %c  %c\n", sign,
+    fprintf(stderr, _("%c %3o %s %5d  %c  %c  %c  %c\n"), sign,
 	    head[0], code, msg_size, head[3], head[4], head[5], head[6]); 
   } else {
-    fprintf(stderr, "%c %3o %s %5d %2X %2X %2X %2X\n", sign,
+    fprintf(stderr, _("%c %3o %s %5d %2X %2X %2X %2X\n"), sign,
 	    head[0], code, msg_size, head[3], head[4], head[5], head[6]); 
   }
   return;
 }
 
 #ifdef SH_ENCRYPT
+
 /*
  * #define DEBUG_EN2
  *
  * ingest version 1 7-byte header and payload, return version2 header/payload
  * last 4 bytes of outgoing header are set to dummy value
  */
-char * sh_tools_makePack (unsigned char * header, 
+char * sh_tools_makePack (unsigned char * header, int flag,
 			  char * payload, unsigned long payload_size,
 			  keyInstance * keyInstE)
 {
-  UINT32 rpad[3];
-  unsigned char   head[16];
-  double epad;
-  unsigned long    i_epad = 0;
-  unsigned long    i_blk = payload_size / 16;
-  unsigned long    i_blkmax = SH_V2_FULLSIZE / 16;
-  unsigned long    pads = 0;
-  size_t full_size;
-  char * full_ret;
+  BYTE            inBlock[B_SIZ]; 
+  BYTE            outBlock[B_SIZ];
+  char            ivBlock[B_SIZ];
 
-  char                  * p;
-  RIJ_BYTE                    inBlock[B_SIZ]; 
-  RIJ_BYTE                    outBlock[B_SIZ];
-  int                     j;
-  cipherInstance          cipherInst;
-  int                     err_num;
-  int                     blkfac;
-  int                     oflow = 0;
+  UINT32          rpad[3];
+  unsigned char   head[16];
+  double          epad;
+  unsigned long   i_epad = 0;
+  unsigned long   i_blk = payload_size / 16;
+  unsigned long   i_blkmax = SH_V2_FULLSIZE / 16;
+  unsigned long   pads = 0;
+  size_t          full_size;
+  char          * full_ret;
+
+  unsigned char * p;
+  int             j;
+  cipherInstance  cipherInst;
+  int             err_num;
+  int             blkfac;
+  int             oflow = 0;
+  char expbuf[SH_ERRBUF_SIZE];
 
   /* 
      SL_REQUIRE (i_blk*16 == payload_size, _("payload_size % 16 != 0"));
   */
   if ((i_blk * 16) != payload_size) ++i_blk;
-#ifdef DEBUG_EN2
-  fprintf(stderr, "SEND <%d> blocks <%d>\n", payload_size, i_blk);
-#endif
+
   /* random_pad
    */
   rpad[1] = taus_get ();
@@ -1472,10 +1625,6 @@ char * sh_tools_makePack (unsigned char * header,
   rpad[2] = taus_get ();
   memcpy (&head[8],  &rpad[2],    4);
 
-  /* protocol
-   */
-  /* memcpy (&head[8],  &header[3], 4); */
-
   /* size (payload)
    */ 
   head[12] = header[1];
@@ -1485,16 +1634,9 @@ char * sh_tools_makePack (unsigned char * header,
 
   if (i_blk < i_blkmax) 
   {
-    pads = i_blkmax - i_blk;
-    /* memcpy((char *) &rpad[2], &head[12], 4); */
-    epad = taus_get_double (&rpad);
-#ifdef DEBUG_EN2
-    fprintf(stderr, "PAD1 <%d> <%f>\n", pads, epad);
-#endif
+    pads   = i_blkmax - i_blk;
+    epad   = taus_get_double (&rpad);
     i_epad = (unsigned long) (pads * epad);
-#ifdef DEBUG_EN2
-    fprintf(stderr, "PAD2 <%d> <%d>\n", i_epad, (i_epad*16));
-#endif
   }
 
   full_size =  16;                        /* head     */
@@ -1515,57 +1657,64 @@ char * sh_tools_makePack (unsigned char * header,
     }
 
   full_ret = SH_ALLOC(full_size);
+
   memcpy(full_ret,                   head,    16);
   if (payload != NULL && !oflow)
-    {
-      memcpy(&full_ret[16],              payload, payload_size);
-    }
+    memcpy(&full_ret[16],              payload, payload_size);
+
   if ((i_blk*16) > payload_size && !oflow) 
     {
-#ifdef DEBUG_EN2
-      fprintf(stderr, "SEN2 <%d>\n", (i_blk*16) - payload_size);
-#endif
       memset(&full_ret[16+payload_size], '\0', (i_blk*16) - payload_size);
       payload_size = i_blk * 16;
     }
   memset(&full_ret[16+payload_size], '\0', i_epad*16);
-#ifdef DEBUG_EN2
-  fprintf(stderr, "SEN3 <%d> <%d>\n", full_size, i_epad*16);
-#endif
 
   /* rewrite header
    */
   header[1]   = (unsigned int)(full_size/256);
   header[2]   = (unsigned int)(full_size - (256 * header[1]));
-  /* don't erase protocol from header 
-     memset(&header[3], '\0', 4);
-  */
-  p = full_ret; blkfac = full_size / 16;
 
-  err_num = cipherInit (&cipherInst, MODE_CBC, NULL);
+  p      = (unsigned char *) full_ret; 
+  blkfac = full_size / B_SIZ;
+
+  err_num = rijndael_cipherInit (&cipherInst, MODE_CBC, NULL);
   
-  if (err_num < 0) 
-    {
-      char expbuf[SH_ERRBUF_SIZE];
-      sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
-		      errorExplain(err_num, expbuf, sizeof(expbuf)), 
-		      _("sh_tools_makePack: cipherInit"));
+  if (err_num < 0) {
+    sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+		    errorExplain(err_num, expbuf, sizeof(expbuf)), 
+		    _("sh_tools_makePack: rijndael_cipherInit")); }
+
+  if (probe_ok(flag)) {
+    memcpy(inBlock, p, B_SIZ);
+    err_num = rijndael_blockEncrypt(&cipherInst, keyInstE, 
+				    inBlock, 128, outBlock);
+    if (err_num >= 0) {
+      memcpy(p, outBlock, B_SIZ); p += B_SIZ;
+      memcpy(ivBlock, outBlock, sizeof(ivBlock));
+      err_num = rijndael_cipherInit (&cipherInst, MODE_CBC, ivBlock);
+      if (err_num >= 0) {
+	err_num = rijndael_blockEncrypt(&cipherInst, keyInstE, 
+					p, 128*(blkfac-1), p);
+      }
     }
-  for (j = 0; j < blkfac; ++j)
-    {
+  }
+
+  else {
+    for (j = 0; j < blkfac; ++j) {
       memcpy(inBlock, p, B_SIZ);
-      err_num = blockEncrypt(&cipherInst, keyInstE, 
-			     inBlock, 128 * BNUM, outBlock);
-      if (err_num < 0)
-	{
-	  char expbuf[SH_ERRBUF_SIZE];
-	  sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
-			  errorExplain(err_num, expbuf, sizeof(expbuf)), 
-			  _("sh_tools_makePack: blockEncrypt"));
-	}
+      err_num = rijndael_blockEncrypt(&cipherInst, keyInstE, 
+				      inBlock, 128, outBlock);
+      
+      if (err_num < 0) {
+	sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+			errorExplain(err_num, expbuf, sizeof(expbuf)), 
+			_("sh_tools_makePack: rijndael_blockEncrypt"));
+	break; }
+      
       memcpy(p, outBlock, B_SIZ);
       p += B_SIZ;
     }
+  }
 
   return full_ret;
 }
@@ -1573,16 +1722,17 @@ char * sh_tools_makePack (unsigned char * header,
 /* write a 7-byte header and return payload as expected by version 1
  * last 4 bytes of incoming header are dummy
  */
-char * sh_tools_revertPack (unsigned char * header, char * message, 
+char * sh_tools_revertPack (unsigned char * header, int flag, char * message, 
 			    keyInstance * keyInstD,
 			    unsigned long message_size)
 {
+  BYTE                    inBlock[B_SIZ]; 
+  BYTE                    outBlock[B_SIZ];
+  char                    ivBlock[B_SIZ];
   unsigned long   msg_size;
   char          * msg_ret;
 
-  char                  * p;
-  RIJ_BYTE                    inBlock[B_SIZ]; 
-  RIJ_BYTE                    outBlock[B_SIZ];
+  unsigned char         * p;
   int                     j;
   cipherInstance          cipherInst;
   int                     err_num;
@@ -1590,41 +1740,52 @@ char * sh_tools_revertPack (unsigned char * header, char * message,
   char expbuf[SH_ERRBUF_SIZE];
 
   msg_size = (256 * (unsigned int)header[1] + (unsigned int)header[2]);
-#ifdef DEBUG_EN2
-  fprintf(stderr, "RECV <%lu>\n", msg_size);
-#endif
-  if (msg_size > message_size) {
+  if (msg_size > message_size) 
     msg_size = message_size;
-#ifdef DEBUG_EN2
-    fprintf(stderr, "RECV TRUNC1 <%lu>\n", msg_size);
-#endif
-  }
 
-  p = message; blkfac = msg_size / 16;
+  p = (unsigned char *) message; blkfac = msg_size / 16;
 
-  err_num = cipherInit (&cipherInst, MODE_CBC, NULL);
+  err_num = rijndael_cipherInit (&cipherInst, MODE_CBC, NULL);
   
   if (err_num < 0) 
     {
       sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
 		      errorExplain(err_num, expbuf, sizeof(expbuf)), 
-		      _("sh_tools_revertPack: cipherInit"));
+		      _("sh_tools_revertPack: rijndael_cipherInit"));
     }
-  for (j = 0; j < blkfac; ++j)
-    {
+
+  if (probe_ok(flag)) {
+    memcpy(inBlock, p, B_SIZ);
+    err_num = rijndael_blockDecrypt(&cipherInst, keyInstD, 
+				    inBlock, 128, outBlock);
+    if (err_num >= 0) {
+      memcpy(p, outBlock, B_SIZ); p += B_SIZ;
+      memcpy(ivBlock, inBlock, sizeof(ivBlock));
+      err_num = rijndael_cipherInit (&cipherInst, MODE_CBC, ivBlock);
+      if (err_num >= 0) {
+	err_num = rijndael_blockDecrypt(&cipherInst, keyInstD, 
+					p, 128*(blkfac-1), p);
+      }
+    }
+  }
+
+  else {
+    for (j = 0; j < blkfac; ++j) {
       memcpy(inBlock, p, B_SIZ);
-      err_num = blockDecrypt(&cipherInst, keyInstD, 
-			     inBlock, 128 * BNUM, outBlock);
-      if (err_num < 0)
-	{
-	  sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
-			  errorExplain(err_num, expbuf, sizeof(expbuf)), 
-			  _("sh_tools_revertPack: blockDecrypt"));
-	}
+      err_num = rijndael_blockDecrypt(&cipherInst, keyInstD, 
+				      inBlock, 128, outBlock);
+  
+      if (err_num < 0) {
+	sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+			errorExplain(err_num, expbuf, sizeof(expbuf)), 
+			_("sh_tools_revertPack: rijndael_blockDecrypt"));
+	break; }
+      
       memcpy(p, outBlock, B_SIZ);
       p += B_SIZ;
     }
-  
+  }
+
   /* rewrite size in header
    */
   header[1]   = message[12];
@@ -1636,33 +1797,19 @@ char * sh_tools_revertPack (unsigned char * header, char * message,
       msg_size    = message_size-16;
       header[1]   = (unsigned int)(msg_size/256);
       header[2]   = (unsigned int)(msg_size - (256 * header[1]));
-#ifdef DEBUG_EN2
-      fprintf(stderr, "RECV TRUNC2 <%lu>\n", msg_size);
-#endif
     }
-#ifdef DEBUG_EN2
-  fprintf(stderr, "REC2 <%lu>\n", msg_size);
-#endif
-  /* protocol
-   */
-  /* memcpy(&header[3], &message[8], 4); */
 
   /* payload 
    */
   msg_ret = SH_ALLOC(msg_size+1);
   if (msg_size > 0)
-    {
-      memcpy(msg_ret, &message[16], msg_size);
-    }
+    memcpy(msg_ret, &message[16], msg_size);
   msg_ret[msg_size] = '\0';
-#ifdef DEBUG_EN2
-  fprintf(stderr, "REC3 <%lu>\n", msg_size);
-#endif
-  SH_FREE(message);
 
+  SH_FREE(message);
   return msg_ret;
 }
-#endif
+#endif /* #ifdef SH_ENCRYPT */
 
 int sh_tools_hash_add(char * key, char * buf, int buflen)
 {
@@ -1701,6 +1848,9 @@ int sh_tools_hash_vfy(char * key, char * buf, int buflen)
   SL_RETURN((1), _("sh_tools_hash_vfy"));
 }
 
+#endif /* defined(SH_WITH_CLIENT) || defined(SH_WITH_SERVER) */
+
+
 /* ------------------------------------------ */
 
 #if defined (SH_WITH_SERVER)
@@ -1787,51 +1937,39 @@ int hash_check(char * key,
 
 #if defined (SH_WITH_SERVER)
 
-char * get_client_conf_file (char * peer, unsigned long * length)
+char * get_client_conf_file (const char * peer, unsigned long * length)
 {
   char * ret;
   int    status;
   struct stat buf;
   char * base;
-  size_t size;
 
   SL_ENTER(_("get_client_conf_file"));
 
   base = sh_util_strdup(DEFAULT_DATAROOT);
+  ret = sh_util_strconcat(base, _("/rc."), peer, NULL);
+  if (!ret) 
+    { SH_FREE(base); *length = 0; SL_RETURN(NULL, _("get_client_conf_file")); }
 
-  size = sl_strlen(base);
-  if (sl_ok_adds(size, sl_strlen(peer)))
-      size += sl_strlen(peer);
-  if (sl_ok_adds(size, 6))
-    size += 6;
-
-  ret = SH_ALLOC(size);
-  sl_strlcpy(ret, base, size);
-  sl_strlcat(ret, _("/rc."), size);
-  sl_strlcat(ret, peer, size);
-  
   status = retry_stat (FIL__, __LINE__, ret, &buf);
-
   if (status == 0)
     goto lab_end;
   else
     sh_error_handle(SH_ERR_WARN, FIL__, __LINE__, status, MSG_E_ACCESS,
 		    (long) sh.effective.uid, ret);
 
-  sl_strlcpy(ret, base, size);
-  sl_strlcat(ret, "/rc", size);
+  SH_FREE(ret); ret = sh_util_strconcat(base, _("/rc"), NULL);
+  if (!ret) 
+    { SH_FREE(base); *length = 0; SL_RETURN(NULL, _("get_client_conf_file")); }
   
   status = retry_stat (FIL__, __LINE__, ret, &buf);
-
   if (status == 0)
     goto lab_end;
   else
     sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, status, MSG_E_ACCESS,
 		    (long) sh.effective.uid, ret);
 
-  SH_FREE(base);
-  SH_FREE(ret);
-  *length=0;
+  SH_FREE(base); SH_FREE(ret); *length=0;
   SL_RETURN(NULL, _("get_client_conf_file"));
 
  lab_end:
@@ -1839,62 +1977,48 @@ char * get_client_conf_file (char * peer, unsigned long * length)
     {
       sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, status, MSG_E_SUBGEN,
 		    _("File too large"), _("get_client_conf_file"));
-      SH_FREE(base);
+      SH_FREE(base); SH_FREE(ret); *length = 0;
       SL_RETURN(NULL, _("get_client_conf_file"));
     }
-  *length = (unsigned long) buf.st_size;
-  SH_FREE(base);
+
+  SH_FREE(base); *length = (unsigned long) buf.st_size;
   SL_RETURN(ret, _("get_client_conf_file"));
 }
 
-char * get_client_data_file (char * peer, unsigned long * length)
+char * get_client_data_file (const char * peer, unsigned long * length)
 {
   char * ret;
   int    status;
   struct stat buf;
-
   char * base;
-  size_t size;
 
   SL_ENTER(_("get_client_data_file"));
 
   base = sh_util_strdup(DEFAULT_DATAROOT);
+  ret = sh_util_strconcat(base, _("/file."), peer, NULL);
+  if (!ret) 
+    { SH_FREE(base); *length = 0; SL_RETURN(NULL, _("get_client_data_file")); }
 
-  size = sl_strlen(base);
-  if (sl_ok_adds(size, sl_strlen(peer)))
-      size += sl_strlen(peer);
-  if (sl_ok_adds(size, 8))
-    size += 8;
-
-  ret = SH_ALLOC(size);
-  sl_strlcpy(ret, base, size);
-  sl_strlcat(ret, _("/file."), size);
-  sl_strlcat(ret, peer, size);
-  
   status = retry_stat (FIL__, __LINE__, ret, &buf);
-
   if (status == 0)
     goto lab1_end;
   else
     sh_error_handle(SH_ERR_WARN, FIL__, __LINE__, status, MSG_E_ACCESS,
 		    (long) sh.effective.uid, ret);
 
+  SH_FREE(ret); 
+  ret = sh_util_strconcat(base, _("/file"), NULL);
+  if (!ret) 
+    { SH_FREE(base); *length = 0; SL_RETURN(NULL, _("get_client_data_file")); }
 
-  sl_strlcpy(ret, base, size);
-  sl_strlcat(ret, _("/file"), size);
-  
   status = retry_stat (FIL__, __LINE__, ret, &buf);
-
   if (status == 0)
     goto lab1_end;
   else
     sh_error_handle(SH_ERR_WARN, FIL__, __LINE__, status, MSG_E_ACCESS,
 		    (long) sh.effective.uid, ret);
 
-
-  *length = 0;
-  SH_FREE(base);
-  SH_FREE(ret);
+  *length = 0; SH_FREE(base); SH_FREE(ret);
   SL_RETURN(NULL, _("get_client_data_file"));
 
  lab1_end:
@@ -1902,14 +2026,49 @@ char * get_client_data_file (char * peer, unsigned long * length)
     {
       sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, status, MSG_E_SUBGEN,
 		    _("File too large"), _("get_client_data_file"));
-      SH_FREE(base);
+      SH_FREE(base); SH_FREE(ret); *length = 0;
       SL_RETURN(NULL, _("get_client_data_file"));
     }
-  *length = (unsigned long) buf.st_size;
-  SH_FREE(base);
+
+  *length = (unsigned long) buf.st_size; SH_FREE(base);
   SL_RETURN(ret, _("get_client_data_file"));
-  
 }
+
+char * get_client_uuid_file (const char * peer, unsigned long * length, const char * uuid)
+{
+  char * ret;
+  int    status;
+  struct stat buf;
+  char * base;
+
+  SL_ENTER(_("get_client_uuid_file"));
+
+  base = sh_util_strdup(DEFAULT_DATAROOT);
+  ret = sh_util_strconcat(base, _("/file."), peer, ".", uuid, NULL);
+  SH_FREE(base);
+  if (!ret) 
+    { *length = 0; SL_RETURN(NULL, _("get_client_uuid_file")); }
+
+  status = retry_stat (FIL__, __LINE__, ret, &buf);
+  if (status != 0)
+    {
+      sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, status, MSG_E_ACCESS,
+		      (long) sh.effective.uid, ret);
+      SH_FREE(ret); *length = 0;
+      SL_RETURN(NULL, _("get_client_uuid_file"));
+    }
+  else if (buf.st_size > 0x7fffffff)
+    {
+      sh_error_handle(SH_ERR_ERR, FIL__, __LINE__, status, MSG_E_SUBGEN,
+		    _("File too large"), _("get_client_uuid_file"));
+      SH_FREE(ret); *length = 0;
+      SL_RETURN(NULL, _("get_client_data_file"));
+    }
+
+  *length = (unsigned long) buf.st_size;
+  SL_RETURN(ret, _("get_client_uuid_file"));
+}
+
 #endif
 
 #if defined(SH_WITH_CLIENT) || defined(SH_WITH_SERVER) || defined(SH_STEALTH) || defined(WITH_GPG) || defined(WITH_PGP)
@@ -2010,12 +2169,11 @@ SL_TICKET open_tmp ()
   fd = sl_open_safe_rdwr (FIL__, __LINE__, file, SL_YESPRIV);
   if (SL_ISERROR(fd))
     {
-      sh_error_handle(SH_ERR_ALL, FIL__, __LINE__, fd, MSG_E_SUBGEN, 
+      sh_error_handle((-1), FIL__, __LINE__, fd, MSG_E_SUBGEN, 
 		      _("Error opening temporary file"), _("open_tmp"));
       TPT(( 0, FIL__, __LINE__, _("msg=<Error %d temporary file %s>\n"), 
 	    fd, file));
     }
-  
 
   SH_FREE (file);
   SH_FREE(my_tmp_dir);
