@@ -223,6 +223,7 @@ int sh_util_ask_update(const char * path)
     {
       if (NULL != ttyname(STDERR_FILENO))
         {
+	  /* cppcheck-suppress leakReturnValNotUsed */
           if (NULL == freopen(ttyname(STDERR_FILENO), "r", stdin))
             {
               sh_error_handle ((-1), FIL__, __LINE__, 0, 
@@ -308,11 +309,11 @@ char * sh_util_acl_compact(char * buf, ssize_t len)
 	    goto exit_it;
 	  }
 	}
-	out[rem] = ','; ++rem;
-	while (p[1] == '\n') ++p; /* scan over consecutive newlines */
+	out[rem] = ','; ++rem;    /* <-- ensures (rem > 0) is true    */ 
+	while (p[1] == '\n') ++p; /* scan over consecutive newlines   */
 	state = 0;
 	if (p[1] == '\0') {
-	  if (rem > 0) out[rem-1] = '\0';
+	  out[rem-1] = '\0';      /* rem > 0 because of 4 lines above */
 	  break;
 	}
       }
@@ -781,29 +782,20 @@ static char * sh_util_hmac_tiger (char * hexkey,
   UINT32      kbuf[KEY_BYT/sizeof(UINT32)];
   char hashbuf[KEYBUF_SIZE];
 
-
+  int         result;
   size_t      i;
 
   SL_ENTER(_("sh_util_hmac_tiger"));
-  ASSERT((KEY_BLOCK <= (KEY_LEN/2)), _("KEY_BLOCK <= (KEY_LEN/2)"))
 
-  if (KEY_BLOCK > (KEY_LEN/2))
-    {
-      (void) sh_tiger_hash (NULL, TIGER_DATA, 0, hashbuf, sizeof(hashbuf));
-      sl_strlcpy(res, hashbuf, len);
-      SL_RETURN(res, _("sh_util_hmac_tiger"));
-    }
+  ASSERT((KEY_BLOCK <= (KEY_LEN/2)), _("KEY_BLOCK <= (KEY_LEN/2)"));
 
   memcpy (K, zap, KEY_BLOCK);
 
-  if (sh_util_hextobinary (K, hexkey, KEY_LEN) < 0)
-    {
-      (void) sh_tiger_hash (NULL, TIGER_DATA, 0, hashbuf, sizeof(hashbuf));
-      sl_strlcpy(res, hashbuf, len);
-      SL_RETURN(res, _("sh_util_hmac_tiger"));
-    }
+  result = sh_util_hextobinary (K, hexkey, KEY_LEN);
 
-  if (sl_ok_adds(textlen, KEY_BLOCK))
+  ASSERT((result >= 0), _("result >= 0"));
+
+  if ((result >= 0) && sl_ok_adds(textlen, KEY_BLOCK))
     {
       inner = (char *) SH_ALLOC (textlen + KEY_BLOCK); 
 
@@ -829,25 +821,17 @@ static char * sh_util_hmac_tiger (char * hexkey,
 
   /* now compute the hash 
    */
-  h1 = sh_tiger_hash_uint32 ( outer,
-			      TIGER_DATA,
-			      KEY_BLOCK,
+  h1 = sh_tiger_hash_uint32 ( outer, TIGER_DATA, KEY_BLOCK,
 			      kbuf, KEY_BYT/sizeof(UINT32));
   for (i = 0; i < (KEY_LEN/8); ++i)
-    {
-      /* cc[i] = h1[i]; */
-      copy_four ( (unsigned char *) &(cc[i]), h1[i]);
-    }
+    copy_four ( (unsigned char *) &(cc[i]), h1[i]);
 
-  h2 = sh_tiger_hash_uint32 ( inner,
-			      TIGER_DATA,
+  h2 = sh_tiger_hash_uint32 ( inner, TIGER_DATA,
 			      (unsigned long) KEY_BLOCK+textlen,
 			      kbuf, KEY_BYT/sizeof(UINT32));
   for (i = KEY_LEN/8; i < (KEY_LEN/4); ++i)
-    {
-      copy_four ( (unsigned char *) &(cc[i]), h2[i - (KEY_LEN/8)]);
-      /* cc[i] = h2[i - (KEY_LEN/8)]; */
-    }
+    copy_four ( (unsigned char *) &(cc[i]), h2[i - (KEY_LEN/8)]);
+
   SH_FREE(inner);
   
   (void) sh_tiger_hash ((char *) &cc[0],
@@ -1032,7 +1016,7 @@ void sh_util_cpylong (char * dest, const char * src, int len )
  */
 
 
-/* interval [0, 4294967296]
+/* interval [0, 4294967295]
  */
 static UINT32 taus_get_long (void *vstate)
 {
@@ -1159,18 +1143,23 @@ int taus_seed ()
 {
   char                 bufx[9 * sizeof(UINT32) + 1];
   int                  status;
-  static unsigned long seed_time = 0;
+  static unsigned long seed_time    = 0;
+  static unsigned long seed_counter = 3000;
   unsigned long        gtime;
 
   SL_ENTER(_("taus_seed"));
 
   if (skey->rngI == GOOD)
     {
-      if ( (sh_unix_longtime () - seed_time) < 7200)
+      ++seed_counter;
+
+      if ( ((sh_unix_longtime () - seed_time) < 1800) &&
+	   ( seed_counter                     < 3000))
 	SL_RETURN( (0), _("taus_seed"));
     }
   
-  seed_time = sh_unix_longtime ();
+  seed_time    = sh_unix_longtime ();
+  seed_counter = 0;
 
   status = sh_entropy (24, bufx);
 
@@ -1228,7 +1217,7 @@ int sh_util_set_newkey (const char * new_in)
   SL_TICKET fp;
   SL_TICKET fout;
   char * key;
-  char * path;
+  char * path = NULL;
   char * outpath = NULL;
   unsigned char * image = NULL;
   long s = 0;
@@ -1254,7 +1243,7 @@ int sh_util_set_newkey (const char * new_in)
       return -1;
     }
 
-  if (NULL == (new = malloc(strlen(new_in) + 1)))
+  if (NULL == (new = calloc(1,strlen(new_in) + 1)))
     goto bail_mem;
   sl_strncpy(new, new_in, strlen(new_in) + 1);
 
@@ -1281,7 +1270,7 @@ int sh_util_set_newkey (const char * new_in)
 
   len = strlen(path) + 1 + 4;
   /*@-usedef@*/
-  if (NULL == (outpath = malloc(len)))
+  if (NULL == (outpath = calloc(1,len)))
     goto bail_mem;
   /*@-usedef@*/
   sl_snprintf (outpath, len, _("%s.out"), path);
@@ -1309,14 +1298,19 @@ int sh_util_set_newkey (const char * new_in)
     }
 
 
-  image = malloc (4096);
+  image = calloc(1,4096);
   if (!image)
     goto bail_mem;
   while (0 < (ii = sl_read (fp, &image[s], 4096)))
     {
+      unsigned char * ptr;
       ilen += ii;
       s    += 4096;
-      image = realloc (image, (size_t) (4096 + s));
+      ptr = realloc (image, (size_t) (4096 + s));
+      if (ptr)
+	image = ptr;
+      else
+	{ free(image); image = NULL; }
       if (!image)
 	goto bail_mem;
     }
@@ -1828,7 +1822,7 @@ int sh_util_obscure_utf8 (const char * c)
 
 int sh_util_obscurename (ShErrLevel level, const char * name_orig, int flag)
 {
-  const unsigned char * name = (unsigned char *) name_orig;
+  const unsigned char * name = (const unsigned char *) name_orig;
   char * safe;
   unsigned int i;
   size_t len = 0;
@@ -2302,7 +2296,7 @@ size_t sh_util_base64_dec (unsigned char *out,
   unsigned char c;
   unsigned char b;
   size_t lout = 0;
-  int    w = 0;
+  unsigned int  w = 0;
 
   if (out && in)
     {
