@@ -1150,8 +1150,8 @@ static   gid_t   egid;
 static   gid_t   rgid;
 static   gid_t   rgid_orig;
 
-static   int     uids_are_stored = SL_FALSE;
-static   int     suid_is_set     = SL_TRUE;
+static   int     uids_are_stored = S_FALSE;
+static   int     suid_is_set     = S_TRUE;
 
 #ifdef HAVE_SETRESUID
 extern       int setresuid (uid_t truid, uid_t teuid, uid_t tsuid);
@@ -1165,7 +1165,7 @@ extern       int setresgid (gid_t trgid, gid_t tegid, gid_t tsgid);
  */
 int sl_is_suid()
 {
-  if (uids_are_stored == SL_FALSE)
+  if (uids_are_stored == S_FALSE)
     {
       if (getuid() == geteuid() && getgid() == getegid())
 	return (0);     /* FALSE */
@@ -1188,8 +1188,8 @@ int sl_is_suid()
 int sl_get_euid(uid_t * ret)
 {
   SL_ENTER(_("sl_get_euid"));
-  /* SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));*/
-  if (uids_are_stored == SL_TRUE)
+  /* SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));*/
+  if (uids_are_stored == S_TRUE)
     *ret = euid;
   else
     *ret = geteuid();
@@ -1198,8 +1198,8 @@ int sl_get_euid(uid_t * ret)
 
 uid_t sl_ret_euid()
 {
-  /* SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));*/
-  if (uids_are_stored == SL_TRUE)
+  /* SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));*/
+  if (uids_are_stored == S_TRUE)
     return (euid);
   else
     return (geteuid());
@@ -1212,8 +1212,8 @@ uid_t sl_ret_euid()
 int sl_get_egid(gid_t * ret)
 {
   SL_ENTER(_("sl_get_egid"));
-  /* SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));*/
-  if (uids_are_stored == SL_TRUE)
+  /* SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));*/
+  if (uids_are_stored == S_TRUE)
     *ret = egid;
   else
     *ret = getegid();
@@ -1227,8 +1227,8 @@ int sl_get_egid(gid_t * ret)
 int sl_get_ruid(uid_t * ret)
 {
   SL_ENTER(_("sl_get_ruid"));
-  /* SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));*/
-  if (uids_are_stored == SL_TRUE)
+  /* SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));*/
+  if (uids_are_stored == S_TRUE)
     *ret = ruid;
   else
     *ret = getuid();
@@ -1242,8 +1242,8 @@ int sl_get_ruid(uid_t * ret)
 int sl_get_rgid(gid_t * ret)
 {
   SL_ENTER(_("sl_get_rgid"));
-  /* SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));*/
-  if (uids_are_stored == SL_TRUE)
+  /* SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));*/
+  if (uids_are_stored == S_TRUE)
     *ret = rgid;
   else
     *ret = getgid();
@@ -1257,8 +1257,8 @@ int sl_get_rgid(gid_t * ret)
 int sl_get_ruid_orig(uid_t * ret)
 {
   SL_ENTER(_("sl_get_ruid_orig"));
-  /* SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));*/
-  if (uids_are_stored == SL_TRUE)
+  /* SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));*/
+  if (uids_are_stored == S_TRUE)
     *ret = ruid_orig;
   else
     *ret = getuid();
@@ -1272,8 +1272,8 @@ int sl_get_ruid_orig(uid_t * ret)
 int sl_get_rgid_orig(gid_t * ret)
 {
   SL_ENTER(_("sl_get_rgid_orig"));
-  /* SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));*/
-  if (uids_are_stored == SL_TRUE)
+  /* SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));*/
+  if (uids_are_stored == S_TRUE)
     *ret = rgid_orig;
   else
     *ret = getgid();
@@ -1298,19 +1298,19 @@ int sl_set_suid ()
 
   SL_ENTER(_("sl_set_suid"));
 
-  if (uids_are_stored == SL_FALSE)
+  if (uids_are_stored == S_FALSE)
     {
       SL_IRETURN(SL_ENONE, _("sl_set_suid"));
     }
 
-  SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));  
+  SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));  
 
   if (ruid == euid && rgid == egid) 
     {
-      suid_is_set = SL_TRUE;
+      suid_is_set = S_TRUE;
       SL_IRETURN(SL_ENONE, _("sl_set_suid"));
     }  
-  SL_REQUIRE(suid_is_set     == SL_FALSE, _("suid_is_set == SL_FALSE"));  
+  SL_REQUIRE(suid_is_set     == S_FALSE, _("suid_is_set == S_FALSE"));  
 
 #if defined(HAVE_SETRESUID)
   retval = setresuid (sh_uid_neg, euid, sh_uid_neg);
@@ -1339,7 +1339,7 @@ int sl_set_suid ()
   suid_warn_flag = 1;
 
   SL_REQUIRE(retval == 0, _("retval == 0"));
-  suid_is_set = SL_TRUE;
+  suid_is_set = S_TRUE;
   SL_IRETURN(SL_ENONE, _("sl_set_suid"));
 }
 
@@ -1353,19 +1353,19 @@ int sl_unset_suid ()
 
   SL_ENTER(_("sl_unset_suid"));
 
-  if (uids_are_stored == SL_FALSE)
+  if (uids_are_stored == S_FALSE)
     {
       SL_IRETURN(SL_ENONE, _("sl_unset_suid"));
     }
 
-  SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));
+  SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));
 
   if (ruid == euid && rgid == egid)
     {
-      suid_is_set = SL_FALSE;
+      suid_is_set = S_FALSE;
       SL_IRETURN(SL_ENONE, _("sl_unset_suid"));
     }  
-  SL_REQUIRE(suid_is_set     == SL_TRUE, _("suid_is_set == SL_TRUE"));  
+  SL_REQUIRE(suid_is_set     == S_TRUE, _("suid_is_set == S_TRUE"));  
 
 #if defined(HAVE_SETRESUID)
   retval = setresgid (sh_gid_neg, rgid, sh_gid_neg);
@@ -1393,7 +1393,7 @@ int sl_unset_suid ()
   suid_warn_flag = 0;
 
   SL_REQUIRE(retval == 0, _("retval == 0"));
-  suid_is_set = SL_FALSE;
+  suid_is_set = S_FALSE;
   SL_IRETURN(SL_ENONE, _("sl_unset_suid"));
 }
 
@@ -1404,7 +1404,7 @@ int sl_unset_suid ()
 int sl_save_uids()
 {
   SL_ENTER(_("sl_save_uids"));
-  if (uids_are_stored == SL_TRUE) 
+  if (uids_are_stored == S_TRUE) 
     SL_IRETURN(SL_EREPEAT, _("sl_save_uids"));
 
   ruid_orig = getuid();
@@ -1413,7 +1413,7 @@ int sl_save_uids()
   euid = geteuid();
   ruid = ruid_orig;
   rgid = rgid_orig;
-  uids_are_stored = SL_TRUE;
+  uids_are_stored = S_TRUE;
 
   SL_IRETURN(SL_ENONE, _("sl_save_uids"));
 }
@@ -1426,7 +1426,7 @@ extern int  sh_unix_initgroups2 (uid_t in_pid, gid_t in_gid);
 int sl_drop_privileges()
 {
   SL_ENTER(_("sl_drop_privileges"));
-  SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));
+  SL_REQUIRE(uids_are_stored == S_TRUE, _("uids_are_stored == S_TRUE"));
 
   SL_REQUIRE(setgid(rgid_orig) == 0, _("setgid(rgid_orig) == 0"));
   SL_REQUIRE(sh_unix_initgroups2(ruid_orig, rgid_orig) == 0, _("sh_unix_initgroups2(ruid_orig,rgid_orig) == 0"));
@@ -1451,7 +1451,7 @@ int sl_drop_privileges()
 int sl_policy_get_root()
 {
   SL_ENTER(_("sl_policy_get_root"));
-  SL_REQUIRE(uids_are_stored == SL_FALSE, _("uids_are_stored == SL_FALSE"));
+  SL_REQUIRE(uids_are_stored == S_FALSE, _("uids_are_stored == S_FALSE"));
 
   SL_REQUIRE (sl_save_uids() == SL_ENONE, _("sl_save_uids() == SL_ENONE"));
 
@@ -1464,7 +1464,7 @@ int sl_policy_get_root()
       ruid = euid;
       rgid = egid;
     }
-  suid_is_set = SL_TRUE;
+  suid_is_set = S_TRUE;
   if (euid == 0)
     {
       SL_REQUIRE(sh_unix_initgroups2(euid, egid) == 0, _("sh_unix_initgroups2(euid,egid) == 0"));
@@ -1483,7 +1483,7 @@ int sl_policy_get_root()
 int sl_policy_get_real(char * user)
 {
   SL_ENTER(_("sl_policy_get_real"));
-  SL_REQUIRE(uids_are_stored == SL_FALSE, _("uids_are_stored == SL_FALSE"));
+  SL_REQUIRE(uids_are_stored == S_FALSE, _("uids_are_stored == S_FALSE"));
   SL_REQUIRE (sl_save_uids() == SL_ENONE, _("sl_save_uids() == SL_ENONE"));
 
   if (euid == 0 || ruid == 0)
@@ -1492,7 +1492,7 @@ int sl_policy_get_real(char * user)
       struct passwd    pwd;
       char          *  buffer;
       struct passwd *  tempres;
-      buffer = malloc(SH_PWBUF_SIZE);
+      buffer = calloc(1,SH_PWBUF_SIZE);
       SL_REQUIRE (buffer != NULL, _("buffer != NULL"));
       sh_getpwnam_r(user, &pwd, buffer, SH_PWBUF_SIZE, &tempres);
 #else
@@ -1516,7 +1516,7 @@ int sl_policy_get_real(char * user)
   SL_REQUIRE (sl_drop_privileges() == SL_ENONE,
 	      _("sl_drop_privileges() == SL_ENONE"));
 
-  suid_is_set = SL_TRUE;
+  suid_is_set = S_TRUE;
   SL_IRETURN(SL_ENONE, _("sl_policy_get_real"));
 }
 
@@ -1531,7 +1531,7 @@ int sl_policy_get_user(const char * user)
   SL_ENTER(_("sl_policy_get_user"));
 
   SL_REQUIRE(user != NULL, _("user != NULL"));
-  SL_REQUIRE(uids_are_stored == SL_FALSE, _("uids_are_stored == SL_FALSE"));
+  SL_REQUIRE(uids_are_stored == S_FALSE, _("uids_are_stored == S_FALSE"));
   SL_REQUIRE (sl_save_uids() == SL_ENONE, _("sl_save_uids() == SL_ENONE"));
 
 #ifndef SH_ALLOW_SUID
@@ -1541,7 +1541,7 @@ int sl_policy_get_user(const char * user)
       struct passwd    pwd;
       char          *  buffer;
       struct passwd *  tempres;
-      buffer = malloc(SH_PWBUF_SIZE);
+      buffer = calloc(1,SH_PWBUF_SIZE);
       SL_REQUIRE (buffer != NULL, _("buffer != NULL"));
       sh_getpwnam_r(user, &pwd, buffer, SH_PWBUF_SIZE, &tempres);
 #else
@@ -1737,6 +1737,7 @@ SL_TICKET sl_make_ticket (const char * ofile, int oline,
   SL_ENTER(_("sl_make_ticket"));
   /* Make entry.
    */
+  /* cppcheck-suppress arrayIndexOutOfBoundsCond */
   if (fd >= MAXFD || fd < 0)
      {
 	SL_IRETURN(SL_TOOMANY, _("sl_make_ticket"));
@@ -1755,14 +1756,14 @@ SL_TICKET sl_make_ticket (const char * ofile, int oline,
       ofiles[fd] = NULL;
     }
 
-  if ( (ofiles[fd] = (SL_OFILE *) malloc(sizeof(SL_OFILE))) == NULL)
+  if ( (ofiles[fd] = calloc(1,sizeof(SL_OFILE))) == NULL)
     {
       SL_IRETURN(SL_EMEM, _("sl_make_ticket"));
     }
 
   len = sl_strlen(filename)+1;
 
-  if ( (ofiles[fd]->path = (char *) malloc(len) ) == NULL)
+  if ( (ofiles[fd]->path = calloc(1,len) ) == NULL)
     {
       free (ofiles[fd]);
       ofiles[fd] = NULL;
@@ -1786,7 +1787,7 @@ SL_TICKET sl_make_ticket (const char * ofile, int oline,
   ofiles[fd]->fd      = fd;
   ofiles[fd]->content = NULL;
   ofiles[fd]->stream  = stream;
-  ofiles[fd]->flush   = SL_FALSE;
+  ofiles[fd]->flush   = S_FALSE;
 
   sl_strlcpy(ofiles[fd]->ofile, ofile, SL_OFILE_SIZE);
   ofiles[fd]->oline = oline;
@@ -2009,6 +2010,7 @@ int sl_open_file (const char * ofile, int oline,
 
   /* Make entry.
    */
+  /* cppcheck-suppress arrayIndexOutOfBoundsCond */
   if (fd >= MAXFD)
      {
 	sl_close_fd(FIL__, __LINE__, fd);
@@ -2031,7 +2033,7 @@ int sl_open_file (const char * ofile, int oline,
       ofiles[fd] = NULL;
     }
 
-  if ( (ofiles[fd] = (SL_OFILE *) malloc(sizeof(SL_OFILE))) == NULL)
+  if ( (ofiles[fd] = calloc(1,sizeof(SL_OFILE))) == NULL)
     {
       sl_close_fd(FIL__, __LINE__, fd);
       SL_IRETURN(SL_EMEM, _("sl_open_file"));
@@ -2039,7 +2041,7 @@ int sl_open_file (const char * ofile, int oline,
 
   len = sl_strlen(filename)+1;
 
-  if ( (ofiles[fd]->path = (char *) malloc(len) ) == NULL)
+  if ( (ofiles[fd]->path = calloc(1,len) ) == NULL)
     {
       free (ofiles[fd]);
       ofiles[fd] = NULL;
@@ -2065,7 +2067,7 @@ int sl_open_file (const char * ofile, int oline,
   ofiles[fd]->fd      = fd;
   ofiles[fd]->content = NULL;
   ofiles[fd]->stream  = NULL;
-  ofiles[fd]->flush   = SL_FALSE;
+  ofiles[fd]->flush   = S_FALSE;
 
   sl_strlcpy(ofiles[fd]->ofile, ofile, SL_OFILE_SIZE);
   ofiles[fd]->oline = oline;
@@ -2189,7 +2191,7 @@ static int sl_check_mincore(int fd)
 }
 #endif
 
-static int sl_drop_cache = SL_FALSE;
+static int sl_drop_cache = S_FALSE;
 
 int sl_set_drop_cache(const char * str)
 {
@@ -2210,13 +2212,13 @@ SL_TICKET sl_open_fastread (const char * ofile, int oline,
 
 #if defined(HAVE_POSIX_FADVISE) && defined(HAVE_MINCORE) && defined(POSIX_FADV_DONTNEED)
 
-  if (SL_FALSE != sl_drop_cache && !SL_ISERROR(status))
+  if (S_FALSE != sl_drop_cache && !SL_ISERROR(status))
     {
       int fd = get_the_fd(status);
       if (fd >= 0)
 	{
 	  if (0 == sl_check_mincore(fd))
-	    ofiles[fd]->flush = SL_TRUE;
+	    ofiles[fd]->flush = S_TRUE;
 	}
     }
 
@@ -2352,7 +2354,7 @@ int sl_close (SL_TICKET ticket)
   if (ofiles[fd] != NULL)
     {
 #if defined(HAVE_POSIX_FADVISE) && defined(HAVE_MINCORE) && defined(POSIX_FADV_DONTNEED)
-      if (ofiles[fd]->flush == SL_TRUE)
+      if (ofiles[fd]->flush == S_TRUE)
 	{
 	  posix_fadvise(fd, 0, 0, POSIX_FADV_DONTNEED);
 	}
@@ -2576,7 +2578,7 @@ int sl_read_timeout_fd (int fd, void * buf_in, size_t count,
   time_t tdiff;
   extern volatile int sig_termfast;
  
-  if (is_nonblocking == SL_FALSE)
+  if (is_nonblocking == S_FALSE)
     {
       /* set to non-blocking mode 
        */
@@ -2627,7 +2629,7 @@ int sl_read_timeout_fd (int fd, void * buf_in, size_t count,
 	      else
 		{
 		  error = errno;
-		  if (is_nonblocking == SL_FALSE)
+		  if (is_nonblocking == S_FALSE)
 		      retry_fcntl(FIL__, __LINE__, fd, F_SETFL, sflags);
 		  TPT(( 0, FIL__, __LINE__, _("msg=<read error>")));
 		  errno = error;
@@ -2644,7 +2646,7 @@ int sl_read_timeout_fd (int fd, void * buf_in, size_t count,
 	}
       else if (retval == 0)
 	{
-	  if (is_nonblocking == SL_FALSE)
+	  if (is_nonblocking == S_FALSE)
 	      retry_fcntl(FIL__, __LINE__, fd, F_SETFL, sflags);
 	  TPT(( 0, FIL__, __LINE__, _("msg=<timeout>")));
 	  errno = 0;
@@ -2655,7 +2657,7 @@ int sl_read_timeout_fd (int fd, void * buf_in, size_t count,
       else
 	{
 	  error = errno;
-	  if (is_nonblocking == SL_FALSE)
+	  if (is_nonblocking == S_FALSE)
 	      retry_fcntl(FIL__, __LINE__, fd, F_SETFL, sflags);
 	  TPT(( 0, FIL__, __LINE__, _("msg=<timeout>")));
 	  errno = error;
@@ -2664,7 +2666,7 @@ int sl_read_timeout_fd (int fd, void * buf_in, size_t count,
 
       if (sig_termfast == 1) 
 	{
-	  if (is_nonblocking == SL_FALSE)
+	  if (is_nonblocking == S_FALSE)
 	      retry_fcntl(FIL__, __LINE__, fd, F_SETFL, sflags);
 	  TPT(( 0, FIL__, __LINE__, _("msg=<terminated>")));
 	  errno = 0;
@@ -2676,7 +2678,7 @@ int sl_read_timeout_fd (int fd, void * buf_in, size_t count,
 
       if (tdiff > timeout)
 	{
-	  if (is_nonblocking == SL_FALSE)
+	  if (is_nonblocking == S_FALSE)
 	      retry_fcntl(FIL__, __LINE__, fd, F_SETFL, sflags);
 	  TPT(( 0, FIL__, __LINE__, _("msg=<timeout>")));
 	  errno = 0;
@@ -2686,7 +2688,7 @@ int sl_read_timeout_fd (int fd, void * buf_in, size_t count,
 	}
     }
 
-  if (is_nonblocking == SL_FALSE)
+  if (is_nonblocking == S_FALSE)
     retry_fcntl(FIL__, __LINE__, fd, F_SETFL, sflags);
   return ((int) bytes);
 }
@@ -2895,8 +2897,8 @@ int sl_write_line_fast (SL_TICKET ticket, void * msg, long nbytes)
  * ---------------------------------------------------------------- */
 
 extern uid_t rootonly[];
-extern int   EUIDSLOT;
-extern int   ORIG_EUIDSLOT;
+extern unsigned int   EUIDSLOT;
+extern unsigned int   ORIG_EUIDSLOT;
 
 extern char  tf_path[MAXFILENAME];	/* Error path for trust function. */
 extern uid_t tf_euid;	                /* Space for EUID of process.     */
@@ -3005,7 +3007,7 @@ static int trust_count = 0;
 
 int  sl_trust_purge_user (void)
 {
-  int i;
+  unsigned int i;
 
   EUIDSLOT = ORIG_EUIDSLOT;
   trust_count = 0;
@@ -3081,8 +3083,8 @@ static void sl_clear_trusted_file(struct sl_trustfile_store * file)
 int sl_trustfile_euid(const char * filename, uid_t teuid)
 {
   long          status;
-  static time_t old = 0;
-  static time_t now;
+  static size_t old = 0;
+  static size_t now;
 
   SL_ENTER(_("sl_trustfile_euid"));
 
@@ -3091,6 +3093,7 @@ int sl_trustfile_euid(const char * filename, uid_t teuid)
     SL_IRETURN(SL_EBADNAME, _("sl_trustfile_euid"));
 
   now = time(NULL);
+
   if (now < (old + 300))
     {
       if (NULL != sl_check_trusted_file(filename, teuid))
@@ -3126,23 +3129,23 @@ int sl_trustfile_euid(const char * filename, uid_t teuid)
 int sl_ok_muli (int a, int b) /* a*b */
 {
   if ((b == 0) || (a >= (INT_MIN / b) && a <= (INT_MAX / b)))
-    return SL_TRUE; /* no overflow */
-  return SL_FALSE;
+    return S_TRUE; /* no overflow */
+  return S_FALSE;
 }
 
 int sl_ok_muls (size_t a, size_t b) /* a*b */
 {
   if ((b == 0) || (a <= (SIZE_MAX / b)))
-    return SL_TRUE; /* no overflow */
-  return SL_FALSE;
+    return S_TRUE; /* no overflow */
+  return S_FALSE;
 }
 
 int sl_ok_divi (int a, int b) /* a/b */
 {
   (void) a;
   if (b != 0)
-    return SL_TRUE; /* no overflow */
-  return SL_FALSE;
+    return S_TRUE; /* no overflow */
+  return S_FALSE;
 }
 
 int sl_ok_addi (int a, int b) /* a+b */
@@ -3150,26 +3153,26 @@ int sl_ok_addi (int a, int b) /* a+b */
   if (a >= 0 && b >= 0)
     {
       if (a <= (INT_MAX - b))
-	return SL_TRUE; /* no overflow */
+	return S_TRUE; /* no overflow */
       else
-	return SL_FALSE;
+	return S_FALSE;
     }
   else if (a < 0 && b < 0)
     {
       if (a >= (INT_MIN - b))
-	return SL_TRUE; /* no overflow */
+	return S_TRUE; /* no overflow */
       else
-	return SL_FALSE;
+	return S_FALSE;
     }
-  return SL_TRUE;
+  return S_TRUE;
 }
 
 int sl_ok_adds (size_t a, size_t b) /* a+b */
 {
   if (a <= (SIZE_MAX - b))
-    return SL_TRUE; /* no overflow */
+    return S_TRUE; /* no overflow */
   else
-    return SL_FALSE;
+    return S_FALSE;
 }
 
 int sl_ok_subi (int a, int b) /* a-b */
@@ -3177,16 +3180,16 @@ int sl_ok_subi (int a, int b) /* a-b */
   if (a >= 0 && b < 0)
     {
       if (a <= (INT_MAX + b))
-	return SL_TRUE; /* no overflow */
+	return S_TRUE; /* no overflow */
       else
-	return SL_FALSE;
+	return S_FALSE;
     }
   else if (a < 0 && b >= 0)
     {
       if (a >= (INT_MIN + b))
-	return SL_TRUE; /* no overflow */
+	return S_TRUE; /* no overflow */
       else
-	return SL_FALSE;
+	return S_FALSE;
     }
-  return SL_TRUE;
+  return S_TRUE;
 }
