@@ -177,7 +177,7 @@ static int readphdrtable_32(int fd, Elf32_Ehdr const *ehdr, Elf32_Phdr **phdrs)
 	return err("ELF file has no program header table.");
 
     size = ehdr->e_phnum * sizeof **phdrs;
-    if (!(*phdrs = malloc(size)))
+    if (!(*phdrs = calloc(1,size)))
 	return err("Out of memory!");
 
     errno = 0;
@@ -195,7 +195,7 @@ static int readphdrtable_64(int fd, Elf64_Ehdr const *ehdr, Elf64_Phdr **phdrs)
 	return err("ELF file has no program header table.");
 
     size = ehdr->e_phnum * sizeof **phdrs;
-    if (!(*phdrs = malloc(size)))
+    if (!(*phdrs = calloc(1,size)))
 	return err("Out of memory!");
 
     errno = 0;
