@@ -137,7 +137,7 @@ fi
 
 usage() {
     echo "test.sh [options] <test_number> [hostname]"
-    echo "        [-q|--quiet|-v|--verbose] [-s|--stoponerr] [--no-cleanup]"
+    echo "        [-q|--quiet|-v|--verbose] [-s|--stoponerr] [-n|--no-cleanup]"
     echo "        [--srcdir=top_srcdir] [--color=always|never|auto]"
     echo
     echo "  ${S}test.sh  1${E}  -- Compile with many different options"
@@ -148,12 +148,20 @@ usage() {
     echo "  ${S}test.sh  6${E}  -- Controlling the daemon"
     echo "  ${S}test.sh  7${E}  -- GnuPG signed files / prelude log"
     echo "  ${S}test.sh  8${E}  -- Suidcheck"
+    echo "  ${S}test.sh  9${E}  -- Process check"
+    echo "  ${S}test.sh 10${E}  -- Port check"
+    echo "  ${S}test.sh 11${E}  -- CL verify option" 
+    echo "  ${S}test.sh 12${E}  -- CL create DeltaDB" 
+    echo "  ${S}test.sh 13${E}  -- CL create/verify partial DB" 
 
-    echo "  ${S}test.sh 10${E}  -- Test c/s init/check      (testrc_2.in)"
-    echo "  ${S}test.sh 11${E}  -- Test full c/s init/check (testrc_2.in)"
-    echo "  ${S}test.sh 12${E}  -- Test full c/s w/gpg      (testrc_2.in)"
-    echo "  ${S}test.sh 13${E}  -- Test full c/s w/mysql    (testrc_2.in)"
-    echo "  ${S}test.sh 14${E}  -- Test full c/s w/postgres (testrc_2.in)"
+    echo "  ${S}test.sh 20${E}  -- Test c/s init/check      (testrc_2.in)"
+    echo "  ${S}test.sh 21${E}  -- Test full c/s init/check (testrc_2.in)"
+    echo "  ${S}test.sh 22${E}  -- Test full c/s w/gpg      (testrc_2.in)"
+    echo "  ${S}test.sh 23${E}  -- Test full c/s w/mysql    (testrc_2.in)"
+    echo "  ${S}test.sh 24${E}  -- Test full c/s w/postgres (testrc_2.in)"
+    echo "  ${S}test.sh 25${E}  -- Test server w/yulectl    (testrc_2.in)"
+    echo "  ${S}test.sh 26${E}  -- Test c/s case one        (testrc_2.in)"
+    echo "  ${S}test.sh 27${E}  -- Test c/s case two        (testrc_2.in)"
     echo "  ${S}test.sh all${E} -- All tests"
 }
 scripts () {
@@ -161,8 +169,10 @@ scripts () {
     echo "Scripts used by tests:"
     echo "  (1) testcompile.sh (2) testhash.sh     (3) testrun_1.sh   (4) testrun_1a.sh"
     echo "  (5) testext.sh     (6) testtimesrv.sh  (7) testrun_1b.sh  (8) testrun_1c.sh" 
-    echo " (10) testrun_2.sh  (11) testrun_2a.sh  (12) testrun_2b.sh (13) testrun_2c.sh"
-    echo " (14) testrun_2d.sh"
+    echo "  (9) testrun_1d.sh (10) testrun_1e.sh  (11) testrun_1f.sh (12) testrun_1g.sh" 
+    echo " (13) testrun_1h.sh"
+    echo " (20) testrun_2.sh  (21) testrun_2a.sh  (22) testrun_2b.sh (23) testrun_2c.sh"
+    echo " (24) testrun_2d.sh (25) testrun_2e.sh  (26) testrun_2f.sh (27) testrun_2g.sh"
 }
 
 #
@@ -184,7 +194,7 @@ do
         -v|--verbose)  verbose=on; quiet= ;;
         -q|--quiet)    quiet=on; verbose= ;;
         -s|--stoponerr)     stoponerr=on;;
-	--no-cleanup) cleanup= ;;
+	-n|--no-cleanup) cleanup= ;;
 	--really-all) doall=on;;
 	--valgrind) usevalgrind=on;;
 	--srcdir=*)    TOP_SRCDIR=`echo $1 | sed s,--srcdir=,,`; export TOP_SRCDIR;;
@@ -281,10 +291,10 @@ chmod go-w .
 #
 #
 #
-if test x$UID != x; then
+if test x$UID != x -a x$UID != x0; then
   TRUST="--with-trusted=0,2,$UID"
 else
-  TRUST="--with-trusted=0,2,500"
+  TRUST="--with-trusted=0,2,1000"
 fi
 export TRUST
 #
@@ -292,7 +302,7 @@ export TRUST
 #
 MAKE=`which gmake`
 if test "x$?" = x1 ; then
-    MAKE="make -s"
+    MAKE="make -s -j 3"
 else
     MAKE=`which gmake | sed -e "s%\([a-z:]\) .*%\1%g"` 
     if test "x$MAKE" = x; then
@@ -359,22 +369,26 @@ log_msg ()
 	# also, need to account for terminal control sequences
 	len=`echo "$msg" | awk '/1;30m/ { print length()-10; }; !/1;30m/ { print length();}'`
 	let "cols = cols - len" >/dev/null
-	moveto='['$cols'C'
-	echo "${msg}${moveto}${ccode}[${3}]${E}"
+	if [ $cols -ge 0 ]; then
+	    moveto='['$cols'C'
+	    echo "${msg}${moveto}${ccode}[${3}]${E}"
+	else
+	    echo "${msg}${ccode}[${3}]${E}"
+	fi
     fi
 }
 
 log_fail () { 
-    log_msg "$1" "$2" failure "$3"; 
+    [ -z "$quiet" ] && log_msg "$1" "$2" failure "$3"; 
     let "failcount = failcount + 1" >/dev/null; 
     test -z "$stoponerr" || exit 1; 
 }
 log_ok ()   { 
-    log_msg "$1" "$2" success "$3"; 
+    [ -z "$quiet" ] && log_msg "$1" "$2" success "$3"; 
     let "okcount = okcount + 1" >/dev/null; 
 }
 log_skip () { 
-    log_msg "$1" "$2" skipped "$3"; 
+    [ -z "$quiet" ] && log_msg "$1" "$2" skipped "$3"; 
     let "skipcount = skipcount + 1" >/dev/null; 
 }
 
@@ -424,10 +438,12 @@ do_cleanup () {
     rm -f testrc_1.dyn
     rm -f testrc_2
     rm -f testrc_22
+    rm -f testrc_1ext
     rm -f ./.samhain_file
+    rm -f file.*.*-*-*-*-*
     rm -f ./.samhain_log*
     rm -f ./.samhain_lock*
-    test -d testrun_testdata && chmod -R 0700 testrun_testdata
+    test -d testrun_testdata && chmod -f -R 0700 testrun_testdata
     test -d .quarantine && rm -rf .quarantine
     rm -rf testrun_testdata
     rm -f test_log_db
@@ -436,11 +452,17 @@ do_cleanup () {
     rm -f test_log_yulectl
     rm -f yule.html
     rm -f yule.html2
+    rm -f test_dnmalloc
+    rm -f tmp_list_file
+    rm -f test_filter.txt
 }
 
 print_summary ()
 {
-    let "gcount = okcount + skipcount + failcount" >/dev/null;
+    # let "gcount = okcount + skipcount + failcount" >/dev/null;
+    gcount=$MAXTEST;
+    let "failcount = gcount - okcount - skipcount" >/dev/null;
+
     [ -z "$quiet" ] && { 
 	echo
 	echo "__ ${S}Tests: ${gcount}  Ok: ${okcount} Skipped: ${skipcount} Failed: ${failcount}${E}"
@@ -470,8 +492,14 @@ find_path () { (
 ); }
 
 find_hostname () {
-    tmp=`hostname -f 2>/dev/null`
-    if [ $? -ne 0 ]; then
+
+    uname -a | grep Linux >/dev/null
+    if [ $? -eq 0 ]; then
+	tmp=`hostname -f 2>/dev/null`
+	if [ $? -ne 0 ]; then
+	    tmp=`hostname 2>/dev/null`
+	fi
+    else
 	tmp=`hostname 2>/dev/null`
     fi
     if [ -z "$tmp" ]; then
@@ -593,39 +621,92 @@ if test x$1 = x8; then
     print_summary
     exit $?
 fi
+if test x$1 = x9; then
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1d.sh
+    testrun1d
+    print_summary
+    exit $?
+fi
 if test x$1 = x10; then
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1e.sh
+    testrun1e
+    print_summary
+    exit $?
+fi
+if test x$1 = x11; then
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1f.sh
+    testrun1f
+    print_summary
+    exit $?
+fi
+if test x$1 = x12; then
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1g.sh
+    testrun1g
+    print_summary
+    exit $?
+fi
+if test x$1 = x13; then
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1h.sh
+    testrun1h
+    print_summary
+    exit $?
+fi
+if test x$1 = x20; then
     . ${SCRIPTDIR}/testrun_2.sh 
     testrun2 $hostname
     print_summary
     exit $?
 fi
-if test x$1 = x11; then
+if test x$1 = x21; then
     . ${SCRIPTDIR}/testrun_2a.sh
     testrun2a $hostname
     print_summary
     exit $?
 fi
-if test x$1 = x12; then
+if test x$1 = x22; then
     . ${SCRIPTDIR}/testrun_2a.sh
     . ${SCRIPTDIR}/testrun_2b.sh
     testrun2b $hostname
     print_summary
     exit $?
 fi
-if test x$1 = x13; then
+if test x$1 = x23; then
     . ${SCRIPTDIR}/testrun_2a.sh
     . ${SCRIPTDIR}/testrun_2c.sh 
     testrun2c $hostname
     print_summary
     exit $?
 fi
-if test x$1 = x14; then
+if test x$1 = x24; then
     . ${SCRIPTDIR}/testrun_2a.sh
     . ${SCRIPTDIR}/testrun_2d.sh
     testrun2d $hostname
     print_summary
     exit $?
 fi
+if test x$1 = x25; then
+    . ${SCRIPTDIR}/testrun_2e.sh
+    testrun2e $hostname
+    print_summary
+    exit $?
+fi
+if test x$1 = x26; then
+    . ${SCRIPTDIR}/testrun_2f.sh
+    testrun2f $hostname
+    print_summary
+    exit $?
+fi
+if test x$1 = x27; then
+    . ${SCRIPTDIR}/testrun_2g.sh
+    testrun2g $hostname
+    print_summary
+    exit $?
+fi
 if test x$1 = xall; then
     TEST_MAX=0
     . ${SCRIPTDIR}/testcompile.sh
@@ -644,6 +725,16 @@ if test x$1 = xall; then
     let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
     . ${SCRIPTDIR}/testrun_1c.sh
     let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
+    . ${SCRIPTDIR}/testrun_1d.sh
+    let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
+    . ${SCRIPTDIR}/testrun_1e.sh
+    let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
+    . ${SCRIPTDIR}/testrun_1f.sh
+    let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
+    . ${SCRIPTDIR}/testrun_1g.sh
+    let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
+    . ${SCRIPTDIR}/testrun_1h.sh
+    let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
     . ${SCRIPTDIR}/testrun_2.sh
     let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
     . ${SCRIPTDIR}/testrun_2a.sh
@@ -654,6 +745,12 @@ if test x$1 = xall; then
     let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
     . ${SCRIPTDIR}/testrun_2d.sh
     let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
+    . ${SCRIPTDIR}/testrun_2e.sh
+    let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
+    . ${SCRIPTDIR}/testrun_2f.sh
+    let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
+    . ${SCRIPTDIR}/testrun_2g.sh
+    let "TEST_MAX = TEST_MAX + MAXTEST" >/dev/null
     #
     # ${SCRIPTDIR}/testtimesrv.sh
     # ${SCRIPTDIR}/testrun_1b.sh
@@ -687,6 +784,31 @@ if test x$1 = xall; then
     MAXTEST=${TEST_MAX}; export MAXTEST
     testrun1c
     #
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1d.sh
+    MAXTEST=${TEST_MAX}; export MAXTEST
+    testrun1d
+    #
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1e.sh
+    MAXTEST=${TEST_MAX}; export MAXTEST
+    testrun1e
+    #
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1f.sh
+    MAXTEST=${TEST_MAX}; export MAXTEST
+    testrun1f
+    #
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1g.sh
+    MAXTEST=${TEST_MAX}; export MAXTEST
+    testrun1g
+    #
+    . ${SCRIPTDIR}/testrun_1.sh
+    . ${SCRIPTDIR}/testrun_1h.sh
+    MAXTEST=${TEST_MAX}; export MAXTEST
+    testrun1h
+    #
     . ${SCRIPTDIR}/testrun_2.sh
     MAXTEST=${TEST_MAX}; export MAXTEST
     testrun2 $hostname
@@ -707,6 +829,18 @@ if test x$1 = xall; then
     MAXTEST=${TEST_MAX}; export MAXTEST
     testrun2d $hostname
     #
+    . ${SCRIPTDIR}/testrun_2e.sh
+    MAXTEST=${TEST_MAX}; export MAXTEST
+    testrun2e $hostname
+    #
+    . ${SCRIPTDIR}/testrun_2f.sh
+    MAXTEST=${TEST_MAX}; export MAXTEST
+    testrun2f $hostname
+    #
+    . ${SCRIPTDIR}/testrun_2g.sh
+    MAXTEST=${TEST_MAX}; export MAXTEST
+    testrun2g $hostname
+    #
     print_summary
     exit 0
 fi
@@ -715,9 +849,17 @@ usage;
 
 exit 1;
 
+# gpg -a --clearsign --not-dash-escaped testrc.gpg
+# gpg -a --clearsign --not-dash-escaped .samhain_file
+# tar czvf foo.tgz testrc.gpg.asc .samhain_file.asc
+# cat foo.tgz >>test/test.sh
+
 __ARCHIVE_FOLLOWS__
- -tC Mo0#@Hą	4&m׎I9Me6U(M[bwv |W$ĉݺMHl?q1;ʥ
-ҩTXjyu֦iUM*hLBMiPI'J}A0?gG!ټUek6JV5oJ0m>X*Ms"\$uEe"{c0# w6aq;B"7P8q%[QHZT#[#8i "j1:¤/:(	eJ	'5p]}d32!	f1љRšLgMLTl!τi1wOiA4ج`i#1LDürCPCqLIY,D,S:JuMrpxMd	
-mp-2ZE`GdW%bUm˘{悙])de`x
-Kl\j:u=7az+k[]g|#<X{ZE:e	Y]+xĸ
-=Kjt*FӤ_W?}|11܌I\XWiO{wGy(?޼|c_/x͋i	؋sra׊hc'Ckt#9M%d3l-v.<7KH$D"H$D"H$D"H$	?:v (  
\ No newline at end of file
+ p{U =lF!s-diD&%r#Q֗نQ0҉bd+Kеk@75k塀l}!KaD:C=ՂؖRA
+8Mw;َ(#R1ܜ[2?=^0@b~^R4cX3IvdBʒUY)7]nbD11QBpIH&yQ6E/" tB&p,B,bL	KRseUȶe 6xǱ{Eٛ jENk0h!YP3ɾqG(ki;P)ʖl8:QmlJT16JjcR릓.	%d8I:7
+@s9!1ReC+OgM޾QBiIb5ۖ-\M}.TT#g+c4l`g6jgx υe[ӆbzp9ƒjg6AHFinxV+G#Ү@e4Q,;4Hi9ᢦ># ڱz
++nh& SN0ZI8h8Zb<^~$J+Q(|ݎvzYw;t|:}O㟗Ҵ^_Jto	,R"(|#r"z _ǖ<Nm%]Ͼ:,3v?KIo?c`{҇;C]eT^^qNTpP7]=<kPqj
+?/OPãӧ&|9=̨0k!D/qMe2t[fkn4v[~oW:m7zZ
+z"&墣w_Snh8uiNk;jݯtkfW7v^^vǻNNz-5OA_jV($ﮢ
+^HMb[y'hIw5o2b!f|81|o
+@ @ @ .7<̐ (  
