@@ -19,8 +19,8 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 
-# dnmalloc + flawfinder + (36 * 3)
-MAXTEST=110; export MAXTEST
+# dnmalloc + cppcheck + flawfinder + (38 * 2) - 8
+MAXTEST=79; export MAXTEST
 
 run_dnmalloc ()
 {
@@ -82,29 +82,25 @@ run_dnmalloc ()
     fi
 }
 
-run_uno ()
+run_cppcheck ()
 {
-    if [ -z "$doall" ]; then
-	[ -z "$quiet" ] && log_skip $2 ${MAXTEST} "$TEST (uno)";
-	return 0
-    fi
     #
-    UNO=`find_path uno`
+    CPC=`find_path cppcheck`
     #
-    if [ -z "$UNO" ]; then
-	log_skip $num $MAXTEST 'check w/uno (not in PATH)'
+    if [ -z "$CPC" ]; then
+	log_skip $num ${MAXTEST} 'check w/cppcheck (not in PATH)'
 	return 0
     fi
     #
     cd src/
-    stat=`uno -DHAVE_MMAP=1 -DUSE_UNO=1 -t -I. -I.. -I../include sh_*.c samhain.c slib.c dnmalloc.c zAVLTree.c trustfile.c rijndael-*.c bignum.c 2>&1 | wc -l`
-    if [ $stat -ne 1 ]; then
-	uno -DSH_WITH_CLIENT -DSH_WITH_SERVER -DHAVE_MMAP=1 -DUSE_UNO=1 -t -I. -I.. -I../include sh_*.c samhain.c slib.c dnmalloc.c zAVLTree.c trustfile.c rijndael-*.c bignum.c >>../test_log 2>&1
+    stat=`cppcheck --quiet --inline-suppr --force -j 4 --template="{file},{line},{severity},{id},{message}" -I. -I.. -I../include sh_*.c samhain.c slib.c dnmalloc.c zAVLTree.c trustfile.c rijndael-*.c bignum.c 2>&1 | wc -l`
+    if [ $stat -ne 0 ]; then
+	cppcheck --quiet --inline-suppr --force -j 4 --template="{file},{line},{severity},{id},{message}" -I. -I.. -I../include sh_*.c samhain.c slib.c dnmalloc.c zAVLTree.c trustfile.c rijndael-*.c bignum.c >>../test_log 2>&1
 	retval=1
-	[ -z "$quiet" ] && log_fail $2 ${MAXTEST} "$TEST (uno)";
+	[ -z "$quiet" ] && log_fail $2 ${MAXTEST} "check w/cppcheck";
     else
 	retval=0
-	[ -z "$quiet" ] &&     log_ok   $2 ${MAXTEST} "$TEST (uno)";
+	[ -z "$quiet" ] &&     log_ok   $2 ${MAXTEST} "check w/cppcheck";
     fi
     cd ..
     return $retval
@@ -123,81 +119,30 @@ run_flawfinder ()
     fi
 }
 
-run_smatch () 
+run_clang () 
 {
     export CDIR=`pwd`;
 
     if [ -z "$doall" ]; then
-	[ -z "$quiet" ] && log_skip $2 ${MAXTEST} "$TEST (smatch)";
+	[ -z "$quiet" ] && log_skip $2 ${MAXTEST} "$TEST (clang)";
 	return 0
     fi
 
-    if [ ! -f "./x_samhain.c.sm" ]; then
-	[ -z "$quiet" ] && log_skip $2 ${MAXTEST} "$TEST (skip sm)";
+    CLANGPATH=`find_path clang`
+    if [ -n "$CLANGPATH" ]; then
+	CLANG_CC="$CLANGPATH"; export CLANG_CC
+    else
+	[ -z "$quiet" ] && log_skip $2 ${MAXTEST} "$TEST (clang)";
 	return 0
     fi
 
-    if [ "x$3" = "xdebug" ]; then memcheck=debug; else memcheck=simple; fi
-    if [ -f ../../static/sm_scripts/smatch.pm ]; then
-	( 
-	    cd ../../static/sm_scripts;
-	    
-	    for i in ${CDIR}/*.c.sm ; do
-	        # echo $i;
-		cat $i | ./unreached_code.pl | grep -v sh_threaded_module_run;
-		cat $i | ./ampersand_missing.sh;
-		cat $i | ./eqeq.pl;
-		cat $i | ./for_bounds.pl;    # doesn't work?
-		cat $i | ./unchecked_returns.pl;
-		cat $i | ./uninitialized.pl; # doesn't work?
-
-		# from http://people.redhat.com/mstefani/wine/smatch/
-		if [ -f ./while_for_check.pl ]; then
-		    cat $i | ./while_for_check.pl; # works
-		fi 
-		# --> end wine <--
-
-		# samhain specific modifications (list of free/malloc funcs)
-		# doesn't seem to find anything useful
-		if [ $memcheck = xsimple ]; then
-		    if [ -f ./samhain_unfree.pl ]; then
-			cat $i | ./samhain_unfree.pl | \
-			    egrep -v 'x_cutest_.*Test_' | \
-			    egrep -v 'x_sh_unix.c .... .... sh_unix_copyenv';
-		    fi 
-		fi
-		if [ $memcheck = xdebug ]; then
-		    if [ -f ./samhain_unfree_debug.pl ]; then
-			cat $i | ./samhain_unfree_debug.pl | \
-			    egrep -v 'x_cutest_.*Test_' | \
-			    egrep -v 'x_sh_unix.c .... .... sh_unix_copyenv';
-		    fi 
-		fi
-		# --> end samhain specific <--
-
-		echo malloc >  list_null_funcs_uniq;
-		echo getenv >> list_null_funcs_uniq;
-		cat $i | ./deference_check.pl |\
-                     egrep -v 'x_trustfile.c ... ... sl_trustfile' | \
-		     egrep -v 'x_sh_mem.c ';
-		rm -f list_null_funcs_uniq;
-		# rm -f $i
-	    done
-	) >test_log_smatch 2>&1
-	if [ -f test_log_smatch ]; then
-	    lines=`cat test_log_smatch | wc -l`
-	    if [ $lines -ne 0 ]; then
-		cat test_log_smatch
-		rm -f test_log_smatch
-		[ -z "$quiet" ] && log_fail $2 ${MAXTEST} "$TEST (smatch)";
-		return 1
-	    fi
-	fi
-	[ -z "$quiet" ] && log_ok $2 ${MAXTEST} "$TEST (smatch)";
-	return 0
-    fi
-    [ -z "$quiet" ] && log_skip $2 ${MAXTEST} "$TEST (smatch)";
-    return 0
+    SAVE_TEST="$TEST"
+    TEST="$TEST (clang)"
+    testmake $1 $2
+    retval=$?
+    CLANG_CC=""; export CLANG_CC
+    TEST="$SAVE_TEST"
+    return $retval
 }
 
 testmake ()
@@ -211,13 +156,19 @@ testmake ()
 	uname -a | grep Linux >/dev/null
 	if [ $? -eq 0 ]; then
 	    isLinux=1
-	    sed --in-place 's/-Wall/-Wall -Werror/' Makefile
+	    sed --in-place 's/-Wall/-Wall -Werror -Wpointer-arith -Wcast-qual/' Makefile
 	fi
 	#
 	if test x$1 = x0; then
 		[ -z "$verbose" ]     ||  log_msg_ok  "configure...  $TEST";
 		$MAKE clean > /dev/null 2>> test_log
-		$MAKE ${SMATCH} cutest > /dev/null 2>> test_log
+		if [ -z "$CLANG_CC" ]; then
+		    $MAKE cutest > /dev/null 2>> test_log
+		else
+		    sed --in-place 's/-Wno-empty-body/-Wno-empty-body -Wno-invalid-source-encoding/g' Makefile
+		    sed --in-place 's/-fno-strength-reduce//g' Makefile
+		    $MAKE -e CC=$CLANG_CC -e BUILD_CC=$CLANG_CC cutest > /dev/null 2>> test_log
+		fi
 		if test x$? = x0; then
 		    [ -z "$verbose" ] || log_msg_ok   "make cutest... $TEST";
  		else
@@ -246,19 +197,6 @@ testcompile ()
 {
 	log_start "COMPILE"
 
-	if [ "x$doall" = xon ]; then
-	    uname -a | grep x86_64 >/dev/null 
-	    if [ $? -ne 0 ]; then
-		if [ -f /usr/local/gcc-smatch/bin/gcc ]; then  # FIXME
-		    SAVE_CC="${CC}"
-		    SMATCH="DBGDEF=--smatch"; export SMATCH
-		    SAVE_SMATCH="${SMATCH}"; export SAVE_SMATCH
-		    CC="/usr/local/gcc-smatch/bin/gcc"; export CC
-		    SMATCH_CC="${CC}"
-		fi
-	    fi
-	fi
-
 	num=0
 	numfail=0
 
@@ -273,9 +211,14 @@ testcompile ()
 	    if [ $? -eq 0 ]; then
 		C_LOGFILE=" --enable-logfile-monitor "
 	    else
-		ls /usr/local/lib/libpcre* >/dev/null 2>&1
+		ls /usr/lib/*/libpcre* >/dev/null 2>&1
 		if [ $? -eq 0 ]; then
 		    C_LOGFILE=" --enable-logfile-monitor "
+		else
+		    ls /usr/local/lib/libpcre* >/dev/null 2>&1
+		    if [ $? -eq 0 ]; then
+			C_LOGFILE=" --enable-logfile-monitor "
+		    fi
 		fi
 	    fi
 	fi
@@ -295,6 +238,16 @@ testcompile ()
 	run_dnmalloc 0 $num || let "numfail = numfail + 1"  >/dev/null
 	#
 
+	#
+	# test dnmalloc
+	#
+        TEST="${S}check w/cppcheck${E}"
+	#
+	let "num = num + 1" >/dev/null
+	#
+	run_cppcheck 0 $num || let "numfail = numfail + 1"  >/dev/null
+	#
+
 	#
 	# test flawfinder
 	#
@@ -325,9 +278,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1"  >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation
@@ -338,18 +289,12 @@ testcompile ()
 		$MAKE distclean 
 	fi
 	#
-	[ -z "${SMATCH}" ] || { CC="${SAVE_CC}"; export CC; SMATCH=""; export SMATCH; }
-	#
 	${TOP_SRCDIR}/configure --quiet  --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test  --enable-static --enable-suidcheck --enable-process-check ${C_LOGFILE} > /dev/null 2>> test_log
 	#
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1"  >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
-	#
-	[ -z "${SMATCH_CC}" ] || { CC="${SMATCH_CC}"; export CC; SMATCH="${SAVE_SMATCH}"; export SMATCH; }
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	#
 	# test standalone compilation
 	#
@@ -364,9 +309,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1"  >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation
@@ -377,18 +320,12 @@ testcompile ()
 		$MAKE distclean 
 	fi
 	#
-	[ -z "${SMATCH}" ] || { CC="${SAVE_CC}"; export CC; SMATCH=""; export SMATCH; }
-	#
 	${TOP_SRCDIR}/configure --quiet  --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test  --enable-process-check --enable-port-check --enable-static > /dev/null 2>> test_log
 	#
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1"  >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
-	#
-	[ -z "${SMATCH_CC}" ] || { CC="${SMATCH_CC}"; export CC; SMATCH="${SAVE_SMATCH}"; export SMATCH; }
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	#
 	# test standalone compilation
 	#
@@ -403,9 +340,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1"  >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation
@@ -421,9 +356,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 
 	#
@@ -440,9 +373,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation with --with-nocl=PW
@@ -458,9 +389,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation w/ debug
@@ -476,9 +405,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num debug || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num debug || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation w/ gpg
@@ -501,9 +428,7 @@ testcompile ()
 	    #
 	    testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	    let "num = num + 1" >/dev/null
-	    run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	    let "num = num + 1" >/dev/null
-	    run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	    run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	fi
 
 	#
@@ -520,9 +445,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation w/loginwatch
@@ -538,9 +461,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation w/mysql
@@ -556,9 +477,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num "skip" || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation w/mysql and stealth
@@ -574,9 +493,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num "skip" || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation w/postgresql
@@ -592,9 +509,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num "skip" || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation w/postgresql+stealth
@@ -610,9 +525,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num "skip" || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation
@@ -628,9 +541,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test standalone compilation
@@ -646,9 +557,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	# echo; echo "${S}__ TEST CLIENT/SERVER __${E}"; echo;
 
@@ -666,9 +575,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	
 	if test -r "Makefile"; then
 		$MAKE clean
@@ -679,9 +586,47 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
+
+	#
+	# test client/server compilation w/prelude
+	#
+	TEST="${S}client/server application w/prelude${E}"
+	#
+	if [ -z "$doall" ]; then
+	    let "num = num + 1" >/dev/null
+	    [ -z "$quiet" ] && log_skip $num ${MAXTEST} "$TEST";
+	    let "num = num + 1" >/dev/null
+	    [ -z "$quiet" ] && log_skip $num ${MAXTEST} "$TEST (smatch)";
+
+	    let "num = num + 1" >/dev/null
+	    [ -z "$quiet" ] && log_skip $num ${MAXTEST} "$TEST";
+	    let "num = num + 1" >/dev/null
+	    [ -z "$quiet" ] && log_skip $num ${MAXTEST} "$TEST (smatch)";
+	else
+	    if test -r "Makefile"; then
+		$MAKE clean
+	    fi
+	    #
+	    ${TOP_SRCDIR}/configure --quiet --enable-network=client  --enable-srp --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test   --with-prelude > /dev/null 2>> test_log   
+	    #
+	    let "num = num + 1" >/dev/null
+	    testmake $? $num || let "numfail = numfail + 1" >/dev/null
+	    let "num = num + 1" >/dev/null
+	    run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
+	    
+	    if test -r "Makefile"; then
+		$MAKE clean
+	    fi
+	    #
+	    ${TOP_SRCDIR}/configure --quiet --enable-network=server  --enable-srp --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test   --with-prelude > /dev/null 2>> test_log   
+	    #
+	    let "num = num + 1" >/dev/null
+	    testmake $? $num || let "numfail = numfail + 1" >/dev/null
+	    let "num = num + 1" >/dev/null
+	    run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
+	    #
+	fi
 
 	#
 	# test client/server compilation
@@ -692,16 +637,12 @@ testcompile ()
 		$MAKE clean
 	fi
 	#
-	[ -z "${SMATCH}" ] || { CC="${SAVE_CC}"; export CC; SMATCH=""; export SMATCH; }
-	#
 	${TOP_SRCDIR}/configure --quiet --enable-network=client  --enable-static --enable-srp --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test   --with-timeserver=127.0.0.1 ${C_LOGFILE} > /dev/null 2>> test_log   
 	#
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	
 	if test -r "Makefile"; then
 		$MAKE clean
@@ -712,11 +653,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
-	#
-	[ -z "${SMATCH_CC}" ] || { CC="${SMATCH_CC}"; export CC; SMATCH="${SAVE_SMATCH}"; export SMATCH; }
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	#
 	# test c/s compilation w/ gpg
 	#
@@ -742,9 +679,7 @@ testcompile ()
 	    #
 	    testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	    let "num = num + 1" >/dev/null
-	    run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	    let "num = num + 1" >/dev/null
-	    run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	    run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	    #
 	    if test -r "Makefile"; then
 		$MAKE clean
@@ -755,29 +690,25 @@ testcompile ()
 	    let "num = num + 1" >/dev/null
 	    testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	    let "num = num + 1" >/dev/null
-	    run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	    let "num = num + 1" >/dev/null
-	    run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	    run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	fi
 
 
 	#
 	# test client/server compilation
 	#
-	TEST="${S}client/server application w/o srp${E}"
+	TEST="${S}client/server application w/o srp, w/udp${E}"
 	#
 	if test -r "Makefile"; then
 		$MAKE clean
 	fi
 	#
-	${TOP_SRCDIR}/configure --quiet --enable-network=server --disable-srp --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test > /dev/null 2>> test_log   
+	${TOP_SRCDIR}/configure --quiet --enable-network=server --enable-udp --disable-srp --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test > /dev/null 2>> test_log   
 	#
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	#
 	if test -r "Makefile"; then
 		$MAKE clean
@@ -788,9 +719,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test client/server compilation w/ debug
@@ -806,9 +735,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num debug || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num debug || let "numfail = numfail + 1"  >/dev/null
 	#
 	if test -r "Makefile"; then
 		$MAKE clean
@@ -819,9 +746,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num debug || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num debug || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test client/server compilation w/stealth
@@ -837,9 +762,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	#
 	if test -r "Makefile"; then
 		$MAKE clean
@@ -850,40 +773,34 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test  client/server compilation w/logwatch
 	#
-	TEST="${S}client/server application w/login-watch${E}"
+	TEST="${S}client/server application w/login-watch,udp,no_ipv6${E}"
 	#
 	if test -r "Makefile"; then
 		$MAKE clean
 	fi
 	#
-	${TOP_SRCDIR}/configure --quiet --enable-network=server  --enable-srp --enable-login-watch --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test  > /dev/null 2>> test_log  
+	${TOP_SRCDIR}/configure --quiet --enable-network=server  --enable-udp --disable-ipv6 --enable-srp --enable-login-watch --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test  > /dev/null 2>> test_log  
 	#
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	#
 	if test -r "Makefile"; then
 		$MAKE clean
 	fi
 	#
-	${TOP_SRCDIR}/configure --quiet --enable-network=client  --enable-srp --enable-login-watch --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test  > /dev/null 2>> test_log  
+	${TOP_SRCDIR}/configure --quiet --enable-network=client  --disable-ipv6 --enable-srp --enable-login-watch --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=$PW_DIR/samhainrc.test  > /dev/null 2>> test_log  
 	#
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test client/server compilation
@@ -899,9 +816,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num debug || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num debug || let "numfail = numfail + 1"  >/dev/null
 	#
 	if test -r "Makefile"; then
 		$MAKE clean
@@ -912,9 +827,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num debug || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num debug || let "numfail = numfail + 1"  >/dev/null
 
 	#
 	# test client/server compilation
@@ -930,9 +843,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 	#
 	if test -r "Makefile"; then
 		$MAKE clean
@@ -943,11 +854,7 @@ testcompile ()
 	let "num = num + 1" >/dev/null
 	testmake $? $num || let "numfail = numfail + 1" >/dev/null
 	let "num = num + 1" >/dev/null
-	run_smatch $? $num || let "numfail = numfail + 1"  >/dev/null
-	let "num = num + 1" >/dev/null
-	run_uno $? $num || let "numfail = numfail + 1"  >/dev/null
-
-	[ -z "${SMATCH}" ] || { CC="${SAVE_CC}"; export CC; }
+	run_clang $? $num || let "numfail = numfail + 1"  >/dev/null
 
 	log_end "COMPILE"
 }
