@@ -41,6 +41,7 @@ TFILES="x y z"; export TFILES
 testrun1_setup=0
 
 MAXTEST=15; export MAXTEST
+MAXTEST_3=15; export MAXTEST_3
 
 test_dirs () {
     for ff in $CDIRS; do
@@ -1050,7 +1051,12 @@ run_init ()
 
 run_check ()
 {
-    ${VALGRIND} ./samhain -t check -p none -l debug 2>>test_log_valgrind
+    if [ "x$1" = "x"  ]; then
+	logsev=debug
+    else
+	logsev=$1
+    fi
+    ${VALGRIND} ./samhain -t check -p none -l $logsev 2>>test_log_valgrind
  
     if test x$? = x0; then
 
@@ -1138,16 +1144,21 @@ prep_testdata ()
 	return 1;
     }
 
+    echo "${BASE}" > ./tmp_list_file
+
     for ff in $TDIRS; do
 	mkdir "${BASE}/${ff}" || { 
 	    [ -z "$quiet" ] &&   log_msg_fail "mkdir ${BASE}/${ff}"; 
 	    return 1;
 	}
+	echo "${BASE}/${ff}" >> ./tmp_list_file
 	chmod 0755 "${BASE}/${ff}"
 	for gg in $TFILES; do
 	    echo "1 This is a test file" > "${BASE}/${ff}/${gg}"
 	    chmod 0644 "${BASE}/${ff}/${gg}"
+	    echo "${BASE}/${ff}/${gg}"  >> ./tmp_list_file
 	    ln -s "${BASE}/${ff}/${gg}" "${BASE}/${ff}/l_${gg}"
+	    echo "${BASE}/${ff}/l_${gg}"  >> ./tmp_list_file
 	done
 	echo "2 This is a test file" >> "${BASE}/${ff}/y"
 	echo "2 This is a test file" >> "${BASE}/${ff}/z"
@@ -1201,7 +1212,7 @@ testrun_internal ()
 	tcount=1
 	POLICY=`eval echo '"$'"TESTPOLICY_$tcount"'"'`
 
-	until [ -z "$POLICY" ]
+	until [ ${tcount} -gt ${MAXTEST_3} ]
 	do
 	  prep_init
 	  check_err $? ${tcount}; errval=$?
