@@ -22,9 +22,9 @@
 LOGFILE="$PW_DIR/.samhain_log"; export LOGFILE
 RCFILE="$PW_DIR/testrc_2";  export RCFILE
 
-SERVER_BUILDOPTS="--quiet  $TRUST --enable-network=server --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=REQ_FROM_SERVER$PW_DIR/testrc_2 --with-data-file=REQ_FROM_SERVER$PW_DIR/.samhain_file --with-logserver=${SH_LOCALHOST}  --with-log-file=$PW_DIR/.samhain_log --with-pid-file=$PW_DIR/.samhain_lock"; export SERVER_BUILDOPTS
+SERVER_BUILDOPTS="--quiet  $TRUST --enable-network=server --prefix=$PW_DIR --localstatedir=$PW_DIR --with-config-file=REQ_FROM_SERVER$PW_DIR/testrc_2 --with-data-file=REQ_FROM_SERVER$PW_DIR/.samhain_file --with-logserver=${SH_LOCALHOST}  --with-log-file=$PW_DIR/.samhain_log --with-pid-file=$PW_DIR/.samhain_lock --enable-debug=gdb"; export SERVER_BUILDOPTS
 
-CLIENT_BUILDOPTS="--quiet  $TRUST --enable-network=client --enable-srp --prefix=$PW_DIR --with-tmp-dir=$PW_DIR --localstatedir=$PW_DIR --with-config-file=REQ_FROM_SERVER$RCFILE --with-data-file=REQ_FROM_SERVER$PW_DIR/.samhain_file --with-logserver=localhost  --with-log-file=$LOGFILE --with-pid-file=$PW_DIR/.samhain_lock --enable-suidcheck"; export CLIENT_BUILDOPTS
+CLIENT_BUILDOPTS="--quiet  $TRUST --enable-network=client --enable-srp --prefix=$PW_DIR --with-tmp-dir=$PW_DIR --localstatedir=$PW_DIR --with-config-file=REQ_FROM_SERVER$RCFILE --with-data-file=REQ_FROM_SERVER$PW_DIR/.samhain_file  --with-log-file=$LOGFILE --with-pid-file=$PW_DIR/.samhain_lock --enable-suidcheck --enable-debug"; export CLIENT_BUILDOPTS
 
 do_test_1_a () {
 
@@ -41,11 +41,11 @@ do_test_1_a () {
 
 	[ -z "$verbose" ] || { 
 	    echo; 
-	    echo "${S}Start Client${E}: ./samhain.new -l none -p none -e info -t check"; 
+	    echo "${S}Start Client${E}: ./samhain.new -l none -p none -e info -t check --bind-address=127.0.0.1 --server-host=localhost"; 
 	    echo; 
 	}
 
-	${VALGRIND} ./samhain.new -t check -p none -l none -e info --bind-address=127.0.0.1 >/dev/null 2>>test_log_valgrind
+	${VALGRIND} ./samhain.new -t check -p none -l none -e info --bind-address=127.0.0.1 --server-host=localhost >/dev/null 2>>test_log_valgrind
 	if test x$? = x0; then
 	    [ -z "$verbose" ] || log_msg_ok    "samhain.new -t check";
 	else
@@ -232,7 +232,7 @@ testrun2a_internal ()
 	echo $SHPW > ./testpw
 }
 
-MAXTEST=5; export MAXTEST
+MAXTEST=4; export MAXTEST
 
 testrun2a ()
 {
@@ -251,6 +251,8 @@ testrun2a ()
     else
 	[ -z "$quiet" ] && log_fail 1 ${MAXTEST} "Client download+logging";
     fi
+    ####### EXIT HERE FOR TESTING ######
+    #
     #
     SERVER_BUILDOPTS_ORIG="${SERVER_BUILDOPTS}"
     CLIENT_BUILDOPTS_ORIG="${CLIENT_BUILDOPTS}"
@@ -277,26 +279,15 @@ testrun2a ()
 	[ -z "$quiet" ] && log_fail 3 ${MAXTEST} "Encryption disabled";
     fi
     #
-    SERVER_BUILDOPTS="${SERVER_BUILDOPTS_ORIG} --enable-encrypt=1"; export SERVER_BUILDOPTS
-    CLIENT_BUILDOPTS="${CLIENT_BUILDOPTS_ORIG} --enable-encrypt=1"; export CLIENT_BUILDOPTS
+    SERVER_BUILDOPTS="${SERVER_BUILDOPTS_ORIG} --disable-ipv6"; export SERVER_BUILDOPTS
+    CLIENT_BUILDOPTS="${CLIENT_BUILDOPTS_ORIG} --disable-ipv6"; export CLIENT_BUILDOPTS
     #
     testrun2a_internal
     do_test_1_a
     if [ $? -eq 0 ]; then
-	[ -z "$quiet" ] && log_ok   4 ${MAXTEST} "Encryption (v1)";
+	[ -z "$quiet" ] && log_ok   4 ${MAXTEST} "IPv6 disabled";
     else
-	[ -z "$quiet" ] && log_fail 4 ${MAXTEST} "Encryption (v1)";
-    fi
-    #
-    SERVER_BUILDOPTS="${SERVER_BUILDOPTS_ORIG}"; export SERVER_BUILDOPTS
-    CLIENT_BUILDOPTS="${CLIENT_BUILDOPTS_ORIG} --enable-encrypt=1"; export CLIENT_BUILDOPTS
-    #
-    testrun2a_internal
-    do_test_1_a
-    if [ $? -eq 0 ]; then
-	[ -z "$quiet" ] && log_ok   5 ${MAXTEST} "Encryption backward compat";
-    else
-	[ -z "$quiet" ] && log_fail 5 ${MAXTEST} "Encryption backward compat";
+	[ -z "$quiet" ] && log_fail 4 ${MAXTEST} "IPv6 disabled";
     fi
     #
     if [ -n "$cleanup" ]; then
@@ -309,4 +300,3 @@ testrun2a ()
     #
     log_end "RUN FULL CLIENT/SERVER"
 }
-
