@@ -377,6 +377,8 @@ EOF
 	log_fail 3 ${MAXTEST} "Three sockets open";
     fi
     #
+    cp ${SCRIPTDIR}/testrc_2.in testrc_2
+    #
     GPG=`find_path gpg`
     if [ -z "$GPG" ]; then
         log_skip 4 $MAXTEST 'gpg not found in $PATH'
@@ -386,17 +388,19 @@ EOF
             log_skip 4 $MAXTEST 'public PGP key 0x0F571F6C not present'
         else
 	    testrun_threesockets "$GPG"
-
+	    #
+	    cp ${SCRIPTDIR}/testrc_2.in testrc_2
+	    #
 	    if [ -f ./yule ]; then
-		./yule -D >/dev/null 2>>test_log 
+		./yule -D --set-database-severity=warn >/dev/null 2>>test_log 
 		five_sec_sleep
 		netstat -pant 2>/dev/null | grep 49777 | grep yule >/dev/null 2>&1
 		if [ $? -ne 0 ]; then
-		    [ -z "$quiet" ] && log_fail 4 ${MAXTEST} "Three sockets open (gpg)";
+		    [ -z "$quiet" ] && log_fail 4 ${MAXTEST} "Three sockets open (gpg) - 1";
 		else
 		    NSOCK=`netstat -pand 2>/dev/null | grep STREAM | grep yule | wc -l`
 		    if [ $NSOCK -ne 2 ]; then
-			[ -z "$quiet" ] && log_fail 4 ${MAXTEST} "Three sockets open (gpg)";
+			[ -z "$quiet" ] && log_fail 4 ${MAXTEST} "Three sockets open (gpg) - 2";
 			netstat -pand 2>/dev/null | grep yule 
 		    else
 			[ -z "$quiet" ] && log_ok   4 ${MAXTEST} "Three sockets open (gpg)";
@@ -405,7 +409,7 @@ EOF
 		PID=`cat .samhain_lock`
 		kill $PID
 	    else
-		log_fail 4 ${MAXTEST} "Three sockets open (gpg)";
+		log_fail 4 ${MAXTEST} "Three sockets open (gpg) - 3";
 	    fi
 	fi
     fi
