@@ -100,6 +100,15 @@ testrun2d ()
     #
     testrun2a_internal
     #
+    for ff in ./rc.*; do
+    ORIGINAL="SetUdpActive=no"
+    REPLACEMENT="ReportCheckflags=yes"
+    ex -s rc.morrigan.localdomain <<EOF
+%s/$ORIGINAL/$REPLACEMENT/g
+wq
+EOF
+    done
+    #
     ORIGINAL="DatabaseSeverity=none"
     REPLACEMENT="DatabaseSeverity=info"
     ex -s $RCFILE <<EOF
